*> \brief \b DLAE2
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLAE2 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlae2.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlae2.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlae2.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLAE2( A, B, C, RT1, RT2 )
* 
*       .. Scalar Arguments ..
*       DOUBLE PRECISION   A, B, C, RT1, RT2
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLAE2  computes the eigenvalues of a 2-by-2 symmetric matrix
*>    [  A   B  ]
*>    [  B   C  ].
*> On return, RT1 is the eigenvalue of larger absolute value, and RT2
*> is the eigenvalue of smaller absolute value.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] A
*> \verbatim
*>          A is DOUBLE PRECISION
*>          The (1,1) element of the 2-by-2 matrix.
*> \endverbatim
*>
*> \param[in] B
*> \verbatim
*>          B is DOUBLE PRECISION
*>          The (1,2) and (2,1) elements of the 2-by-2 matrix.
*> \endverbatim
*>
*> \param[in] C
*> \verbatim
*>          C is DOUBLE PRECISION
*>          The (2,2) element of the 2-by-2 matrix.
*> \endverbatim
*>
*> \param[out] RT1
*> \verbatim
*>          RT1 is DOUBLE PRECISION
*>          The eigenvalue of larger absolute value.
*> \endverbatim
*>
*> \param[out] RT2
*> \verbatim
*>          RT2 is DOUBLE PRECISION
*>          The eigenvalue of smaller absolute value.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERauxiliary
*
*> \par Further Details:
*  =====================
*>
*> \verbatim
*>
*>  RT1 is accurate to a few ulps barring over/underflow.
*>
*>  RT2 may be inaccurate if there is massive cancellation in the
*>  determinant A*C-B*B; higher precision or correctly rounded or
*>  correctly truncated arithmetic would be needed to compute RT2
*>  accurately in all cases.
*>
*>  Overflow is possible only if RT1 is within a factor of 5 of overflow.
*>  Underflow is harmless if the input data is 0 or exceeds
*>     underflow_threshold / macheps.
*> \endverbatim
*>
*  =====================================================================
!      SUBROUTINE DLAE2( A, B, C, RT1, RT2 )
      SUBROUTINE DDLAE2( Ah,Al, Bh,Bl, Ch,Cl, RT1h,RT1l, RT2h,RT2l )
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
!      DOUBLE PRECISION   A, B, C, RT1, RT2
      double precision Ah, Al
      double precision Bh, Bl
      double precision Ch, Cl
      double precision RT1h, RT1l
      double precision RT2h, RT2l
*     ..
*
* =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ONE
!      PARAMETER          ( ONE = 1.0D0 )
!      DOUBLE PRECISION   TWO
!      PARAMETER          ( TWO = 2.0D0 )
!      DOUBLE PRECISION   ZERO
!      PARAMETER          ( ZERO = 0.0D0 )
!      DOUBLE PRECISION   HALF
!      PARAMETER          ( HALF = 0.5D0 )
      double precision ONEh, ONEl
      double precision TWOh, TWOl
      double precision ZEROh, ZEROl
      double precision HALFh, HALFl
*     ..
*     .. Local Scalars ..
!      DOUBLE PRECISION   AB, ACMN, ACMX, ADF, DF, RT, SM, TB
      double precision ABh, ABl
      double precision ACMNh, ACMNl
      double precision ACMXh, ACMXl
      double precision ADFh, ADFl
      double precision DFh, DFl
      double precision RTh, RTl
      double precision SMh, SMl
      double precision TBh, TBl
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, SQRT
*     ..
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l

      ONEh = 1.0d+0
      ONEl = 0.0d+0
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      TWOh = 2.0d0
      TWOl = 0.0d+0
      HALFh = 0.5d0
      HALFl = 0.0d+0

*     .. Executable Statements ..
*
*     Compute the eigenvalues
*
!      SM = A + C
!<<<  SM = A + C
      T_p1 = Ah + Ch
      T_p2 = T_p1 - Ah
      SMh = T_p1
      SMl = (Ah - (T_p1 - T_p2))
     +      + (Ch - T_p2)
      SMl = SMl + Al + Cl
      T_s1 = SMh + SMl
      SMl = SMl - (T_s1 - SMh)
      SMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  SM = A + C
!      DF = A - C
!<<<  DF = A - C
      T_p1 = Ah - Ch
      T_p2 = T_p1 - Ah
      DFh = T_p1
      DFl = (Ah - (T_p1 - T_p2))
     +         - (Ch + T_p2)
      DFl = DFl + Al - Cl
      T_s1 = DFh + DFl
      DFl = DFl - (T_s1 - DFh)
      DFh = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  DF = A - C
!      ADF = ABS( DF )
!<<<  ADF = abs(DF)
      if (DFh .lt. 0.0d+0) then
        ADFh = -DFh
        ADFl = -DFl
      elseif (
     +  (DFh .eq. 0.0d+0).and.
     +  (DFl .lt. 0.0d+0)) then
        ADFh = DFh
        ADFl = -DFl
      else
        ADFh = DFh
        ADFl = DFl
      endif
!>>>  ADF = abs(DF)
!      TB = B + B
!<<<  TB = B + B
      T_p1 = Bh + Bh
      T_p2 = T_p1 - Bh
      TBh = T_p1
      TBl = (Bh - (T_p1 - T_p2))
     +      + (Bh - T_p2)
      TBl = TBl + Bl + Bl
      T_s1 = TBh + TBl
      TBl = TBl - (T_s1 - TBh)
      TBh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  TB = B + B
!      AB = ABS( TB )
!<<<  AB = abs(TB)
      if (TBh .lt. 0.0d+0) then
        ABh = -TBh
        ABl = -TBl
      elseif (
     +  (TBh .eq. 0.0d+0).and.
     +  (TBl .lt. 0.0d+0)) then
        ABh = TBh
        ABl = -TBl
      else
        ABh = TBh
        ABl = TBl
      endif
!>>>  AB = abs(TB)
!<<<  tmp1 = abs(A)
      if (Ah .lt. 0.0d+0) then
        tmp1h = -Ah
        tmp1l = -Al
      elseif (
     +  (Ah .eq. 0.0d+0).and.
     +  (Al .lt. 0.0d+0)) then
        tmp1h = Ah
        tmp1l = -Al
      else
        tmp1h = Ah
        tmp1l = Al
      endif
!>>>  tmp1 = abs(A)
!<<<  tmp2 = abs(C)
      if (Ch .lt. 0.0d+0) then
        tmp2h = -Ch
        tmp2l = -Cl
      elseif (
     +  (Ch .eq. 0.0d+0).and.
     +  (Cl .lt. 0.0d+0)) then
        tmp2h = Ch
        tmp2l = -Cl
      else
        tmp2h = Ch
        tmp2l = Cl
      endif
!>>>  tmp2 = abs(C)
      IF( 
!         ABS( A ).GT.ABS( C )
!       (tmp1 .gt. tmp2)
     + ((tmp1h .gt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .gt. tmp2l)))
     &    ) THEN
!         ACMX = A
      ACMXh = Ah
      ACMXl = Al
!         ACMN = C
      ACMNh = Ch
      ACMNl = Cl
      ELSE
!         ACMX = C
      ACMXh = Ch
      ACMXl = Cl
!         ACMN = A
      ACMNh = Ah
      ACMNl = Al
      END IF
      IF( 
!         ADF.GT.AB
!       (ADF .gt. AB)
     + ((ADFh .gt. ABh) .or.
     +  ((ADFh .eq. ABh) .and.
     +   (ADFl .gt. ABl)))
     &    ) THEN
!         RT = ADF*SQRT( ONE+( AB / ADF )**2 )
!<<<  tmp1 = AB / ADF
      T_q1 = ABh / ADFh
      tmp1h = ADFh * T_q1
      T_u1 = 134217729.0 * ADFh
      T_p1 = T_u1
     +         - (T_u1 - ADFh)
      T_p2 = ADFh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp1l = ((T_p1*T_r1
     +       - tmp1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp1l = tmp1l + (ADFl*T_q1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
      T_s1 = ABh - tmp1h
      T_r1 = T_s1 - ABh
      T_s2 = (ABh 
     +         - (T_s1 - T_r1))
     +         - (tmp1h + T_r1)
      T_s2 = T_s2 - tmp1l
      T_s2 = T_s2 + ABl
      T_q2 = (T_s1 + T_s2)
     +         / ADFh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = AB / ADF
!<<<  tmp2 = tmp1 * tmp1
      tmp2h = tmp1h * tmp1h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*tmp1l
     +      + tmp1l*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * tmp1
!<<<  tmp3 = ONE + tmp2
      T_p1 = ONEh + tmp2h
      T_p2 = T_p1 - ONEh
      tmp3h = T_p1
      tmp3l = (ONEh - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + ONEl + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = ONE + tmp2
!<<<  tmp4 = sqrt(tmp3)
      if (tmp3h.eq.0.0D+0.and.tmp3l.eq.0.0D+0) then
        tmp4h = 0.0d+0
        tmp4l = 0.0d+0
      endif
      if (tmp3h.lt.0.0d+0.or.
     +   (tmp3h.eq.0.0d+0.and.tmp3l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp4h = 0.0d+0
        tmp4l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp3h)
      T_ax = tmp3h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp3h,tmp3l,T_z1,T_z2)
          T_p1 = tmp3h - T_z1
          T_p2 = T_p1 - tmp3h
          T_t2l = (tmp3h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp3l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp4h = T_t1h
          tmp4l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp4 = sqrt(tmp3)
!<<<  RT = ADF * tmp4
      RTh = ADFh * tmp4h
      T_u1 = 134217729.0 * ADFh
      T_p1 = T_u1
     +         - (T_u1 - ADFh)
      T_p2 = ADFh - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      RTl = ((T_p1*T_r1
     +        - RTh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      RTl = RTl
     +      + (ADFh*tmp4l
     +      + ADFl*tmp4h)
      T_s1 = RTh + RTl
      RTl = RTl - (T_s1 - RTh)
      RTh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  RT = ADF * tmp4
      ELSE IF( 
!              ADF.LT.AB
!       (ADF .lt. AB)
     + ((ADFh .lt. ABh) .or.
     +  ((ADFh .eq. ABh) .and.
     +   (ADFl .lt. ABl)))
     &         ) THEN
!         RT = AB*SQRT( ONE+( ADF / AB )**2 )
!<<<  tmp1 = ADF / AB
      T_q1 = ADFh / ABh
      tmp1h = ABh * T_q1
      T_u1 = 134217729.0 * ABh
      T_p1 = T_u1
     +         - (T_u1 - ABh)
      T_p2 = ABh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp1l = ((T_p1*T_r1
     +       - tmp1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp1l = tmp1l + (ABl*T_q1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
      T_s1 = ADFh - tmp1h
      T_r1 = T_s1 - ADFh
      T_s2 = (ADFh 
     +         - (T_s1 - T_r1))
     +         - (tmp1h + T_r1)
      T_s2 = T_s2 - tmp1l
      T_s2 = T_s2 + ADFl
      T_q2 = (T_s1 + T_s2)
     +         / ABh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = ADF / AB
!<<<  tmp2 = tmp1 * tmp1
      tmp2h = tmp1h * tmp1h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*tmp1l
     +      + tmp1l*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * tmp1
!<<<  tmp3 = ONE + tmp2
      T_p1 = ONEh + tmp2h
      T_p2 = T_p1 - ONEh
      tmp3h = T_p1
      tmp3l = (ONEh - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      tmp3l = tmp3l + ONEl + tmp2l
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = ONE + tmp2
!<<<  tmp4 = sqrt(tmp3)
      if (tmp3h.eq.0.0D+0.and.tmp3l.eq.0.0D+0) then
        tmp4h = 0.0d+0
        tmp4l = 0.0d+0
      endif
      if (tmp3h.lt.0.0d+0.or.
     +   (tmp3h.eq.0.0d+0.and.tmp3l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp4h = 0.0d+0
        tmp4l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp3h)
      T_ax = tmp3h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp3h,tmp3l,T_z1,T_z2)
          T_p1 = tmp3h - T_z1
          T_p2 = T_p1 - tmp3h
          T_t2l = (tmp3h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp3l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp4h = T_t1h
          tmp4l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp4 = sqrt(tmp3)
!<<<  RT = AB * tmp4
      RTh = ABh * tmp4h
      T_u1 = 134217729.0 * ABh
      T_p1 = T_u1
     +         - (T_u1 - ABh)
      T_p2 = ABh - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      RTl = ((T_p1*T_r1
     +        - RTh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      RTl = RTl
     +      + (ABh*tmp4l
     +      + ABl*tmp4h)
      T_s1 = RTh + RTl
      RTl = RTl - (T_s1 - RTh)
      RTh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  RT = AB * tmp4
      ELSE
*
*        Includes case AB=ADF=0
*
!         RT = AB*SQRT( TWO )
!<<<  tmp1 = sqrt(TWO)
      if (TWOh.eq.0.0D+0.and.TWOl.eq.0.0D+0) then
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
      endif
      if (TWOh.lt.0.0d+0.or.
     +   (TWOh.eq.0.0d+0.and.TWOl.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp1h = 0.0d+0
        tmp1l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(TWOh)
      T_ax = TWOh * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           TWOh,TWOl,T_z1,T_z2)
          T_p1 = TWOh - T_z1
          T_p2 = T_p1 - TWOh
          T_t2l = (TWOh
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + TWOl - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp1h = T_t1h
          tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = sqrt(TWO)
!<<<  RT = AB * tmp1
      RTh = ABh * tmp1h
      T_u1 = 134217729.0 * ABh
      T_p1 = T_u1
     +         - (T_u1 - ABh)
      T_p2 = ABh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      RTl = ((T_p1*T_r1
     +        - RTh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      RTl = RTl
     +      + (ABh*tmp1l
     +      + ABl*tmp1h)
      T_s1 = RTh + RTl
      RTl = RTl - (T_s1 - RTh)
      RTh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  RT = AB * tmp1
      END IF
      IF( 
!         SM.LT.ZERO
!       (SM .lt. ZERO)
     + ((SMh .lt. ZEROh) .or.
     +  ((SMh .eq. ZEROh) .and.
     +   (SMl .lt. ZEROl)))
     &    ) THEN
!         RT1 = HALF*( SM-RT )
!<<<  tmp1 = SM - RT
      T_p1 = SMh - RTh
      T_p2 = T_p1 - SMh
      tmp1h = T_p1
      tmp1l = (SMh - (T_p1 - T_p2))
     +         - (RTh + T_p2)
      tmp1l = tmp1l + SMl - RTl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = SM - RT
!<<<  RT1 = HALF * tmp1
      RT1h = HALFh * tmp1h
      T_u1 = 134217729.0 * HALFh
      T_p1 = T_u1
     +         - (T_u1 - HALFh)
      T_p2 = HALFh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      RT1l = ((T_p1*T_r1
     +        - RT1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      RT1l = RT1l
     +      + (HALFh*tmp1l
     +      + HALFl*tmp1h)
      T_s1 = RT1h + RT1l
      RT1l = RT1l - (T_s1 - RT1h)
      RT1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  RT1 = HALF * tmp1
*
*        Order of execution important.
*        To get fully accurate smaller eigenvalue,
*        next line needs to be executed in higher precision.
*
!         RT2 = ( ACMX / RT1 )*ACMN - ( B / RT1 )*B
!<<<  tmp1 = B / RT1
      T_q1 = Bh / RT1h
      tmp1h = RT1h * T_q1
      T_u1 = 134217729.0 * RT1h
      T_p1 = T_u1
     +         - (T_u1 - RT1h)
      T_p2 = RT1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp1l = ((T_p1*T_r1
     +       - tmp1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp1l = tmp1l + (RT1l*T_q1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
      T_s1 = Bh - tmp1h
      T_r1 = T_s1 - Bh
      T_s2 = (Bh 
     +         - (T_s1 - T_r1))
     +         - (tmp1h + T_r1)
      T_s2 = T_s2 - tmp1l
      T_s2 = T_s2 + Bl
      T_q2 = (T_s1 + T_s2)
     +         / RT1h
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = B / RT1
!<<<  tmp2 = tmp1 * B
      tmp2h = tmp1h * Bh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Bl
     +      + tmp1l*Bh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * B
!<<<  tmp3 = ACMX / RT1
      T_q1 = ACMXh / RT1h
      tmp3h = RT1h * T_q1
      T_u1 = 134217729.0 * RT1h
      T_p1 = T_u1
     +         - (T_u1 - RT1h)
      T_p2 = RT1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp3l = ((T_p1*T_r1
     +       - tmp3h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp3l = tmp3l + (RT1l*T_q1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
      T_s1 = ACMXh - tmp3h
      T_r1 = T_s1 - ACMXh
      T_s2 = (ACMXh 
     +         - (T_s1 - T_r1))
     +         - (tmp3h + T_r1)
      T_s2 = T_s2 - tmp3l
      T_s2 = T_s2 + ACMXl
      T_q2 = (T_s1 + T_s2)
     +         / RT1h
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = ACMX / RT1
!<<<  tmp4 = tmp3 * ACMN
      tmp4h = tmp3h * ACMNh
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * ACMNh
      T_r1 = T_u1
     +         - (T_u1 - ACMNh)
      T_r2 = ACMNh - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*ACMNl
     +      + tmp3l*ACMNh)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * ACMN
!<<<  RT2 = tmp4 - tmp2
      T_p1 = tmp4h - tmp2h
      T_p2 = T_p1 - tmp4h
      RT2h = T_p1
      RT2l = (tmp4h - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      RT2l = RT2l + tmp4l - tmp2l
      T_s1 = RT2h + RT2l
      RT2l = RT2l - (T_s1 - RT2h)
      RT2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  RT2 = tmp4 - tmp2
      ELSE IF(
!              SM.GT.ZERO
!       (SM .gt. ZERO)
     + ((SMh .gt. ZEROh) .or.
     +  ((SMh .eq. ZEROh) .and.
     +   (SMl .gt. ZEROl)))
     &          ) THEN
!         RT1 = HALF*( SM+RT )
!<<<  tmp1 = SM + RT
      T_p1 = SMh + RTh
      T_p2 = T_p1 - SMh
      tmp1h = T_p1
      tmp1l = (SMh - (T_p1 - T_p2))
     +      + (RTh - T_p2)
      tmp1l = tmp1l + SMl + RTl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = SM + RT
!<<<  RT1 = HALF * tmp1
      RT1h = HALFh * tmp1h
      T_u1 = 134217729.0 * HALFh
      T_p1 = T_u1
     +         - (T_u1 - HALFh)
      T_p2 = HALFh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      RT1l = ((T_p1*T_r1
     +        - RT1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      RT1l = RT1l
     +      + (HALFh*tmp1l
     +      + HALFl*tmp1h)
      T_s1 = RT1h + RT1l
      RT1l = RT1l - (T_s1 - RT1h)
      RT1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  RT1 = HALF * tmp1
*
*        Order of execution important.
*        To get fully accurate smaller eigenvalue,
*        next line needs to be executed in higher precision.
*
!         RT2 = ( ACMX / RT1 )*ACMN - ( B / RT1 )*B
!<<<  tmp1 = B / RT1
      T_q1 = Bh / RT1h
      tmp1h = RT1h * T_q1
      T_u1 = 134217729.0 * RT1h
      T_p1 = T_u1
     +         - (T_u1 - RT1h)
      T_p2 = RT1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp1l = ((T_p1*T_r1
     +       - tmp1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp1l = tmp1l + (RT1l*T_q1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
      T_s1 = Bh - tmp1h
      T_r1 = T_s1 - Bh
      T_s2 = (Bh 
     +         - (T_s1 - T_r1))
     +         - (tmp1h + T_r1)
      T_s2 = T_s2 - tmp1l
      T_s2 = T_s2 + Bl
      T_q2 = (T_s1 + T_s2)
     +         / RT1h
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = B / RT1
!<<<  tmp2 = tmp1 * B
      tmp2h = tmp1h * Bh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Bl
     +      + tmp1l*Bh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * B
!<<<  tmp3 = ACMX / RT1
      T_q1 = ACMXh / RT1h
      tmp3h = RT1h * T_q1
      T_u1 = 134217729.0 * RT1h
      T_p1 = T_u1
     +         - (T_u1 - RT1h)
      T_p2 = RT1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp3l = ((T_p1*T_r1
     +       - tmp3h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp3l = tmp3l + (RT1l*T_q1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
      T_s1 = ACMXh - tmp3h
      T_r1 = T_s1 - ACMXh
      T_s2 = (ACMXh 
     +         - (T_s1 - T_r1))
     +         - (tmp3h + T_r1)
      T_s2 = T_s2 - tmp3l
      T_s2 = T_s2 + ACMXl
      T_q2 = (T_s1 + T_s2)
     +         / RT1h
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = ACMX / RT1
!<<<  tmp4 = tmp3 * ACMN
      tmp4h = tmp3h * ACMNh
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * ACMNh
      T_r1 = T_u1
     +         - (T_u1 - ACMNh)
      T_r2 = ACMNh - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*ACMNl
     +      + tmp3l*ACMNh)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * ACMN
!<<<  RT2 = tmp4 - tmp2
      T_p1 = tmp4h - tmp2h
      T_p2 = T_p1 - tmp4h
      RT2h = T_p1
      RT2l = (tmp4h - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      RT2l = RT2l + tmp4l - tmp2l
      T_s1 = RT2h + RT2l
      RT2l = RT2l - (T_s1 - RT2h)
      RT2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  RT2 = tmp4 - tmp2
      ELSE
*
*        Includes case RT1 = RT2 = 0
*
!         RT1 = HALF*RT
!<<<  RT1 = HALF * RT
      RT1h = HALFh * RTh
      T_u1 = 134217729.0 * HALFh
      T_p1 = T_u1
     +         - (T_u1 - HALFh)
      T_p2 = HALFh - T_p1
      T_u1 = 134217729.0 * RTh
      T_r1 = T_u1
     +         - (T_u1 - RTh)
      T_r2 = RTh - T_r1
      RT1l = ((T_p1*T_r1
     +        - RT1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      RT1l = RT1l
     +      + (HALFh*RTl
     +      + HALFl*RTh)
      T_s1 = RT1h + RT1l
      RT1l = RT1l - (T_s1 - RT1h)
      RT1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  RT1 = HALF * RT
!         RT2 = -HALF*RT
!<<<  RT2 = -RT1
      RT2h = -RT1h
      RT2l = -RT1l
!>>>  RT2 = -RT1
      END IF
      RETURN
*
*     End of DLAE2
*
      END
