*> \brief \b DLANST
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLANST + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlanst.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlanst.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlanst.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       DOUBLE PRECISION FUNCTION DLANST( NORM, N, D, E )
* 
*       .. Scalar Arguments ..
*       CHARACTER          NORM
*       INTEGER            N
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   D( * ), E( * )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLANST  returns the value of the one norm,  or the Frobenius norm, or
*> the  infinity norm,  or the  element of  largest absolute value  of a
*> real symmetric tridiagonal matrix A.
*> \endverbatim
*>
*> \return DLANST
*> \verbatim
*>
*>    DLANST = ( max(abs(A(i,j))), NORM = 'M' or 'm'
*>             (
*>             ( norm1(A),         NORM = '1', 'O' or 'o'
*>             (
*>             ( normI(A),         NORM = 'I' or 'i'
*>             (
*>             ( normF(A),         NORM = 'F', 'f', 'E' or 'e'
*>
*> where  norm1  denotes the  one norm of a matrix (maximum column sum),
*> normI  denotes the  infinity norm  of a matrix  (maximum row sum) and
*> normF  denotes the  Frobenius norm of a matrix (square root of sum of
*> squares).  Note that  max(abs(A(i,j)))  is not a consistent matrix norm.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] NORM
*> \verbatim
*>          NORM is CHARACTER*1
*>          Specifies the value to be returned in DLANST as described
*>          above.
*> \endverbatim
*>
*> \param[in] N
*> \verbatim
*>          N is INTEGER
*>          The order of the matrix A.  N >= 0.  When N = 0, DLANST is
*>          set to zero.
*> \endverbatim
*>
*> \param[in] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (N)
*>          The diagonal elements of A.
*> \endverbatim
*>
*> \param[in] E
*> \verbatim
*>          E is DOUBLE PRECISION array, dimension (N-1)
*>          The (n-1) sub-diagonal or super-diagonal elements of A.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERauxiliary
*
*  =====================================================================
!      DOUBLE PRECISION FUNCTION DLANST( NORM, N, D, E )
      SUBROUTINE DDLANST( NORM, N, Dh,Dl, Eh, El, DLANSTh, DLANSTl )
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      CHARACTER          NORM
      INTEGER            N
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   D( * ), E( * )
      double precision Dh(*), Dl(*)
      double precision Eh(*), El(*)
      double precision DLANSTh, DLANSTl
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
!      DOUBLE PRECISION   ONE, ZERO
!      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
      double precision ONEh, ONEl
      double precision ZEROh, ZEROl
*     ..
*     .. Local Scalars ..
      INTEGER            I
!      DOUBLE PRECISION   ANORM, SCALE, SUM
      double precision ANORMh, ANORMl
      double precision SCALEh, SCALEl
      double precision SUMh, SUMl
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL           DDLASSQ
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, SQRT
*     ..
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
*     .. Executable Statements ..
*
      IF( N.LE.0 ) THEN
!         ANORM = ZERO
          ANORMh = ZEROh
          ANORMl = ZEROl
      ELSE IF( LSAME( NORM, 'M' ) ) THEN
*
*        Find max(abs(A(i,j))).
*
!         ANORM = ABS( D( N ) )
!<<<      ANORM = abs(D(N))
          if (Dh(N) .lt. 0.0d+0) then
            ANORMh = -Dh(N)
            ANORMl = -Dl(N)
          elseif (
     +      (Dh(N) .eq. 0.0d+0).and.
     +      (Dl(N) .lt. 0.0d+0)) then
            ANORMh = Dh(N)
            ANORMl = -Dl(N)
          else
            ANORMh = Dh(N)
            ANORMl = Dl(N)
          endif
!>>>      ANORM = abs(D(N))
         DO 10 I = 1, N - 1
!            ANORM = MAX( ANORM, ABS( D( I ) ) )
!<<<      tmp1 = abs(D(I))
          if (Dh(I) .lt. 0.0d+0) then
            tmp1h = -Dh(I)
            tmp1l = -Dl(I)
          elseif (
     +      (Dh(I) .eq. 0.0d+0).and.
     +      (Dl(I) .lt. 0.0d+0)) then
            tmp1h = Dh(I)
            tmp1l = -Dl(I)
          else
            tmp1h = Dh(I)
            tmp1l = Dl(I)
          endif
!>>>      tmp1 = abs(D(I))
             IF( 
!           (ANORM .ge. tmp1)
     +     ((ANORMh .gt. tmp1h) .or.
     +      ((ANORMh .eq. tmp1h) .and.
     +       (ANORMl .ge. tmp1l)))
     &          ) THEN
                ANORMh = ANORMh
                ANORMl = ANORMl
             ELSE
                ANORMh = tmp1h
                ANORMl = tmp1l
             END IF
!            ANORM = MAX( ANORM, ABS( E( I ) ) )
!<<<            tmp2 = abs(E(I))
                if (Eh(I) .lt. 0.0d+0) then
                  tmp2h = -Eh(I)
                  tmp2l = -El(I)
                elseif (
     +            (Eh(I) .eq. 0.0d+0).and.
     +            (El(I) .lt. 0.0d+0)) then
                  tmp2h = Eh(I)
                  tmp2l = -El(I)
                else
                  tmp2h = Eh(I)
                  tmp2l = El(I)
                endif
!>>>            tmp2 = abs(E(I))
             IF( 
!                 (ANORM .ge. tmp2)
     +           ((ANORMh .gt. tmp2h) .or.
     +            ((ANORMh .eq. tmp2h) .and.
     +             (ANORMl .ge. tmp2l)))
     &          ) THEN
                ANORMh = ANORMh
                ANORMl = ANORMl
             ELSE
                ANORMh = tmp2h
                ANORMl = tmp2l
             END IF
   10    CONTINUE
      ELSE IF( LSAME( NORM, 'O' ) .OR. NORM.EQ.'1' .OR.
     $         LSAME( NORM, 'I' ) ) THEN
*
*        Find norm1(A).
*
         IF( N.EQ.1 ) THEN
!            ANORM = ABS( D( 1 ) )
!<<<            ANORM = abs(D(1))
                if (Dh(1) .lt. 0.0d+0) then
                  ANORMh = -Dh(1)
                  ANORMl = -Dl(1)
                elseif (
     +            (Dh(1) .eq. 0.0d+0).and.
     +            (Dl(1) .lt. 0.0d+0)) then
                  ANORMh = Dh(1)
                  ANORMl = -Dl(1)
                else
                  ANORMh = Dh(1)
                  ANORMl = Dl(1)
                endif
!>>>            ANORM = abs(D(1))
         ELSE
!            ANORM = MAX( ABS( D( 1 ) )+ABS( E( 1 ) ),
!     $              ABS( E( N-1 ) )+ABS( D( N ) ) )
!<<<            tmp1 = abs(D(1))
                if (Dh(1) .lt. 0.0d+0) then
                  tmp1h = -Dh(1)
                  tmp1l = -Dl(1)
                elseif (
     +            (Dh(1) .eq. 0.0d+0).and.
     +            (Dl(1) .lt. 0.0d+0)) then
                  tmp1h = Dh(1)
                  tmp1l = -Dl(1)
                else
                  tmp1h = Dh(1)
                  tmp1l = Dl(1)
                endif
!>>>            tmp1 = abs(D(1))
!<<<            tmp2 = abs(E(1))
                if (Eh(1) .lt. 0.0d+0) then
                  tmp2h = -Eh(1)
                  tmp2l = -El(1)
                elseif (
     +            (Eh(1) .eq. 0.0d+0).and.
     +            (El(1) .lt. 0.0d+0)) then
                  tmp2h = Eh(1)
                  tmp2l = -El(1)
                else
                  tmp2h = Eh(1)
                  tmp2l = El(1)
                endif
!>>>            tmp2 = abs(E(1))
!<<<            tmp3 = abs(E(N-1))
                if (Eh(N-1) .lt. 0.0d+0) then
                  tmp3h = -Eh(N-1)
                  tmp3l = -El(N-1)
                elseif (
     +            (Eh(N-1) .eq. 0.0d+0).and.
     +            (El(N-1) .lt. 0.0d+0)) then
                  tmp3h = Eh(N-1)
                  tmp3l = -El(N-1)
                else
                  tmp3h = Eh(N-1)
                  tmp3l = El(N-1)
                endif
!>>>            tmp3 = abs(E(N-1))
!<<<            tmp4 = abs(D(N))
                if (Dh(N) .lt. 0.0d+0) then
                  tmp4h = -Dh(N)
                  tmp4l = -Dl(N)
                elseif (
     +            (Dh(N) .eq. 0.0d+0).and.
     +            (Dl(N) .lt. 0.0d+0)) then
                  tmp4h = Dh(N)
                  tmp4l = -Dl(N)
                else
                  tmp4h = Dh(N)
                  tmp4l = Dl(N)
                endif
!>>>            tmp4 = abs(D(N))
!<<<            tmp5 = tmp1 + tmp2
                T_p1 = tmp1h + tmp2h
                T_p2 = T_p1 - tmp1h
                tmp5h = T_p1
                tmp5l = (tmp1h - (T_p1 - T_p2))
     +                + (tmp2h - T_p2)
                tmp5l = tmp5l + tmp1l + tmp2l
                T_s1 = tmp5h + tmp5l
                tmp5l = tmp5l - (T_s1 - tmp5h)
                tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>            tmp5 = tmp1 + tmp2
!<<<            tmp6 = tmp3 + tmp4
                T_p1 = tmp3h + tmp4h
                T_p2 = T_p1 - tmp3h
                tmp6h = T_p1
                tmp6l = (tmp3h - (T_p1 - T_p2))
     +                + (tmp4h - T_p2)
                tmp6l = tmp6l + tmp3l + tmp4l
                T_s1 = tmp6h + tmp6l
                tmp6l = tmp6l - (T_s1 - tmp6h)
                tmp6h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>            tmp6 = tmp3 + tmp4
             IF( 
!                 (tmp5 .ge. tmp6)
     +           ((tmp5h .gt. tmp6h) .or.
     +            ((tmp5h .eq. tmp6h) .and.
     +             (tmp5l .ge. tmp6l)))
     &          ) THEN
                ANORMh = tmp5h
                ANORMl = tmp5l
             ELSE
                ANORMh = tmp6h
                ANORMl = tmp6l
             END IF
            DO 20 I = 2, N - 1
!               ANORM = MAX( ANORM, ABS( D( I ) )+ABS( E( I ) )+
!     $                 ABS( E( I-1 ) ) )
!<<<            tmp1 = abs(D(I))
                if (Dh(I) .lt. 0.0d+0) then
                  tmp1h = -Dh(I)
                  tmp1l = -Dl(I)
                elseif (
     +            (Dh(I) .eq. 0.0d+0).and.
     +            (Dl(I) .lt. 0.0d+0)) then
                  tmp1h = Dh(I)
                  tmp1l = -Dl(I)
                else
                  tmp1h = Dh(I)
                  tmp1l = Dl(I)
                endif
!>>>            tmp1 = abs(D(I))
!<<<            tmp2 = abs(E(I))
                if (Eh(I) .lt. 0.0d+0) then
                  tmp2h = -Eh(I)
                  tmp2l = -El(I)
                elseif (
     +            (Eh(I) .eq. 0.0d+0).and.
     +            (El(I) .lt. 0.0d+0)) then
                  tmp2h = Eh(I)
                  tmp2l = -El(I)
                else
                  tmp2h = Eh(I)
                  tmp2l = El(I)
                endif
!>>>            tmp2 = abs(E(I))
!<<<            tmp3 = abs(E(I-1))
                if (Eh(I-1) .lt. 0.0d+0) then
                  tmp3h = -Eh(I-1)
                  tmp3l = -El(I-1)
                elseif (
     +            (Eh(I-1) .eq. 0.0d+0).and.
     +            (El(I-1) .lt. 0.0d+0)) then
                  tmp3h = Eh(I-1)
                  tmp3l = -El(I-1)
                else
                  tmp3h = Eh(I-1)
                  tmp3l = El(I-1)
                endif
!>>>            tmp3 = abs(E(I-1))
!<<<            tmp4 = tmp1 + tmp2
                T_p1 = tmp1h + tmp2h
                T_p2 = T_p1 - tmp1h
                tmp4h = T_p1
                tmp4l = (tmp1h - (T_p1 - T_p2))
     +                + (tmp2h - T_p2)
                tmp4l = tmp4l + tmp1l + tmp2l
                T_s1 = tmp4h + tmp4l
                tmp4l = tmp4l - (T_s1 - tmp4h)
                tmp4h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>            tmp4 = tmp1 + tmp2
!<<<            tmp5 = tmp4 + tmp3
                T_p1 = tmp4h + tmp3h
                T_p2 = T_p1 - tmp4h
                tmp5h = T_p1
                tmp5l = (tmp4h - (T_p1 - T_p2))
     +                + (tmp3h - T_p2)
                tmp5l = tmp5l + tmp4l + tmp3l
                T_s1 = tmp5h + tmp5l
                tmp5l = tmp5l - (T_s1 - tmp5h)
                tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>            tmp5 = tmp4 + tmp3
             IF( 
!                 (ANORM .ge. tmp5)
     +           ((ANORMh .gt. tmp5h) .or.
     +            ((ANORMh .eq. tmp5h) .and.
     +             (ANORMl .ge. tmp5l)))
     &          ) THEN
                ANORMh = ANORMh
                ANORMl = ANORMl
             ELSE
                ANORMh = tmp5h
                ANORMl = tmp5l
             END IF

   20       CONTINUE
         END IF
      ELSE IF( ( LSAME( NORM, 'F' ) ) .OR. ( LSAME( NORM, 'E' ) ) ) THEN
*
*        Find normF(A).
*
!         SCALE = ZERO
                SCALEh = ZEROh
                SCALEl = ZEROl
!         SUM = ONE
                SUMh = ONEh
                SUMl = ONEl
         IF( N.GT.1 ) THEN
!            CALL DLASSQ( N-1, E, 1, SCALE, SUM )
            CALL DDLASSQ( N-1, Eh,El, 1, SCALEh,SCALEl, SUMh, SUMl )
!            SUM = 2*SUM
                tmp1h = 2
                tmp1l = 0.0d+0
!<<<            SUM = tmp1 * SUM
                T_ch = tmp1h * SUMh
                T_u1 = 134217729.0 * tmp1h
                T_p1 = T_u1
     +                   - (T_u1 - tmp1h)
                T_p2 = tmp1h - T_p1
                T_u1 = 134217729.0 * SUMh
                T_r1 = T_u1
     +                   - (T_u1 - SUMh)
                T_r2 = SUMh - T_r1
                T_cl = ((T_p1*T_r1
     +                   - T_ch)
     +                  + T_p1 * T_r2
     +                  + T_p2 * T_r1)
     +                  + T_p2 * T_r2
                T_cl = T_cl
     +                   + (tmp1h*SUMl
     +                   +  tmp1l*SUMh)
                T_s1 = T_ch + T_cl
                SUMl = T_cl - (T_s1 - T_ch)
                SUMh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>            SUM = tmp1 * SUM
         END IF
!         CALL DLASSQ( N, D, 1, SCALE, SUM )
         CALL DDLASSQ( N, Dh,Dl, 1, SCALEh,SCALEl, SUMh, SUMl )
!         ANORM = SCALE*SQRT( SUM )
!<<<            tmp1 = sqrt(SUM)
                if (SUMh.eq.0.0D+0.and.SUMl.eq.0.0D+0) then
                  tmp1h = 0.0d+0
                  tmp1l = 0.0d+0
                endif
                if (SUMh.lt.0.0d+0.or.
     +             (SUMh.eq.0.0d+0.and.SUMl.lt.0.0d+0)) then
                  write(6,'(a)')
     +               'SQRT CALLED WITH NEGATIVE ARGUMENT'
                  tmp1h = 0.0d+0
                  tmp1l = 0.0d+0
                  return
                endif
                T_x = 1.0D+0/sqrt(SUMh)
                T_ax = SUMh * T_x
!                   call dd_mul(T_y1,T_y2,T_x,
!                               0.0,0.5,0.0)
                    T_y1 = T_x * 0.5d+0
                    T_u1 = 134217729.0 * T_x
                    T_p1 = T_u1
     +                       - (T_u1 - T_x)
                    T_p2 = T_x - T_p1
                    T_u1 = 134217729.0 * 0.5d+0
                    T_r1 = T_u1
     +                       - (T_u1 - 0.5d+0)
                    T_r2 = 0.5d+0 - T_r1
                    T_y2 = ((T_p1*T_r1
     +                       - T_y1)
     +                       + T_p1*T_r2
     +                       + T_p2 * T_r1)
     +                       + T_p2 * T_r2
                    T_s1 = T_y1 + T_y2
                    T_y2 = T_y2
     +                       - (T_s1 - T_y1)
                    T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                   call dd_mul(T_z1,T_z2,T_ax,
!                               0.0,T_ax,0.0)
                    T_z1 = T_ax * T_ax
                    T_u1 = 134217729.0 * T_ax
                    T_p1 = T_u1
     +                       - (T_u1 - T_ax)
                    T_p2 = T_ax - T_p1
                    T_u1 = 134217729.0 * T_ax
                    T_r1 = T_u1
     +                       - (T_u1 - T_ax)
                    T_r2 = T_ax - T_r1
                    T_z2 = ((T_p1 * T_r1
     +                       - T_z1)
     +                       + T_p1 * T_r2
     +                       + T_p2 * T_r1)
     +                       + T_p2*T_r2
                    T_s1 = T_z1 + T_z2
                    T_z2 = T_z2
     +                       - (T_s1 - T_z1)
                    T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                   call dd_sub(T_t2h,T_t2l,
!                           SUMh,SUMl,T_z1,T_z2)
                    T_p1 = SUMh - T_z1
                    T_p2 = T_p1 - SUMh
                    T_t2l = (SUMh
     +                       -(T_p1 - T_p2))
     +                       -(T_z1 + T_p2)
                    T_t2h = T_p1
                    T_t2l = T_t2l
     +                       + SUMl - T_z2
                    T_s1 = T_t2h + T_t2l
                    T_t2l = T_t2l
     +                       - (T_s1 - T_t2h)
                    T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!                   call dd_mul(T_t1h,T_t1l,
!                               T_t2h,T_t2l,
!                               T_y1,T_y2)
                    T_t1h = T_t2h * T_y1
                    T_u1 = 134217729.0 * T_t2h
                    T_p1 = T_u1
     +                       - (T_u1 - T_t2h)
                    T_p2 = T_t2h - T_p1
                    T_u1 = 134217729.0 * T_y1
                    T_r1 = T_u1
     +                       - (T_u1 - T_y1)
                    T_r2 = T_y1 - T_r1
                    T_t1l = ((T_p1*T_r1
     +                       - T_t1h)
     +                       + T_p1*T_r2
     +                       + T_p2*T_r1)
     +                       + T_p2*T_r2
                    T_t1l = T_t1l
     +                       + (T_t2h * T_y2
     +                       +  T_t2l * T_y1)
                    T_s1 = T_t1h + T_t1l
                    T_t1l = T_t1l - (T_s1
     +                        - T_t1h)
                    T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!                   call dd_selfadd(T_t1h,T_t1l,
!                               T_ax,0.0)
                    T_p1 = T_t1h + T_ax
                    T_p2 = T_p1 - T_t1h
                    T_q2 = (T_t1h
     +                       - (T_p1 - T_p2))
     +                       + (T_ax - T_p2)
                    T_q1 = T_p1
                    T_q2 = T_q2 + T_t1l
                    T_t1h = T_q1 + T_q2
                    T_t1l = T_q2
     +                        - (T_t1h - T_q1)
                    tmp1h = T_t1h
                    tmp1l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>            tmp1 = sqrt(SUM)
!<<<            ANORM = SCALE * tmp1
                ANORMh = SCALEh * tmp1h
                T_u1 = 134217729.0 * SCALEh
                T_p1 = T_u1
     +                   - (T_u1 - SCALEh)
                T_p2 = SCALEh - T_p1
                T_u1 = 134217729.0 * tmp1h
                T_r1 = T_u1
     +                   - (T_u1 - tmp1h)
                T_r2 = tmp1h - T_r1
                ANORMl = ((T_p1*T_r1
     +                  - ANORMh)
     +                  + T_p1 * T_r2
     +                  + T_p2 * T_r1)
     +                  + T_p2 * T_r2
                ANORMl = ANORMl
     +                + (SCALEh*tmp1l
     +                + SCALEl*tmp1h)
                T_s1 = ANORMh + ANORMl
                ANORMl = ANORMl - (T_s1 - ANORMh)
                ANORMh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>            ANORM = SCALE * tmp1
      END IF
*
!      DLANST = ANORM
                DLANSTh = ANORMh
                DLANSTl = ANORMl
      RETURN
*
*     End of DLANST
*
      END
