*> \brief \b DLAS2
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLAS2 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlas2.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlas2.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlas2.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLAS2( F, G, H, SSMIN, SSMAX )
* 
*       .. Scalar Arguments ..
*       DOUBLE PRECISION   F, G, H, SSMAX, SSMIN
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLAS2  computes the singular values of the 2-by-2 matrix
*>    [  F   G  ]
*>    [  0   H  ].
*> On return, SSMIN is the smaller singular value and SSMAX is the
*> larger singular value.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] F
*> \verbatim
*>          F is DOUBLE PRECISION
*>          The (1,1) element of the 2-by-2 matrix.
*> \endverbatim
*>
*> \param[in] G
*> \verbatim
*>          G is DOUBLE PRECISION
*>          The (1,2) element of the 2-by-2 matrix.
*> \endverbatim
*>
*> \param[in] H
*> \verbatim
*>          H is DOUBLE PRECISION
*>          The (2,2) element of the 2-by-2 matrix.
*> \endverbatim
*>
*> \param[out] SSMIN
*> \verbatim
*>          SSMIN is DOUBLE PRECISION
*>          The smaller singular value.
*> \endverbatim
*>
*> \param[out] SSMAX
*> \verbatim
*>          SSMAX is DOUBLE PRECISION
*>          The larger singular value.
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERauxiliary
*
*> \par Further Details:
*  =====================
*>
*> \verbatim
*>
*>  Barring over/underflow, all output quantities are correct to within
*>  a few units in the last place (ulps), even in the absence of a guard
*>  digit in addition/subtraction.
*>
*>  In IEEE arithmetic, the code works correctly if one matrix element is
*>  infinite.
*>
*>  Overflow will not occur unless the largest singular value itself
*>  overflows, or is within a few ulps of overflow. (On machines with
*>  partial overflow, like the Cray, overflow may occur if the largest
*>  singular value is within a factor of 2 of overflow.)
*>
*>  Underflow is harmless if underflow is gradual. Otherwise, results
*>  may correspond to a matrix modified by perturbations of size near
*>  the underflow threshold.
*> \endverbatim
*>
*  =====================================================================
!!      SUBROUTINE DLAS2( F, G, H, SSMIN, SSMAX )
      SUBROUTINE dDLAS2( Fh, Fl, Gh, Gl, Hh, Hl, SSMINh, SSMINl,
     $                   SSMAXh, SSMAXl )
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
!!      DOUBLE PRECISION   F, G, H, SSMAX, SSMIN
      double precision Fh, Fl
      double precision Gh, Gl
      double precision Hh, Hl
      double precision SSMAXh, SSMAXl
      double precision SSMINh, SSMINl
*     ..
*
*  ====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION   ZERO
!!      PARAMETER          ( ZERO = 0.0D0 )
!!      DOUBLE PRECISION   ONE
!!      PARAMETER          ( ONE = 1.0D0 )
!!      DOUBLE PRECISION   TWO
!!      PARAMETER          ( TWO = 2.0D0 )
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
      double precision TWOh, TWOl
*     ..
*     .. Local Scalars ..
!!      DOUBLE PRECISION   AS, AT, AU, C, FA, FHMN, FHMX, GA, HA
      double precision ASh, ASl
      double precision ATh, ATl
      double precision AUh, AUl
      double precision Ch, Cl
      double precision FAh, FAl
      double precision FHMNh, FHMNl
      double precision FHMXh, FHMXl
      double precision GAh, GAl
      double precision HAh, HAl
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, MAX, MIN, SQRT
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
      double precision tmp9h, tmp9l
*     ..
*     .. Executable Statements ..
      ZEROh = 0.0D0
      ZEROl = 0.0d+0
      ONEh = 1.0D0
      ONEl = 0.0d+0
      TWOh = 2.0D0
      TWOl = 0.0d+0
*
!!      FA = ABS( F )
!<<<  FA = abs(F)
      if (Fh .lt. 0.0d+0) then
        FAh = -Fh
        FAl = -Fl
      elseif (
     +  (Fh .eq. 0.0d+0).and.
     +  (Fl .lt. 0.0d+0)) then
        FAh = Fh
        FAl = -Fl
      else
        FAh = Fh
        FAl = Fl
      endif
!>>>  FA = abs(F)
!!      GA = ABS( G )
!<<<  GA = abs(G)
      if (Gh .lt. 0.0d+0) then
        GAh = -Gh
        GAl = -Gl
      elseif (
     +  (Gh .eq. 0.0d+0).and.
     +  (Gl .lt. 0.0d+0)) then
        GAh = Gh
        GAl = -Gl
      else
        GAh = Gh
        GAl = Gl
      endif
!>>>  GA = abs(G)
!!      HA = ABS( H )
!<<<  HA = abs(H)
      if (Hh .lt. 0.0d+0) then
        HAh = -Hh
        HAl = -Hl
      elseif (
     +  (Hh .eq. 0.0d+0).and.
     +  (Hl .lt. 0.0d+0)) then
        HAh = Hh
        HAl = -Hl
      else
        HAh = Hh
        HAl = Hl
      endif
!>>>  HA = abs(H)
!!      FHMN = MIN( FA, HA )
      IF(
!       (FA .lt. HA)
     + ((FAh .lt. HAh) .or.
     +  ((FAh .eq. HAh) .and.
     +   (FAl .lt. HAl)))
     $  ) THEN
      FHMNh = FAh
      FHMNl = FAl
      ELSE
      FHMNh = HAh
      FHMNl = HAl
      ENDIF
!!      FHMX = MAX( FA, HA )
      IF(
!       (FA .gt. HA)
     + ((FAh .gt. HAh) .or.
     +  ((FAh .eq. HAh) .and.
     +   (FAl .gt. HAl)))
     $  ) THEN
      FHMXh = FAh
      FHMXl = FAl
      ELSE
      FHMXh = HAh
      FHMXl = HAl
      END IF
!!      IF( FHMN.EQ.ZERO ) THEN
      IF(
!       (FHMN .eq. ZERO)
     + ((FHMNh .eq. ZEROh) .and.
     +  (FHMNl .eq. ZEROl))
     $  ) THEN
!!         SSMIN = ZERO
      SSMINh = ZEROh
      SSMINl = ZEROl
!!         IF( FHMX.EQ.ZERO ) THEN
         IF(
!       (FHMX .eq. ZERO)
     + ((FHMXh .eq. ZEROh) .and.
     +  (FHMXl .eq. ZEROl))
     $     ) THEN
!!            SSMAX = GA
      SSMAXh = GAh
      SSMAXl = GAl
         ELSE
!!            SSMAX = MAX( FHMX, GA )*SQRT( ONE+
!!     $              ( MIN( FHMX, GA ) / MAX( FHMX, GA ) )**2 )
            IF(
!       (FHMX .gt. GA)
     + ((FHMXh .gt. GAh) .or.
     +  ((FHMXh .eq. GAh) .and.
     +   (FHMXl .gt. GAl)))
     $        ) THEN
      tmp1h = FHMXh
      tmp1l = FHMXl
            ELSE
      tmp1h = GAh
      tmp1l = GAl
            END IF
            IF(
!       (FHMX .lt. GA)
     + ((FHMXh .lt. GAh) .or.
     +  ((FHMXh .eq. GAh) .and.
     +   (FHMXl .lt. GAl)))
     $        ) THEN
      tmp2h = FHMXh
      tmp2l = FHMXl
            ELSE
      tmp2h = GAh
      tmp2l = GAl
            END IF
!<<<  tmp3 = tmp2 / tmp1
      T_q1 = tmp2h / tmp1h
      T_ch = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp1l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = tmp2h - T_ch
      T_r1 = T_s1 - tmp2h
      T_s2 = (tmp2h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + tmp2l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = tmp2 / tmp1
!<<<  tmp4 = tmp3 * tmp3
      T_ch = tmp3h * tmp3h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*tmp3l
     +         +  tmp3l*tmp3h)
      T_s1 = T_ch + T_cl
      tmp4l = T_cl - (T_s1 - T_ch)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp4 = tmp3 * tmp3
!<<<  tmp5 = ONE + tmp4
      T_p1 = ONEh + tmp4h
      T_p2 = T_p1 - ONEh
      T_q1 = T_p1
      T_q2 = (ONEh - (T_p1 - T_p2))
     +         + (tmp4h - T_p2)
      T_q2 = T_q2 + ONEl + tmp4l
      tmp5h = T_q1 + T_q2
      tmp5l = T_q2 - (tmp5h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp5 = ONE + tmp4
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      else
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  SSMAX = tmp1 * tmp6
      T_ch = tmp1h * tmp6h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp6h
      T_r1 = T_u1
     +         - (T_u1 - tmp6h)
      T_r2 = tmp6h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp6l
     +         +  tmp1l*tmp6h)
      T_s1 = T_ch + T_cl
      SSMAXl = T_cl - (T_s1 - T_ch)
      SSMAXh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  SSMAX = tmp1 * tmp6
         END IF
      ELSE
!!         IF( GA.LT.FHMX ) THEN
         IF(
!       (GA .lt. FHMX)
     + ((GAh .lt. FHMXh) .or.
     +  ((GAh .eq. FHMXh) .and.
     +   (GAl .lt. FHMXl)))
     $     ) THEN
!!            AS = ONE + FHMN / FHMX
!<<<  tmp1 = FHMN / FHMX
      T_q1 = FHMNh / FHMXh
      T_ch = FHMXh * T_q1
      T_u1 = 134217729.0 * FHMXh
      T_p1 = T_u1
     +         - (T_u1 - FHMXh)
      T_p2 = FHMXh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (FHMXl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = FHMNh - T_ch
      T_r1 = T_s1 - FHMNh
      T_s2 = (FHMNh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + FHMNl
      T_q2 = (T_s1 + T_s2)
     +         / FHMXh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = FHMN / FHMX
!<<<  AS = ONE + tmp1
      T_p1 = ONEh + tmp1h
      T_p2 = T_p1 - ONEh
      T_q1 = T_p1
      T_q2 = (ONEh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + ONEl + tmp1l
      ASh = T_q1 + T_q2
      ASl = T_q2 - (ASh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  AS = ONE + tmp1
!!            AT = ( FHMX-FHMN ) / FHMX
!<<<  tmp1 = FHMX - FHMN
      T_p1 = FHMXh - FHMNh
      T_p2 = T_p1 - FHMXh
      T_ch = T_p1
      T_cl = (FHMXh - (T_p1 - T_p2))
     +         - (FHMNh + T_p2)
      T_cl = T_cl + FHMXl - FHMNl
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = FHMX - FHMN
!<<<  AT = tmp1 / FHMX
      T_q1 = tmp1h / FHMXh
      T_ch = FHMXh * T_q1
      T_u1 = 134217729.0 * FHMXh
      T_p1 = T_u1
     +         - (T_u1 - FHMXh)
      T_p2 = FHMXh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (FHMXl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = tmp1h - T_ch
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / FHMXh
      ATh = T_q1 + T_q2
      ATl = T_q2
     +      - (ATh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  AT = tmp1 / FHMX
!!            AU = ( GA / FHMX )**2
!<<<  tmp1 = GA / FHMX
      T_q1 = GAh / FHMXh
      T_ch = FHMXh * T_q1
      T_u1 = 134217729.0 * FHMXh
      T_p1 = T_u1
     +         - (T_u1 - FHMXh)
      T_p2 = FHMXh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (FHMXl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = GAh - T_ch
      T_r1 = T_s1 - GAh
      T_s2 = (GAh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + GAl
      T_q2 = (T_s1 + T_s2)
     +         / FHMXh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = GA / FHMX
!<<<  AU = tmp1 * tmp1
      T_ch = tmp1h * tmp1h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp1l
     +         +  tmp1l*tmp1h)
      T_s1 = T_ch + T_cl
      AUl = T_cl - (T_s1 - T_ch)
      AUh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  AU = tmp1 * tmp1
!!            C = TWO / ( SQRT( AS*AS+AU )+SQRT( AT*AT+AU ) )
!<<<  tmp1 = AS * AS
      T_ch = ASh * ASh
      T_u1 = 134217729.0 * ASh
      T_p1 = T_u1
     +         - (T_u1 - ASh)
      T_p2 = ASh - T_p1
      T_u1 = 134217729.0 * ASh
      T_r1 = T_u1
     +         - (T_u1 - ASh)
      T_r2 = ASh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (ASh*ASl
     +         +  ASl*ASh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = AS * AS
!<<<  tmp2 = tmp1 + AU
      T_p1 = tmp1h + AUh
      T_p2 = T_p1 - tmp1h
      T_q1 = T_p1
      T_q2 = (tmp1h - (T_p1 - T_p2))
     +         + (AUh - T_p2)
      T_q2 = T_q2 + tmp1l + AUl
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2 - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = tmp1 + AU
!<<<  tmp3 = sqrt(tmp2)
      if (tmp2h.eq.0.0D+0.and.tmp2l.eq.0.0D+0) then
        tmp3h = 0.0d+0
        tmp3l = 0.0d+0
      else
      if (tmp2h.lt.0.0d+0.or.
     +   (tmp2h.eq.0.0d+0.and.tmp2l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp3h = 0.0d+0
        tmp3l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp2h)
      T_ax = tmp2h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp2h,tmp2l,T_z1,T_z2)
          T_p1 = tmp2h - T_z1
          T_p2 = T_p1 - tmp2h
          T_t2l = (tmp2h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp2l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp3h = T_t1h
          tmp3l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp3 = sqrt(tmp2)
!<<<  tmp4 = AT * AT
      T_ch = ATh * ATh
      T_u1 = 134217729.0 * ATh
      T_p1 = T_u1
     +         - (T_u1 - ATh)
      T_p2 = ATh - T_p1
      T_u1 = 134217729.0 * ATh
      T_r1 = T_u1
     +         - (T_u1 - ATh)
      T_r2 = ATh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (ATh*ATl
     +         +  ATl*ATh)
      T_s1 = T_ch + T_cl
      tmp4l = T_cl - (T_s1 - T_ch)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp4 = AT * AT
!<<<  tmp5 = tmp4 + AU
      T_p1 = tmp4h + AUh
      T_p2 = T_p1 - tmp4h
      T_q1 = T_p1
      T_q2 = (tmp4h - (T_p1 - T_p2))
     +         + (AUh - T_p2)
      T_q2 = T_q2 + tmp4l + AUl
      tmp5h = T_q1 + T_q2
      tmp5l = T_q2 - (tmp5h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp5 = tmp4 + AU
!<<<  tmp6 = sqrt(tmp5)
      if (tmp5h.eq.0.0D+0.and.tmp5l.eq.0.0D+0) then
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
      else
      if (tmp5h.lt.0.0d+0.or.
     +   (tmp5h.eq.0.0d+0.and.tmp5l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp6h = 0.0d+0
        tmp6l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp5h)
      T_ax = tmp5h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp5h,tmp5l,T_z1,T_z2)
          T_p1 = tmp5h - T_z1
          T_p2 = T_p1 - tmp5h
          T_t2l = (tmp5h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp5l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp6h = T_t1h
          tmp6l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp6 = sqrt(tmp5)
!<<<  tmp7 = tmp3 + tmp6
      T_p1 = tmp3h + tmp6h
      T_p2 = T_p1 - tmp3h
      T_q1 = T_p1
      T_q2 = (tmp3h - (T_p1 - T_p2))
     +         + (tmp6h - T_p2)
      T_q2 = T_q2 + tmp3l + tmp6l
      tmp7h = T_q1 + T_q2
      tmp7l = T_q2 - (tmp7h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp7 = tmp3 + tmp6
!<<<  C = TWO / tmp7
      T_q1 = TWOh / tmp7h
      T_ch = tmp7h * T_q1
      T_u1 = 134217729.0 * tmp7h
      T_p1 = T_u1
     +         - (T_u1 - tmp7h)
      T_p2 = tmp7h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp7l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = TWOh - T_ch
      T_r1 = T_s1 - TWOh
      T_s2 = (TWOh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + TWOl
      T_q2 = (T_s1 + T_s2)
     +         / tmp7h
      Ch = T_q1 + T_q2
      Cl = T_q2
     +      - (Ch - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  C = TWO / tmp7
!!            SSMIN = FHMN*C
!<<<  SSMIN = FHMN * C
      T_ch = FHMNh * Ch
      T_u1 = 134217729.0 * FHMNh
      T_p1 = T_u1
     +         - (T_u1 - FHMNh)
      T_p2 = FHMNh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (FHMNh*Cl
     +         +  FHMNl*Ch)
      T_s1 = T_ch + T_cl
      SSMINl = T_cl - (T_s1 - T_ch)
      SSMINh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  SSMIN = FHMN * C
!!            SSMAX = FHMX / C
!<<<  SSMAX = FHMX / C
      T_q1 = FHMXh / Ch
      T_ch = Ch * T_q1
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Cl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = FHMXh - T_ch
      T_r1 = T_s1 - FHMXh
      T_s2 = (FHMXh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + FHMXl
      T_q2 = (T_s1 + T_s2)
     +         / Ch
      SSMAXh = T_q1 + T_q2
      SSMAXl = T_q2
     +      - (SSMAXh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SSMAX = FHMX / C
         ELSE
!!            AU = FHMX / GA
!<<<  AU = FHMX / GA
      T_q1 = FHMXh / GAh
      T_ch = GAh * T_q1
      T_u1 = 134217729.0 * GAh
      T_p1 = T_u1
     +         - (T_u1 - GAh)
      T_p2 = GAh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (GAl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = FHMXh - T_ch
      T_r1 = T_s1 - FHMXh
      T_s2 = (FHMXh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + FHMXl
      T_q2 = (T_s1 + T_s2)
     +         / GAh
      AUh = T_q1 + T_q2
      AUl = T_q2
     +      - (AUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  AU = FHMX / GA
!!            IF( AU.EQ.ZERO ) THEN
            IF(
!       (AU .eq. ZERO)
     + ((AUh .eq. ZEROh) .and.
     +  (AUl .eq. ZEROl))
     $        ) THEN
*
*              Avoid possible harmful underflow if exponent range
*              asymmetric (true SSMIN may not underflow even if
*              AU underflows)
*
!!               SSMIN = ( FHMN*FHMX ) / GA
!<<<  tmp1 = FHMN * FHMX
      T_ch = FHMNh * FHMXh
      T_u1 = 134217729.0 * FHMNh
      T_p1 = T_u1
     +         - (T_u1 - FHMNh)
      T_p2 = FHMNh - T_p1
      T_u1 = 134217729.0 * FHMXh
      T_r1 = T_u1
     +         - (T_u1 - FHMXh)
      T_r2 = FHMXh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (FHMNh*FHMXl
     +         +  FHMNl*FHMXh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = FHMN * FHMX
!<<<  SSMIN = tmp1 / GA
      T_q1 = tmp1h / GAh
      T_ch = GAh * T_q1
      T_u1 = 134217729.0 * GAh
      T_p1 = T_u1
     +         - (T_u1 - GAh)
      T_p2 = GAh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (GAl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = tmp1h - T_ch
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / GAh
      SSMINh = T_q1 + T_q2
      SSMINl = T_q2
     +      - (SSMINh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SSMIN = tmp1 / GA
!!               SSMAX = GA
      SSMAXh = GAh
      SSMAXl = GAl
            ELSE
!!               AS = ONE + FHMN / FHMX
!<<<  tmp1 = FHMN / FHMX
      T_q1 = FHMNh / FHMXh
      T_ch = FHMXh * T_q1
      T_u1 = 134217729.0 * FHMXh
      T_p1 = T_u1
     +         - (T_u1 - FHMXh)
      T_p2 = FHMXh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (FHMXl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = FHMNh - T_ch
      T_r1 = T_s1 - FHMNh
      T_s2 = (FHMNh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + FHMNl
      T_q2 = (T_s1 + T_s2)
     +         / FHMXh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = FHMN / FHMX
!<<<  AS = ONE + tmp1
      T_p1 = ONEh + tmp1h
      T_p2 = T_p1 - ONEh
      T_q1 = T_p1
      T_q2 = (ONEh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + ONEl + tmp1l
      ASh = T_q1 + T_q2
      ASl = T_q2 - (ASh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  AS = ONE + tmp1
!!               AT = ( FHMX-FHMN ) / FHMX
!<<<  tmp1 = FHMX - FHMN
      T_p1 = FHMXh - FHMNh
      T_p2 = T_p1 - FHMXh
      T_ch = T_p1
      T_cl = (FHMXh - (T_p1 - T_p2))
     +         - (FHMNh + T_p2)
      T_cl = T_cl + FHMXl - FHMNl
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  tmp1 = FHMX - FHMN
!<<<  AT = tmp1 / FHMX
      T_q1 = tmp1h / FHMXh
      T_ch = FHMXh * T_q1
      T_u1 = 134217729.0 * FHMXh
      T_p1 = T_u1
     +         - (T_u1 - FHMXh)
      T_p2 = FHMXh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (FHMXl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = tmp1h - T_ch
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / FHMXh
      ATh = T_q1 + T_q2
      ATl = T_q2
     +      - (ATh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  AT = tmp1 / FHMX
!!               C = ONE / ( SQRT( ONE+( AS*AU )**2 )+
!!     $             SQRT( ONE+( AT*AU )**2 ) )
!<<<  tmp1 = AS * AU
      T_ch = ASh * AUh
      T_u1 = 134217729.0 * ASh
      T_p1 = T_u1
     +         - (T_u1 - ASh)
      T_p2 = ASh - T_p1
      T_u1 = 134217729.0 * AUh
      T_r1 = T_u1
     +         - (T_u1 - AUh)
      T_r2 = AUh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (ASh*AUl
     +         +  ASl*AUh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = AS * AU
!<<<  tmp2 = tmp1 * tmp1
      T_ch = tmp1h * tmp1h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp1l
     +         +  tmp1l*tmp1h)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = tmp1 * tmp1
!<<<  tmp3 = ONE + tmp2
      T_p1 = ONEh + tmp2h
      T_p2 = T_p1 - ONEh
      T_q1 = T_p1
      T_q2 = (ONEh - (T_p1 - T_p2))
     +         + (tmp2h - T_p2)
      T_q2 = T_q2 + ONEl + tmp2l
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2 - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp3 = ONE + tmp2
!<<<  tmp4 = sqrt(tmp3)
      if (tmp3h.eq.0.0D+0.and.tmp3l.eq.0.0D+0) then
        tmp4h = 0.0d+0
        tmp4l = 0.0d+0
      else
      if (tmp3h.lt.0.0d+0.or.
     +   (tmp3h.eq.0.0d+0.and.tmp3l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp4h = 0.0d+0
        tmp4l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp3h)
      T_ax = tmp3h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp3h,tmp3l,T_z1,T_z2)
          T_p1 = tmp3h - T_z1
          T_p2 = T_p1 - tmp3h
          T_t2l = (tmp3h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp3l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp4h = T_t1h
          tmp4l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp4 = sqrt(tmp3)
!<<<  tmp5 = AT * AU
      T_ch = ATh * AUh
      T_u1 = 134217729.0 * ATh
      T_p1 = T_u1
     +         - (T_u1 - ATh)
      T_p2 = ATh - T_p1
      T_u1 = 134217729.0 * AUh
      T_r1 = T_u1
     +         - (T_u1 - AUh)
      T_r2 = AUh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (ATh*AUl
     +         +  ATl*AUh)
      T_s1 = T_ch + T_cl
      tmp5l = T_cl - (T_s1 - T_ch)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp5 = AT * AU
!<<<  tmp6 = tmp5 * tmp5
      T_ch = tmp5h * tmp5h
      T_u1 = 134217729.0 * tmp5h
      T_p1 = T_u1
     +         - (T_u1 - tmp5h)
      T_p2 = tmp5h - T_p1
      T_u1 = 134217729.0 * tmp5h
      T_r1 = T_u1
     +         - (T_u1 - tmp5h)
      T_r2 = tmp5h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp5h*tmp5l
     +         +  tmp5l*tmp5h)
      T_s1 = T_ch + T_cl
      tmp6l = T_cl - (T_s1 - T_ch)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp6 = tmp5 * tmp5
!<<<  tmp7 = ONE + tmp6
      T_p1 = ONEh + tmp6h
      T_p2 = T_p1 - ONEh
      T_q1 = T_p1
      T_q2 = (ONEh - (T_p1 - T_p2))
     +         + (tmp6h - T_p2)
      T_q2 = T_q2 + ONEl + tmp6l
      tmp7h = T_q1 + T_q2
      tmp7l = T_q2 - (tmp7h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp7 = ONE + tmp6
!<<<  tmp8 = sqrt(tmp7)
      if (tmp7h.eq.0.0D+0.and.tmp7l.eq.0.0D+0) then
        tmp8h = 0.0d+0
        tmp8l = 0.0d+0
      else
      if (tmp7h.lt.0.0d+0.or.
     +   (tmp7h.eq.0.0d+0.and.tmp7l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp8h = 0.0d+0
        tmp8l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp7h)
      T_ax = tmp7h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp7h,tmp7l,T_z1,T_z2)
          T_p1 = tmp7h - T_z1
          T_p2 = T_p1 - tmp7h
          T_t2l = (tmp7h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp7l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp8h = T_t1h
          tmp8l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  tmp8 = sqrt(tmp7)
!<<<  tmp9 = tmp4 + tmp8
      T_p1 = tmp4h + tmp8h
      T_p2 = T_p1 - tmp4h
      T_q1 = T_p1
      T_q2 = (tmp4h - (T_p1 - T_p2))
     +         + (tmp8h - T_p2)
      T_q2 = T_q2 + tmp4l + tmp8l
      tmp9h = T_q1 + T_q2
      tmp9l = T_q2 - (tmp9h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp9 = tmp4 + tmp8
!<<<  C = ONE / tmp9
      T_q1 = ONEh / tmp9h
      T_ch = tmp9h * T_q1
      T_u1 = 134217729.0 * tmp9h
      T_p1 = T_u1
     +         - (T_u1 - tmp9h)
      T_p2 = tmp9h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp9l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = ONEh - T_ch
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / tmp9h
      Ch = T_q1 + T_q2
      Cl = T_q2
     +      - (Ch - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  C = ONE / tmp9
!!               SSMIN = ( FHMN*C )*AU
!<<<  tmp1 = FHMN * C
      T_ch = FHMNh * Ch
      T_u1 = 134217729.0 * FHMNh
      T_p1 = T_u1
     +         - (T_u1 - FHMNh)
      T_p2 = FHMNh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (FHMNh*Cl
     +         +  FHMNl*Ch)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = FHMN * C
!<<<  SSMIN = tmp1 * AU
      T_ch = tmp1h * AUh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * AUh
      T_r1 = T_u1
     +         - (T_u1 - AUh)
      T_r2 = AUh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*AUl
     +         +  tmp1l*AUh)
      T_s1 = T_ch + T_cl
      SSMINl = T_cl - (T_s1 - T_ch)
      SSMINh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  SSMIN = tmp1 * AU
!!               SSMIN = SSMIN + SSMIN
!<<<  SSMIN = SSMIN + SSMIN
      T_p1 = SSMINh + SSMINh
      T_p2 = T_p1 - SSMINh
      T_q1 = T_p1
      T_q2 = (SSMINh - (T_p1 - T_p2))
     +         + (SSMINh - T_p2)
      T_q2 = T_q2 + SSMINl + SSMINl
      SSMINh = T_q1 + T_q2
      SSMINl = T_q2 - (SSMINh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  SSMIN = SSMIN + SSMIN
!!               SSMAX = GA / ( C+C )
!<<<  tmp1 = C + C
      T_p1 = Ch + Ch
      T_p2 = T_p1 - Ch
      T_q1 = T_p1
      T_q2 = (Ch - (T_p1 - T_p2))
     +         + (Ch - T_p2)
      T_q2 = T_q2 + Cl + Cl
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = C + C
!<<<  SSMAX = GA / C
      T_q1 = GAh / Ch
      T_ch = Ch * T_q1
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Cl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = GAh - T_ch
      T_r1 = T_s1 - GAh
      T_s2 = (GAh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + GAl
      T_q2 = (T_s1 + T_s2)
     +         / Ch
      SSMAXh = T_q1 + T_q2
      SSMAXl = T_q2
     +      - (SSMAXh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SSMAX = GA / C
            END IF
         END IF
      END IF
      RETURN
*
*     End of DLAS2
*
      END
