*> \brief \b DLASV2
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLASV2 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlasv2.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlasv2.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlasv2.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLASV2( F, G, H, SSMIN, SSMAX, SNR, CSR, SNL, CSL )
* 
*       .. Scalar Arguments ..
*       DOUBLE PRECISION   CSL, CSR, F, G, H, SNL, SNR, SSMAX, SSMIN
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLASV2 computes the singular value decomposition of a 2-by-2
*> triangular matrix
*>    [  F   G  ]
*>    [  0   H  ].
*> On return, abs(SSMAX) is the larger singular value, abs(SSMIN) is the
*> smaller singular value, and (CSL,SNL) and (CSR,SNR) are the left and
*> right singular vectors for abs(SSMAX), giving the decomposition
*>
*>    [ CSL  SNL ] [  F   G  ] [ CSR -SNR ]  =  [ SSMAX   0   ]
*>    [-SNL  CSL ] [  0   H  ] [ SNR  CSR ]     [  0    SSMIN ].
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] F
*> \verbatim
*>          F is DOUBLE PRECISION
*>          The (1,1) element of the 2-by-2 matrix.
*> \endverbatim
*>
*> \param[in] G
*> \verbatim
*>          G is DOUBLE PRECISION
*>          The (1,2) element of the 2-by-2 matrix.
*> \endverbatim
*>
*> \param[in] H
*> \verbatim
*>          H is DOUBLE PRECISION
*>          The (2,2) element of the 2-by-2 matrix.
*> \endverbatim
*>
*> \param[out] SSMIN
*> \verbatim
*>          SSMIN is DOUBLE PRECISION
*>          abs(SSMIN) is the smaller singular value.
*> \endverbatim
*>
*> \param[out] SSMAX
*> \verbatim
*>          SSMAX is DOUBLE PRECISION
*>          abs(SSMAX) is the larger singular value.
*> \endverbatim
*>
*> \param[out] SNL
*> \verbatim
*>          SNL is DOUBLE PRECISION
*> \endverbatim
*>
*> \param[out] CSL
*> \verbatim
*>          CSL is DOUBLE PRECISION
*>          The vector (CSL, SNL) is a unit left singular vector for the
*>          singular value abs(SSMAX).
*> \endverbatim
*>
*> \param[out] SNR
*> \verbatim
*>          SNR is DOUBLE PRECISION
*> \endverbatim
*>
*> \param[out] CSR
*> \verbatim
*>          CSR is DOUBLE PRECISION
*>          The vector (CSR, SNR) is a unit right singular vector for the
*>          singular value abs(SSMAX).
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERauxiliary
*
*> \par Further Details:
*  =====================
*>
*> \verbatim
*>
*>  Any input parameter may be aliased with any output parameter.
*>
*>  Barring over/underflow and assuming a guard digit in subtraction, all
*>  output quantities are correct to within a few units in the last
*>  place (ulps).
*>
*>  In IEEE arithmetic, the code works correctly if one matrix element is
*>  infinite.
*>
*>  Overflow will not occur unless the largest singular value itself
*>  overflows or is within a few ulps of overflow. (On machines with
*>  partial overflow, like the Cray, overflow may occur if the largest
*>  singular value is within a factor of 2 of overflow.)
*>
*>  Underflow is harmless if underflow is gradual. Otherwise, results
*>  may correspond to a matrix modified by perturbations of size near
*>  the underflow threshold.
*> \endverbatim
*>
*  =====================================================================
!!      SUBROUTINE DLASV2( F, G, H, SSMIN, SSMAX, SNR, CSR, SNL, CSL )
      SUBROUTINE dDLASV2( Fh, Fl, Gh, Gl, Hh, Hl, SSMINh, SSMINl,
     $                    SSMAXh, SSMAXl, SNRh, SNRl, CSRh, CSRl,
     $                    SNLh, SNLl, CSLh, CSLl )
*
*  -- LAPACK auxiliary routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
!!      DOUBLE PRECISION   CSL, CSR, F, G, H, SNL, SNR, SSMAX, SSMIN
      double precision CSLh, CSLl
      double precision CSRh, CSRl
      double precision Fh, Fl
      double precision Gh, Gl
      double precision Hh, Hl
      double precision SNLh, SNLl
      double precision SNRh, SNRl
      double precision SSMAXh, SSMAXl
      double precision SSMINh, SSMINl
*     ..
*
* =====================================================================
*
*     .. Parameters ..
!!      DOUBLE PRECISION   ZERO
!!      PARAMETER          ( ZERO = 0.0D0 )
!!      DOUBLE PRECISION   HALF
!!      PARAMETER          ( HALF = 0.5D0 )
!!      DOUBLE PRECISION   ONE
!!      PARAMETER          ( ONE = 1.0D0 )
!!      DOUBLE PRECISION   TWO
!!      PARAMETER          ( TWO = 2.0D0 )
!!      DOUBLE PRECISION   FOUR
!!      PARAMETER          ( FOUR = 4.0D0 )
      double precision ZEROh, ZEROl
      double precision HALFh, HALFl
      double precision ONEh, ONEl
      double precision TWOh, TWOl
      double precision FOURh, FOURl
*     ..
*     .. Local Scalars ..
      LOGICAL            GASMAL, SWAP
      INTEGER            PMAX
!!      DOUBLE PRECISION   A, CLT, CRT, D, FA, FT, GA, GT, HA, HT, L, M,
!!     $                   MM, R, S, SLT, SRT, T, TEMP, TSIGN, TT
      double precision Ah, Al
      double precision CLTh, CLTl
      double precision CRTh, CRTl
      double precision Dh, Dl
      double precision FAh, FAl
      double precision FTh, FTl
      double precision GAh, GAl
      double precision GTh, GTl
      double precision HAh, HAl
      double precision HTh, HTl
      double precision Lh, Ll
      double precision Mh, Ml
      double precision MMh, MMl
      double precision Rh, Rl
      double precision Sh, Sl
      double precision SLTh, SLTl
      double precision SRTh, SRTl
      double precision Th, Tl
      double precision TEMPh, TEMPl
      double precision TSIGNh, TSIGNl
      double precision TTh, TTl
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, SIGN, SQRT
*     ..
*     .. External Functions ..
!!      DOUBLE PRECISION   DLAMCH
!!      EXTERNAL           DLAMCH
      EXTERNAL           dDLAMCH
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
*     ..
*     .. Executable Statements ..
      ZEROh = 0.0D0
      ZEROl = 0.0d+0
      HALFh = 0.5D0
      HALFl = 0.0d+0
      ONEh = 1.0D0
      ONEl = 0.0d+0
      TWOh = 2.0D0
      TWOl = 0.0d+0
      FOURh = 4.0D0
      FOURl = 0.0d+0
*
!!      FT = F
      FTh = Fh
      FTl = Fl
!!      FA = ABS( FT )
!<<<  FA = abs(FT)
      if (FTh .lt. 0.0d+0) then
        FAh = -FTh
        FAl = -FTl
      elseif (
     +  (FTh .eq. 0.0d+0).and.
     +  (FTl .lt. 0.0d+0)) then
        FAh = FTh
        FAl = -FTl
      else
        FAh = FTh
        FAl = FTl
      endif
!>>>  FA = abs(FT)
!!      HT = H
      HTh = Hh
      HTl = Hl
!!      HA = ABS( H )
!<<<  HA = abs(H)
      if (Hh .lt. 0.0d+0) then
        HAh = -Hh
        HAl = -Hl
      elseif (
     +  (Hh .eq. 0.0d+0).and.
     +  (Hl .lt. 0.0d+0)) then
        HAh = Hh
        HAl = -Hl
      else
        HAh = Hh
        HAl = Hl
      endif
!>>>  HA = abs(H)
*
*     PMAX points to the maximum absolute element of matrix
*       PMAX = 1 if F largest in absolute values
*       PMAX = 2 if G largest in absolute values
*       PMAX = 3 if H largest in absolute values
*
      PMAX = 1
!!      SWAP = ( HA.GT.FA )
      IF(
!       (HA .gt. FA)
     + ((HAh .gt. FAh) .or.
     +  ((HAh .eq. FAh) .and.
     +   (HAl .gt. FAl)))
     $  ) THEN
         SWAP = .TRUE.
      ELSE
         SWAP = .FALSE.
      END IF
      IF( SWAP ) THEN
         PMAX = 3
!!         TEMP = FT
      TEMPh = FTh
      TEMPl = FTl
!!         FT = HT
      FTh = HTh
      FTl = HTl
!!         HT = TEMP
      HTh = TEMPh
      HTl = TEMPl
!!         TEMP = FA
      TEMPh = FAh
      TEMPl = FAl
!!         FA = HA
      FAh = HAh
      FAl = HAl
!!         HA = TEMP
      HAh = TEMPh
      HAl = TEMPl
*
*        Now FA .ge. HA
*
      END IF
!!      GT = G
      GTh = Gh
      GTl = Gl
!!      GA = ABS( GT )
!<<<  GA = abs(GT)
      if (GTh .lt. 0.0d+0) then
        GAh = -GTh
        GAl = -GTl
      elseif (
     +  (GTh .eq. 0.0d+0).and.
     +  (GTl .lt. 0.0d+0)) then
        GAh = GTh
        GAl = -GTl
      else
        GAh = GTh
        GAl = GTl
      endif
!>>>  GA = abs(GT)
!!      IF( GA.EQ.ZERO ) THEN
      IF(
!       (GA .eq. ZERO)
     + ((GAh .eq. ZEROh) .and.
     +  (GAl .eq. ZEROl))
     $  ) THEN
*
*        Diagonal matrix
*
!!         SSMIN = HA
      SSMINh = HAh
      SSMINl = HAl
!!         SSMAX = FA
      SSMAXh = FAh
      SSMAXl = FAl
!!         CLT = ONE
      CLTh = ONEh
      CLTl = ONEl
!!         CRT = ONE
      CRTh = ONEh
      CRTl = ONEl
!!         SLT = ZERO
      SLTh = ZEROh
      SLTl = ZEROl
!!         SRT = ZERO
      SRTh = ZEROh
      SRTl = ZEROl
      ELSE
         GASMAL = .TRUE.
!!         IF( GA.GT.FA ) THEN
         IF(
!       (GA .gt. FA)
     + ((GAh .gt. FAh) .or.
     +  ((GAh .eq. FAh) .and.
     +   (GAl .gt. FAl)))
     $     ) THEN
            PMAX = 2
!!            IF( ( FA / GA ).LT.DLAMCH( 'EPS' ) ) THEN
!<<<  tmp1 = FA / GA
      T_q1 = FAh / GAh
      T_ch = GAh * T_q1
      T_u1 = 134217729.0 * GAh
      T_p1 = T_u1
     +         - (T_u1 - GAh)
      T_p2 = GAh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (GAl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = FAh - T_ch
      T_r1 = T_s1 - FAh
      T_s2 = (FAh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + FAl
      T_q2 = (T_s1 + T_s2)
     +         / GAh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = FA / GA
            CALL dDLAMCH( 'EPS', tmp2h, tmp2l )
            IF(
!       (tmp1 .lt. tmp2)
     + ((tmp1h .lt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .lt. tmp2l)))
     $        ) THEN
*
*              Case of very large GA
*
               GASMAL = .FALSE.
!!               SSMAX = GA
      SSMAXh = GAh
      SSMAXl = GAl
!!               IF( HA.GT.ONE ) THEN
               IF(
!       (HA .gt. ONE)
     + ((HAh .gt. ONEh) .or.
     +  ((HAh .eq. ONEh) .and.
     +   (HAl .gt. ONEl)))
     $           ) THEN
!!                  SSMIN = FA / ( GA / HA )
!<<<  tmp1 = GA / HA
      T_q1 = GAh / HAh
      T_ch = HAh * T_q1
      T_u1 = 134217729.0 * HAh
      T_p1 = T_u1
     +         - (T_u1 - HAh)
      T_p2 = HAh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (HAl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = GAh - T_ch
      T_r1 = T_s1 - GAh
      T_s2 = (GAh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + GAl
      T_q2 = (T_s1 + T_s2)
     +         / HAh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = GA / HA
!<<<  SSMIN = FA / tmp1
      T_q1 = FAh / tmp1h
      T_ch = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp1l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = FAh - T_ch
      T_r1 = T_s1 - FAh
      T_s2 = (FAh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + FAl
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      SSMINh = T_q1 + T_q2
      SSMINl = T_q2
     +      - (SSMINh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SSMIN = FA / tmp1
               ELSE
!!                  SSMIN = ( FA / GA )*HA
!<<<  tmp1 = FA / GA
      T_q1 = FAh / GAh
      T_ch = GAh * T_q1
      T_u1 = 134217729.0 * GAh
      T_p1 = T_u1
     +         - (T_u1 - GAh)
      T_p2 = GAh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (GAl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = FAh - T_ch
      T_r1 = T_s1 - FAh
      T_s2 = (FAh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + FAl
      T_q2 = (T_s1 + T_s2)
     +         / GAh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = FA / GA
!<<<  SSMIN = tmp1 * HA
      T_ch = tmp1h * HAh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * HAh
      T_r1 = T_u1
     +         - (T_u1 - HAh)
      T_r2 = HAh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*HAl
     +         +  tmp1l*HAh)
      T_s1 = T_ch + T_cl
      SSMINl = T_cl - (T_s1 - T_ch)
      SSMINh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  SSMIN = tmp1 * HA
               END IF
!!               CLT = ONE
      CLTh = ONEh
      CLTl = ONEl
!!               SLT = HT / GT
!<<<  SLT = HT / GT
      T_q1 = HTh / GTh
      T_ch = GTh * T_q1
      T_u1 = 134217729.0 * GTh
      T_p1 = T_u1
     +         - (T_u1 - GTh)
      T_p2 = GTh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (GTl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = HTh - T_ch
      T_r1 = T_s1 - HTh
      T_s2 = (HTh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + HTl
      T_q2 = (T_s1 + T_s2)
     +         / GTh
      SLTh = T_q1 + T_q2
      SLTl = T_q2
     +      - (SLTh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SLT = HT / GT
!!               SRT = ONE
      SRTh = ONEh
      SRTl = ONEl
!!               CRT = FT / GT
!<<<  CRT = FT / GT
      T_q1 = FTh / GTh
      T_ch = GTh * T_q1
      T_u1 = 134217729.0 * GTh
      T_p1 = T_u1
     +         - (T_u1 - GTh)
      T_p2 = GTh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (GTl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = FTh - T_ch
      T_r1 = T_s1 - FTh
      T_s2 = (FTh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + FTl
      T_q2 = (T_s1 + T_s2)
     +         / GTh
      CRTh = T_q1 + T_q2
      CRTl = T_q2
     +      - (CRTh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  CRT = FT / GT
            END IF
         END IF
         IF( GASMAL ) THEN
*
*           Normal case
*
!!            D = FA - HA
!<<<  D = FA - HA
      T_p1 = FAh - HAh
      T_p2 = T_p1 - FAh
      T_ch = T_p1
      T_cl = (FAh - (T_p1 - T_p2))
     +         - (HAh + T_p2)
      T_cl = T_cl + FAl - HAl
      T_s1 = T_ch + T_cl
      Dl = T_cl - (T_s1 - T_ch)
      Dh = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  D = FA - HA
!!            IF( D.EQ.FA ) THEN
            IF(
!       (D .eq. FA)
     + ((Dh .eq. FAh) .and.
     +  (Dl .eq. FAl))
     $        ) THEN
*
*              Copes with infinite F or H
*
!!               L = ONE
      Lh = ONEh
      Ll = ONEl
            ELSE
!!               L = D / FA
!<<<  L = D / FA
      T_q1 = Dh / FAh
      T_ch = FAh * T_q1
      T_u1 = 134217729.0 * FAh
      T_p1 = T_u1
     +         - (T_u1 - FAh)
      T_p2 = FAh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (FAl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Dh - T_ch
      T_r1 = T_s1 - Dh
      T_s2 = (Dh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Dl
      T_q2 = (T_s1 + T_s2)
     +         / FAh
      Lh = T_q1 + T_q2
      Ll = T_q2
     +      - (Lh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  L = D / FA
            END IF
*
*           Note that 0 .le. L .le. 1
*
!!            M = GT / FT
!<<<  M = GT / FT
      T_q1 = GTh / FTh
      T_ch = FTh * T_q1
      T_u1 = 134217729.0 * FTh
      T_p1 = T_u1
     +         - (T_u1 - FTh)
      T_p2 = FTh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (FTl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = GTh - T_ch
      T_r1 = T_s1 - GTh
      T_s2 = (GTh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + GTl
      T_q2 = (T_s1 + T_s2)
     +         / FTh
      Mh = T_q1 + T_q2
      Ml = T_q2
     +      - (Mh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  M = GT / FT
*
*           Note that abs(M) .le. 1/macheps
*
!!            T = TWO - L
!<<<  T = TWO - L
      T_p1 = TWOh - Lh
      T_p2 = T_p1 - TWOh
      T_ch = T_p1
      T_cl = (TWOh - (T_p1 - T_p2))
     +         - (Lh + T_p2)
      T_cl = T_cl + TWOl - Ll
      T_s1 = T_ch + T_cl
      Tl = T_cl - (T_s1 - T_ch)
      Th = T_s1
#ifdef FPCOUNTER
#include "fselfsub.h"
#endif
!>>>  T = TWO - L
*
*           Note that T .ge. 1
*
!!            MM = M*M
!<<<  MM = M * M
      T_ch = Mh * Mh
      T_u1 = 134217729.0 * Mh
      T_p1 = T_u1
     +         - (T_u1 - Mh)
      T_p2 = Mh - T_p1
      T_u1 = 134217729.0 * Mh
      T_r1 = T_u1
     +         - (T_u1 - Mh)
      T_r2 = Mh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Mh*Ml
     +         +  Ml*Mh)
      T_s1 = T_ch + T_cl
      MMl = T_cl - (T_s1 - T_ch)
      MMh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  MM = M * M
!!            TT = T*T
!<<<  TT = T * T
      T_ch = Th * Th
      T_u1 = 134217729.0 * Th
      T_p1 = T_u1
     +         - (T_u1 - Th)
      T_p2 = Th - T_p1
      T_u1 = 134217729.0 * Th
      T_r1 = T_u1
     +         - (T_u1 - Th)
      T_r2 = Th - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Th*Tl
     +         +  Tl*Th)
      T_s1 = T_ch + T_cl
      TTl = T_cl - (T_s1 - T_ch)
      TTh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TT = T * T
!!            S = SQRT( TT+MM )
!<<<  tmp1 = TT + MM
      T_p1 = TTh + MMh
      T_p2 = T_p1 - TTh
      T_q1 = T_p1
      T_q2 = (TTh - (T_p1 - T_p2))
     +         + (MMh - T_p2)
      T_q2 = T_q2 + TTl + MMl
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = TT + MM
!<<<  S = sqrt(tmp1)
      if (tmp1h.eq.0.0D+0.and.tmp1l.eq.0.0D+0) then
        Sh = 0.0d+0
        Sl = 0.0d+0
      else
      if (tmp1h.lt.0.0d+0.or.
     +   (tmp1h.eq.0.0d+0.and.tmp1l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        Sh = 0.0d+0
        Sl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp1h)
      T_ax = tmp1h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp1h,tmp1l,T_z1,T_z2)
          T_p1 = tmp1h - T_z1
          T_p2 = T_p1 - tmp1h
          T_t2l = (tmp1h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp1l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          Sh = T_t1h
          Sl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  S = sqrt(tmp1)
*
*           Note that 1 .le. S .le. 1 + 1/macheps
*
!!            IF( L.EQ.ZERO ) THEN
            IF(
!       (L .eq. ZERO)
     + ((Lh .eq. ZEROh) .and.
     +  (Ll .eq. ZEROl))
     $        ) THEN
!!               R = ABS( M )
!<<<  R = abs(M)
      if (Mh .lt. 0.0d+0) then
        Rh = -Mh
        Rl = -Ml
      elseif (
     +  (Mh .eq. 0.0d+0).and.
     +  (Ml .lt. 0.0d+0)) then
        Rh = Mh
        Rl = -Ml
      else
        Rh = Mh
        Rl = Ml
      endif
!>>>  R = abs(M)
            ELSE
!!               R = SQRT( L*L+MM )
!<<<  tmp1 = L * L
      T_ch = Lh * Lh
      T_u1 = 134217729.0 * Lh
      T_p1 = T_u1
     +         - (T_u1 - Lh)
      T_p2 = Lh - T_p1
      T_u1 = 134217729.0 * Lh
      T_r1 = T_u1
     +         - (T_u1 - Lh)
      T_r2 = Lh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Lh*Ll
     +         +  Ll*Lh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = L * L
!<<<  tmp2 = tmp1 + MM
      T_p1 = tmp1h + MMh
      T_p2 = T_p1 - tmp1h
      T_q1 = T_p1
      T_q2 = (tmp1h - (T_p1 - T_p2))
     +         + (MMh - T_p2)
      T_q2 = T_q2 + tmp1l + MMl
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2 - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = tmp1 + MM
!<<<  R = sqrt(tmp2)
      if (tmp2h.eq.0.0D+0.and.tmp2l.eq.0.0D+0) then
        Rh = 0.0d+0
        Rl = 0.0d+0
      else
      if (tmp2h.lt.0.0d+0.or.
     +   (tmp2h.eq.0.0d+0.and.tmp2l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        Rh = 0.0d+0
        Rl = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp2h)
      T_ax = tmp2h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp2h,tmp2l,T_z1,T_z2)
          T_p1 = tmp2h - T_z1
          T_p2 = T_p1 - tmp2h
          T_t2l = (tmp2h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp2l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          Rh = T_t1h
          Rl = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  R = sqrt(tmp2)
            END IF
*
*           Note that 0 .le. R .le. 1 + 1/macheps
*
!!            A = HALF*( S+R )
!<<<  tmp1 = S + R
      T_p1 = Sh + Rh
      T_p2 = T_p1 - Sh
      T_q1 = T_p1
      T_q2 = (Sh - (T_p1 - T_p2))
     +         + (Rh - T_p2)
      T_q2 = T_q2 + Sl + Rl
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = S + R
!<<<  A = HALF * tmp1
      T_ch = HALFh * tmp1h
      T_u1 = 134217729.0 * HALFh
      T_p1 = T_u1
     +         - (T_u1 - HALFh)
      T_p2 = HALFh - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (HALFh*tmp1l
     +         +  HALFl*tmp1h)
      T_s1 = T_ch + T_cl
      Al = T_cl - (T_s1 - T_ch)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  A = HALF * tmp1
*
*           Note that 1 .le. A .le. 1 + abs(M)
*
!!            SSMIN = HA / A
!<<<  SSMIN = HA / A
      T_q1 = HAh / Ah
      T_ch = Ah * T_q1
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Al * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = HAh - T_ch
      T_r1 = T_s1 - HAh
      T_s2 = (HAh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + HAl
      T_q2 = (T_s1 + T_s2)
     +         / Ah
      SSMINh = T_q1 + T_q2
      SSMINl = T_q2
     +      - (SSMINh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SSMIN = HA / A
!!            SSMAX = FA*A
!<<<  SSMAX = FA * A
      T_ch = FAh * Ah
      T_u1 = 134217729.0 * FAh
      T_p1 = T_u1
     +         - (T_u1 - FAh)
      T_p2 = FAh - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (FAh*Al
     +         +  FAl*Ah)
      T_s1 = T_ch + T_cl
      SSMAXl = T_cl - (T_s1 - T_ch)
      SSMAXh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  SSMAX = FA * A
!!            IF( MM.EQ.ZERO ) THEN
            IF(
!       (MM .eq. ZERO)
     + ((MMh .eq. ZEROh) .and.
     +  (MMl .eq. ZEROl))
     $        ) THEN
*
*              Note that M is very tiny
*
!!               IF( L.EQ.ZERO ) THEN
               IF(
!       (L .eq. ZERO)
     + ((Lh .eq. ZEROh) .and.
     +  (Ll .eq. ZEROl))
     $           ) THEN
!!                  T = SIGN( TWO, FT )*SIGN( ONE, GT )
                  IF(
!       (FT .ge. 0.0)
     + ((FTh .gt. 0.0d+0) .or.
     +  ((FTh .eq. 0.0d+0) .and.
     +   (FTl .ge. 0.0d+0)))
     $              ) THEN
      tmp1h = TWOh
      tmp1l = TWOl
                  ELSE
!<<<  tmp1 = -TWO
      tmp1h = -TWOh
      tmp1l = -TWOl
!>>>  tmp1 = -TWO
                  END IF
                  IF(
!       (GT .ge. 0.0)
     + ((GTh .gt. 0.0d+0) .or.
     +  ((GTh .eq. 0.0d+0) .and.
     +   (GTl .ge. 0.0d+0)))
     $              ) THEN
      tmp2h = ONEh
      tmp2l = ONEl
                  ELSE
!<<<  tmp2 = -ONE
      tmp2h = -ONEh
      tmp2l = -ONEl
!>>>  tmp2 = -ONE
                  END IF
!<<<  T = tmp1 * tmp2
      T_ch = tmp1h * tmp2h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp2l
     +         +  tmp1l*tmp2h)
      T_s1 = T_ch + T_cl
      Tl = T_cl - (T_s1 - T_ch)
      Th = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  T = tmp1 * tmp2
               ELSE
!!                  T = GT / SIGN( D, FT ) + M / T
!<<<  tmp1 = abs(D)
      if (Dh .lt. 0.0d+0) then
        tmp1h = -Dh
        tmp1l = -Dl
      elseif (
     +  (Dh .eq. 0.0d+0).and.
     +  (Dl .lt. 0.0d+0)) then
        tmp1h = Dh
        tmp1l = -Dl
      else
        tmp1h = Dh
        tmp1l = Dl
      endif
!>>>  tmp1 = abs(D)
                  IF(
!       (FT .ge. 0.0)
     + ((FTh .gt. 0.0d+0) .or.
     +  ((FTh .eq. 0.0d+0) .and.
     +   (FTl .ge. 0.0d+0)))
     $              ) THEN
      tmp2h = tmp1h
      tmp2l = tmp1l
                  ELSE
!<<<  tmp2 = -tmp1
      tmp2h = -tmp1h
      tmp2l = -tmp1l
!>>>  tmp2 = -tmp1
                  END IF
!<<<  tmp3 = GT / tmp2
      T_q1 = GTh / tmp2h
      T_ch = tmp2h * T_q1
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp2l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = GTh - T_ch
      T_r1 = T_s1 - GTh
      T_s2 = (GTh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + GTl
      T_q2 = (T_s1 + T_s2)
     +         / tmp2h
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = GT / tmp2
!<<<  tmp4 = M / T
      T_q1 = Mh / Th
      T_ch = Th * T_q1
      T_u1 = 134217729.0 * Th
      T_p1 = T_u1
     +         - (T_u1 - Th)
      T_p2 = Th - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Tl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Mh - T_ch
      T_r1 = T_s1 - Mh
      T_s2 = (Mh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Ml
      T_q2 = (T_s1 + T_s2)
     +         / Th
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = M / T
!<<<  T = tmp3 + tmp4
      T_p1 = tmp3h + tmp4h
      T_p2 = T_p1 - tmp3h
      T_q1 = T_p1
      T_q2 = (tmp3h - (T_p1 - T_p2))
     +         + (tmp4h - T_p2)
      T_q2 = T_q2 + tmp3l + tmp4l
      Th = T_q1 + T_q2
      Tl = T_q2 - (Th - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  T = tmp3 + tmp4
               END IF
            ELSE
!!               T = ( M / ( S+T )+M / ( R+L ) )*( ONE+A )
!<<<  tmp1 = S + T
      T_p1 = Sh + Th
      T_p2 = T_p1 - Sh
      T_q1 = T_p1
      T_q2 = (Sh - (T_p1 - T_p2))
     +         + (Th - T_p2)
      T_q2 = T_q2 + Sl + Tl
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2 - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp1 = S + T
!<<<  tmp2 = M / tmp1
      T_q1 = Mh / tmp1h
      T_ch = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp1l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Mh - T_ch
      T_r1 = T_s1 - Mh
      T_s2 = (Mh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Ml
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2
     +      - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp2 = M / tmp1
!<<<  tmp3 = R + L
      T_p1 = Rh + Lh
      T_p2 = T_p1 - Rh
      T_q1 = T_p1
      T_q2 = (Rh - (T_p1 - T_p2))
     +         + (Lh - T_p2)
      T_q2 = T_q2 + Rl + Ll
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2 - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp3 = R + L
!<<<  tmp4 = M / tmp3
      T_q1 = Mh / tmp3h
      T_ch = tmp3h * T_q1
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (tmp3l * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Mh - T_ch
      T_r1 = T_s1 - Mh
      T_s2 = (Mh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Ml
      T_q2 = (T_s1 + T_s2)
     +         / tmp3h
      tmp4h = T_q1 + T_q2
      tmp4l = T_q2
     +      - (tmp4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp4 = M / tmp3
!<<<  tmp5 = tmp2 + tmp4
      T_p1 = tmp2h + tmp4h
      T_p2 = T_p1 - tmp2h
      T_q1 = T_p1
      T_q2 = (tmp2h - (T_p1 - T_p2))
     +         + (tmp4h - T_p2)
      T_q2 = T_q2 + tmp2l + tmp4l
      tmp5h = T_q1 + T_q2
      tmp5l = T_q2 - (tmp5h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp4
!<<<  tmp6 = ONE + A
      T_p1 = ONEh + Ah
      T_p2 = T_p1 - ONEh
      T_q1 = T_p1
      T_q2 = (ONEh - (T_p1 - T_p2))
     +         + (Ah - T_p2)
      T_q2 = T_q2 + ONEl + Al
      tmp6h = T_q1 + T_q2
      tmp6l = T_q2 - (tmp6h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp6 = ONE + A
!<<<  T = tmp5 * tmp6
      T_ch = tmp5h * tmp6h
      T_u1 = 134217729.0 * tmp5h
      T_p1 = T_u1
     +         - (T_u1 - tmp5h)
      T_p2 = tmp5h - T_p1
      T_u1 = 134217729.0 * tmp6h
      T_r1 = T_u1
     +         - (T_u1 - tmp6h)
      T_r2 = tmp6h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp5h*tmp6l
     +         +  tmp5l*tmp6h)
      T_s1 = T_ch + T_cl
      Tl = T_cl - (T_s1 - T_ch)
      Th = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  T = tmp5 * tmp6
            END IF
!!            L = SQRT( T*T+FOUR )
!<<<  tmp1 = T * T
      T_ch = Th * Th
      T_u1 = 134217729.0 * Th
      T_p1 = T_u1
     +         - (T_u1 - Th)
      T_p2 = Th - T_p1
      T_u1 = 134217729.0 * Th
      T_r1 = T_u1
     +         - (T_u1 - Th)
      T_r2 = Th - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (Th*Tl
     +         +  Tl*Th)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = T * T
!<<<  tmp2 = tmp1 + FOUR
      T_p1 = tmp1h + FOURh
      T_p2 = T_p1 - tmp1h
      T_q1 = T_p1
      T_q2 = (tmp1h - (T_p1 - T_p2))
     +         + (FOURh - T_p2)
      T_q2 = T_q2 + tmp1l + FOURl
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2 - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = tmp1 + FOUR
!<<<  L = sqrt(tmp2)
      if (tmp2h.eq.0.0D+0.and.tmp2l.eq.0.0D+0) then
        Lh = 0.0d+0
        Ll = 0.0d+0
      else
      if (tmp2h.lt.0.0d+0.or.
     +   (tmp2h.eq.0.0d+0.and.tmp2l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        Lh = 0.0d+0
        Ll = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp2h)
      T_ax = tmp2h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp2h,tmp2l,T_z1,T_z2)
          T_p1 = tmp2h - T_z1
          T_p2 = T_p1 - tmp2h
          T_t2l = (tmp2h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp2l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          Lh = T_t1h
          Ll = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
      endif
!>>>  L = sqrt(tmp2)
!!            CRT = TWO / L
!<<<  CRT = TWO / L
      T_q1 = TWOh / Lh
      T_ch = Lh * T_q1
      T_u1 = 134217729.0 * Lh
      T_p1 = T_u1
     +         - (T_u1 - Lh)
      T_p2 = Lh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Ll * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = TWOh - T_ch
      T_r1 = T_s1 - TWOh
      T_s2 = (TWOh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + TWOl
      T_q2 = (T_s1 + T_s2)
     +         / Lh
      CRTh = T_q1 + T_q2
      CRTl = T_q2
     +      - (CRTh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  CRT = TWO / L
!!            SRT = T / L
!<<<  SRT = T / L
      T_q1 = Th / Lh
      T_ch = Lh * T_q1
      T_u1 = 134217729.0 * Lh
      T_p1 = T_u1
     +         - (T_u1 - Lh)
      T_p2 = Lh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Ll * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Th - T_ch
      T_r1 = T_s1 - Th
      T_s2 = (Th
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Tl
      T_q2 = (T_s1 + T_s2)
     +         / Lh
      SRTh = T_q1 + T_q2
      SRTl = T_q2
     +      - (SRTh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SRT = T / L
!!            CLT = ( CRT+SRT*M ) / A
!<<<  tmp1 = SRT * M
      T_ch = SRTh * Mh
      T_u1 = 134217729.0 * SRTh
      T_p1 = T_u1
     +         - (T_u1 - SRTh)
      T_p2 = SRTh - T_p1
      T_u1 = 134217729.0 * Mh
      T_r1 = T_u1
     +         - (T_u1 - Mh)
      T_r2 = Mh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (SRTh*Ml
     +         +  SRTl*Mh)
      T_s1 = T_ch + T_cl
      tmp1l = T_cl - (T_s1 - T_ch)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp1 = SRT * M
!<<<  tmp2 = CRT + tmp1
      T_p1 = CRTh + tmp1h
      T_p2 = T_p1 - CRTh
      T_q1 = T_p1
      T_q2 = (CRTh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + CRTl + tmp1l
      tmp2h = T_q1 + T_q2
      tmp2l = T_q2 - (tmp2h - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp2 = CRT + tmp1
!<<<  CLT = tmp2 / A
      T_q1 = tmp2h / Ah
      T_ch = Ah * T_q1
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Al * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = tmp2h - T_ch
      T_r1 = T_s1 - tmp2h
      T_s2 = (tmp2h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + tmp2l
      T_q2 = (T_s1 + T_s2)
     +         / Ah
      CLTh = T_q1 + T_q2
      CLTl = T_q2
     +      - (CLTh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  CLT = tmp2 / A
!!            SLT = ( HT / FT )*SRT / A
!<<<  tmp1 = HT / FT
      T_q1 = HTh / FTh
      T_ch = FTh * T_q1
      T_u1 = 134217729.0 * FTh
      T_p1 = T_u1
     +         - (T_u1 - FTh)
      T_p2 = FTh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (FTl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = HTh - T_ch
      T_r1 = T_s1 - HTh
      T_s2 = (HTh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + HTl
      T_q2 = (T_s1 + T_s2)
     +         / FTh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = HT / FT
!<<<  tmp2 = tmp1 * SRT
      T_ch = tmp1h * SRTh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * SRTh
      T_r1 = T_u1
     +         - (T_u1 - SRTh)
      T_r2 = SRTh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*SRTl
     +         +  tmp1l*SRTh)
      T_s1 = T_ch + T_cl
      tmp2l = T_cl - (T_s1 - T_ch)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp2 = tmp1 * SRT
!<<<  SLT = tmp2 / A
      T_q1 = tmp2h / Ah
      T_ch = Ah * T_q1
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (Al * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = tmp2h - T_ch
      T_r1 = T_s1 - tmp2h
      T_s2 = (tmp2h
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + tmp2l
      T_q2 = (T_s1 + T_s2)
     +         / Ah
      SLTh = T_q1 + T_q2
      SLTl = T_q2
     +      - (SLTh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SLT = tmp2 / A
         END IF
      END IF
      IF( SWAP ) THEN
!!         CSL = SRT
      CSLh = SRTh
      CSLl = SRTl
!!         SNL = CRT
      SNLh = CRTh
      SNLl = CRTl
!!         CSR = SLT
      CSRh = SLTh
      CSRl = SLTl
!!         SNR = CLT
      SNRh = CLTh
      SNRl = CLTl
      ELSE
!!         CSL = CLT
      CSLh = CLTh
      CSLl = CLTl
!!         SNL = SLT
      SNLh = SLTh
      SNLl = SLTl
!!         CSR = CRT
      CSRh = CRTh
      CSRl = CRTl
!!         SNR = SRT
      SNRh = SRTh
      SNRl = SRTl
      END IF
*
*     Correct signs of SSMAX and SSMIN
*
!!      IF( PMAX.EQ.1 )
!!     $   TSIGN = SIGN( ONE, CSR )*SIGN( ONE, CSL )*SIGN( ONE, F )
      IF( PMAX.EQ.1 ) THEN
         IF(
!       (CSR .ge. 0.0)
     + ((CSRh .gt. 0.0d+0) .or.
     +  ((CSRh .eq. 0.0d+0) .and.
     +   (CSRl .ge. 0.0d+0)))
     $     ) THEN
      tmp1h = ONEh
      tmp1l = ONEl
         ELSE
!<<<  tmp1 = -ONE
      tmp1h = -ONEh
      tmp1l = -ONEl
!>>>  tmp1 = -ONE
         END IF
         IF(
!       (CSL .ge. 0.0)
     + ((CSLh .gt. 0.0d+0) .or.
     +  ((CSLh .eq. 0.0d+0) .and.
     +   (CSLl .ge. 0.0d+0)))
     $     ) THEN
      tmp2h = ONEh
      tmp2l = ONEl
         ELSE
!<<<  tmp2 = -ONE
      tmp2h = -ONEh
      tmp2l = -ONEl
!>>>  tmp2 = -ONE
         END IF
!<<<  tmp3 = tmp1 * tmp2
      T_ch = tmp1h * tmp2h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp2l
     +         +  tmp1l*tmp2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp1 * tmp2
         IF(
!       (F .ge. 0.0)
     + ((Fh .gt. 0.0d+0) .or.
     +  ((Fh .eq. 0.0d+0) .and.
     +   (Fl .ge. 0.0d+0)))
     $     ) THEN
      tmp4h = ONEh
      tmp4l = ONEl
         ELSE
!<<<  tmp4 = -ONE
      tmp4h = -ONEh
      tmp4l = -ONEl
!>>>  tmp4 = -ONE
         END IF
!<<<  TSIGN = tmp3 * tmp4
      T_ch = tmp3h * tmp4h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*tmp4l
     +         +  tmp3l*tmp4h)
      T_s1 = T_ch + T_cl
      TSIGNl = T_cl - (T_s1 - T_ch)
      TSIGNh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TSIGN = tmp3 * tmp4
      ENDIF
!!      IF( PMAX.EQ.2 )
!!     $   TSIGN = SIGN( ONE, SNR )*SIGN( ONE, CSL )*SIGN( ONE, G )
      IF( PMAX.EQ.2 ) THEN
         IF(
!       (SNR .ge. 0.0)
     + ((SNRh .gt. 0.0d+0) .or.
     +  ((SNRh .eq. 0.0d+0) .and.
     +   (SNRl .ge. 0.0d+0)))
     $     ) THEN
      tmp1h = ONEh
      tmp1l = ONEl
         ELSE
!<<<  tmp1 = -ONE
      tmp1h = -ONEh
      tmp1l = -ONEl
!>>>  tmp1 = -ONE
         END IF
         IF(
!       (CSL .ge. 0.0)
     + ((CSLh .gt. 0.0d+0) .or.
     +  ((CSLh .eq. 0.0d+0) .and.
     +   (CSLl .ge. 0.0d+0)))
     $     ) THEN
      tmp2h = ONEh
      tmp2l = ONEl
         ELSE
!<<<  tmp2 = -ONE
      tmp2h = -ONEh
      tmp2l = -ONEl
!>>>  tmp2 = -ONE
         END IF
!<<<  tmp3 = tmp1 * tmp2
      T_ch = tmp1h * tmp2h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp2l
     +         +  tmp1l*tmp2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp1 * tmp2
         IF(
!       (G .ge. 0.0)
     + ((Gh .gt. 0.0d+0) .or.
     +  ((Gh .eq. 0.0d+0) .and.
     +   (Gl .ge. 0.0d+0)))
     $     ) THEN
      tmp4h = ONEh
      tmp4l = ONEl
         ELSE
!<<<  tmp4 = -ONE
      tmp4h = -ONEh
      tmp4l = -ONEl
!>>>  tmp4 = -ONE
         END IF
!<<<  TSIGN = tmp3 * tmp4
      T_ch = tmp3h * tmp4h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*tmp4l
     +         +  tmp3l*tmp4h)
      T_s1 = T_ch + T_cl
      TSIGNl = T_cl - (T_s1 - T_ch)
      TSIGNh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TSIGN = tmp3 * tmp4
      END IF
!!      IF( PMAX.EQ.3 )
!!     $   TSIGN = SIGN( ONE, SNR )*SIGN( ONE, SNL )*SIGN( ONE, H )
      IF( PMAX.EQ.3 ) THEN
         IF(
!       (SNR .ge. 0.0)
     + ((SNRh .gt. 0.0d+0) .or.
     +  ((SNRh .eq. 0.0d+0) .and.
     +   (SNRl .ge. 0.0d+0)))
     $     ) THEN
      tmp1h = ONEh
      tmp1l = ONEl
         ELSE
!<<<  tmp1 = -ONE
      tmp1h = -ONEh
      tmp1l = -ONEl
!>>>  tmp1 = -ONE
         END IF
         IF(
!       (SNL .ge. 0.0)
     + ((SNLh .gt. 0.0d+0) .or.
     +  ((SNLh .eq. 0.0d+0) .and.
     +   (SNLl .ge. 0.0d+0)))
     $     ) THEN
      tmp2h = ONEh
      tmp2l = ONEl
         ELSE
!<<<  tmp2 = -ONE
      tmp2h = -ONEh
      tmp2l = -ONEl
!>>>  tmp2 = -ONE
         END IF
!<<<  tmp3 = tmp1 * tmp2
      T_ch = tmp1h * tmp2h
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp1h*tmp2l
     +         +  tmp1l*tmp2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = tmp1 * tmp2
         IF(
!       (H .ge. 0.0)
     + ((Hh .gt. 0.0d+0) .or.
     +  ((Hh .eq. 0.0d+0) .and.
     +   (Hl .ge. 0.0d+0)))
     $     ) THEN
      tmp4h = ONEh
      tmp4l = ONEl
         ELSE
!<<<  tmp4 = -ONE
      tmp4h = -ONEh
      tmp4l = -ONEl
!>>>  tmp4 = -ONE
         END IF
!<<<  TSIGN = tmp3 * tmp4
      T_ch = tmp3h * tmp4h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*tmp4l
     +         +  tmp3l*tmp4h)
      T_s1 = T_ch + T_cl
      TSIGNl = T_cl - (T_s1 - T_ch)
      TSIGNh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TSIGN = tmp3 * tmp4
      END IF
!!      SSMAX = SIGN( SSMAX, TSIGN )
!<<<  tmp1 = abs(SSMAX)
      if (SSMAXh .lt. 0.0d+0) then
        tmp1h = -SSMAXh
        tmp1l = -SSMAXl
      elseif (
     +  (SSMAXh .eq. 0.0d+0).and.
     +  (SSMAXl .lt. 0.0d+0)) then
        tmp1h = SSMAXh
        tmp1l = -SSMAXl
      else
        tmp1h = SSMAXh
        tmp1l = SSMAXl
      endif
!>>>  tmp1 = abs(SSMAX)
      IF(
!       (TSIGN .ge. 0.0)
     + ((TSIGNh .gt. 0.0d+0) .or.
     +  ((TSIGNh .eq. 0.0d+0) .and.
     +   (TSIGNl .ge. 0.0d+0)))
     $  ) THEN
      SSMAXh = tmp1h
      SSMAXl = tmp1l
      ELSE
!<<<  SSMAX = -tmp1
      SSMAXh = -tmp1h
      SSMAXl = -tmp1l
!>>>  SSMAX = -tmp1
      END IF

!!      SSMIN = SIGN( SSMIN, TSIGN*SIGN( ONE, F )*SIGN( ONE, H ) )
!<<<  tmp1 = abs(SSMIN)
      if (SSMINh .lt. 0.0d+0) then
        tmp1h = -SSMINh
        tmp1l = -SSMINl
      elseif (
     +  (SSMINh .eq. 0.0d+0).and.
     +  (SSMINl .lt. 0.0d+0)) then
        tmp1h = SSMINh
        tmp1l = -SSMINl
      else
        tmp1h = SSMINh
        tmp1l = SSMINl
      endif
!>>>  tmp1 = abs(SSMIN)
      IF(
!       (F .ge. 0.0)
     + ((Fh .gt. 0.0d+0) .or.
     +  ((Fh .eq. 0.0d+0) .and.
     +   (Fl .ge. 0.0d+0)))
     $  ) THEN
      tmp2h = ONEh
      tmp2l = ONEl
      ELSE
!<<<  tmp2 = -ONE
      tmp2h = -ONEh
      tmp2l = -ONEl
!>>>  tmp2 = -ONE
      END IF
!<<<  tmp3 = TSIGN * tmp2
      T_ch = TSIGNh * tmp2h
      T_u1 = 134217729.0 * TSIGNh
      T_p1 = T_u1
     +         - (T_u1 - TSIGNh)
      T_p2 = TSIGNh - T_p1
      T_u1 = 134217729.0 * tmp2h
      T_r1 = T_u1
     +         - (T_u1 - tmp2h)
      T_r2 = tmp2h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TSIGNh*tmp2l
     +         +  TSIGNl*tmp2h)
      T_s1 = T_ch + T_cl
      tmp3l = T_cl - (T_s1 - T_ch)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp3 = TSIGN * tmp2
      IF(
!       (H .ge. 0.0)
     + ((Hh .gt. 0.0d+0) .or.
     +  ((Hh .eq. 0.0d+0) .and.
     +   (Hl .ge. 0.0d+0)))
     $  ) THEN
      tmp4h = ONEh
      tmp4l = ONEl
      ELSE
!<<<  tmp4 = -ONE
      tmp4h = -ONEh
      tmp4l = -ONEl
!>>>  tmp4 = -ONE
      END IF
!<<<  tmp5 = tmp3 * tmp4
      T_ch = tmp3h * tmp4h
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * tmp4h
      T_r1 = T_u1
     +         - (T_u1 - tmp4h)
      T_r2 = tmp4h - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (tmp3h*tmp4l
     +         +  tmp3l*tmp4h)
      T_s1 = T_ch + T_cl
      tmp5l = T_cl - (T_s1 - T_ch)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  tmp5 = tmp3 * tmp4
      IF(
!       (tmp5 .ge. 0.0)
     + ((tmp5h .gt. 0.0d+0) .or.
     +  ((tmp5h .eq. 0.0d+0) .and.
     +   (tmp5l .ge. 0.0d+0)))
     $  ) THEN
      SSMINh = tmp1h
      SSMINl = tmp1l
      ELSE
!<<<  SSMIN = -tmp1
      SSMINh = -tmp1h
      SSMINl = -tmp1l
!>>>  SSMIN = -tmp1
      END IF
      RETURN
*
*     End of DLASV2
*
      END
