*> \brief \b DLAED6
*
*  =========== DOCUMENTATION ===========
*
* Online html documentation available at 
*            http://www.netlib.org/lapack/explore-html/ 
*
*> \htmlonly
*> Download DLAED6 + dependencies 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/dlaed6.f"> 
*> [TGZ]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/dlaed6.f"> 
*> [ZIP]</a> 
*> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/dlaed6.f"> 
*> [TXT]</a>
*> \endhtmlonly 
*
*  Definition:
*  ===========
*
*       SUBROUTINE DLAED6( KNITER, ORGATI, RHO, D, Z, FINIT, TAU, INFO )
* 
*       .. Scalar Arguments ..
*       LOGICAL            ORGATI
*       INTEGER            INFO, KNITER
*       DOUBLE PRECISION   FINIT, RHO, TAU
*       ..
*       .. Array Arguments ..
*       DOUBLE PRECISION   D( 3 ), Z( 3 )
*       ..
*  
*
*> \par Purpose:
*  =============
*>
*> \verbatim
*>
*> DLAED6 computes the positive or negative root (closest to the origin)
*> of
*>                  z(1)        z(2)        z(3)
*> f(x) =   rho + --------- + ---------- + ---------
*>                 d(1)-x      d(2)-x      d(3)-x
*>
*> It is assumed that
*>
*>       if ORGATI = .true. the root is between d(2) and d(3);
*>       otherwise it is between d(1) and d(2)
*>
*> This routine will be called by DLAED4 when necessary. In most cases,
*> the root sought is the smallest in magnitude, though it might not be
*> in some extremely rare situations.
*> \endverbatim
*
*  Arguments:
*  ==========
*
*> \param[in] KNITER
*> \verbatim
*>          KNITER is INTEGER
*>               Refer to DLAED4 for its significance.
*> \endverbatim
*>
*> \param[in] ORGATI
*> \verbatim
*>          ORGATI is LOGICAL
*>               If ORGATI is true, the needed root is between d(2) and
*>               d(3); otherwise it is between d(1) and d(2).  See
*>               DLAED4 for further details.
*> \endverbatim
*>
*> \param[in] RHO
*> \verbatim
*>          RHO is DOUBLE PRECISION
*>               Refer to the equation f(x) above.
*> \endverbatim
*>
*> \param[in] D
*> \verbatim
*>          D is DOUBLE PRECISION array, dimension (3)
*>               D satisfies d(1) < d(2) < d(3).
*> \endverbatim
*>
*> \param[in] Z
*> \verbatim
*>          Z is DOUBLE PRECISION array, dimension (3)
*>               Each of the elements in z must be positive.
*> \endverbatim
*>
*> \param[in] FINIT
*> \verbatim
*>          FINIT is DOUBLE PRECISION
*>               The value of f at 0. It is more accurate than the one
*>               evaluated inside this routine (if someone wants to do
*>               so).
*> \endverbatim
*>
*> \param[out] TAU
*> \verbatim
*>          TAU is DOUBLE PRECISION
*>               The root of the equation f(x).
*> \endverbatim
*>
*> \param[out] INFO
*> \verbatim
*>          INFO is INTEGER
*>               = 0: successful exit
*>               > 0: if INFO = 1, failure to converge
*> \endverbatim
*
*  Authors:
*  ========
*
*> \author Univ. of Tennessee 
*> \author Univ. of California Berkeley 
*> \author Univ. of Colorado Denver 
*> \author NAG Ltd. 
*
*> \date November 2011
*
*> \ingroup auxOTHERcomputational
*
*> \par Further Details:
*  =====================
*>
*> \verbatim
*>
*>  10/02/03: This version has a few statements commented out for thread
*>  safety (machine parameters are computed on each entry). SJH.
*>
*>  05/10/06: Modified from a new version of Ren-Cang Li, use
*>     Gragg-Thornton-Warner cubic convergent scheme for better stability.
*> \endverbatim
*
*> \par Contributors:
*  ==================
*>
*>     Ren-Cang Li, Computer Science Division, University of California
*>     at Berkeley, USA
*>
*  =====================================================================
!      SUBROUTINE DLAED6( KNITER, ORGATI, RHO, D, Z, FINIT, TAU, INFO )
      SUBROUTINE DDLAED6( KNITER, ORGATI, RHOh, RHOl, Dh, Dl, Zh, Zl, 
     $                    FINITh, FINITl, TAUh, TAUl, INFO )
*
*  -- LAPACK computational routine (version 3.4.0) --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*     November 2011
*
*     .. Scalar Arguments ..
      LOGICAL            ORGATI
      INTEGER            INFO, KNITER
!      DOUBLE PRECISION   FINIT, RHO, TAU
      double precision FINITh, FINITl
      double precision RHOh, RHOl
      double precision TAUh, TAUl
*     ..
*     .. Array Arguments ..
!      DOUBLE PRECISION   D( 3 ), Z( 3 )
      double precision Dh(3), Dl(3)
      double precision Zh(3), Zl(3)
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
      INTEGER            MAXIT
      PARAMETER          ( MAXIT = 40 )
!      DOUBLE PRECISION   ZERO, ONE, TWO, THREE, FOUR, EIGHT
!      PARAMETER          ( ZERO = 0.0D0, ONE = 1.0D0, TWO = 2.0D0,
!     $                   THREE = 3.0D0, FOUR = 4.0D0, EIGHT = 8.0D0 )
      double precision ZEROh, ZEROl
      double precision ONEh, ONEl
      double precision TWOh, TWOl
      double precision THREEh, THREEl
      double precision FOURh, FOURl
      double precision EIGHTh, EIGHTl
*     ..
*     .. External Functions ..
!      DOUBLE PRECISION   DLAMCH
!      EXTERNAL           DLAMCH
       EXTERNAL           DDLAMCH
*     ..
*     .. Local Arrays ..
!      DOUBLE PRECISION   DSCALE( 3 ), ZSCALE( 3 )
      double precision DSCALEh(3), DSCALEl(3)
      double precision ZSCALEh(3), ZSCALEl(3)
*     ..
*     .. Local Scalars ..
      LOGICAL            SCALE
      INTEGER            I, ITER, NITER
      INTEGER            Itemp
!      DOUBLE PRECISION   A, B, BASE, C, DDF, DF, EPS, ERRETM, ETA, F,
!     $                   FC, SCLFAC, SCLINV, SMALL1, SMALL2, SMINV1,
!     $                   SMINV2, TEMP, TEMP1, TEMP2, TEMP3, TEMP4, 
!     $                   LBD, UBD
      double precision Ah, Al
      double precision Bh, Bl
      double precision BASEh, BASEl
      double precision Ch, Cl
      double precision DDFh, DDFl
      double precision DFh, DFl
      double precision EPSh, EPSl
      double precision ERRETMh, ERRETMl
      double precision ETAh, ETAl
      double precision Fh, Fl
      double precision FCh, FCl
      double precision SCLFACh, SCLFACl
      double precision SCLINVh, SCLINVl
      double precision SMALL1h, SMALL1l
      double precision SMALL2h, SMALL2l
      double precision SMINV1h, SMINV1l
      double precision SMINV2h, SMINV2l
      double precision TEMPh, TEMPl
      double precision TEMP1h, TEMP1l
      double precision TEMP2h, TEMP2l
      double precision TEMP3h, TEMP3l
      double precision TEMP4h, TEMP4l
      double precision LBDh, LBDl
      double precision UBDh, UBDl

      double precision tmp1h, tmp1l
      double precision tmp2h, tmp2l
      double precision tmp3h, tmp3l
      double precision tmp4h, tmp4l
      double precision tmp5h, tmp5l
      double precision tmp6h, tmp6l
      double precision tmp7h, tmp7l
      double precision tmp8h, tmp8l
      double precision tmp9h, tmp9l
      double precision tmp10h, tmp10l
      double precision tmp11h, tmp11l
      double precision tmp12h, tmp12l
      double precision tmp13h, tmp13l
      double precision tmp14h, tmp14l
      double precision tmp15h, tmp15l
      double precision tmp16h, tmp16l
      double precision s0h, s0l
      double precision s1h, s1l
      double precision s2h, s2l
!     tmp vars
      double precision T_ch, T_cl
      double precision T_p1, T_p2
      double precision T_q1, T_q2
      double precision T_r1, T_r2
      double precision T_s1, T_s2
      double precision T_t1, T_t2
      double precision T_u1, T_u2
      double precision T_x1, T_x2
      double precision T_y1, T_y2
      double precision T_z1, T_z2
      double precision T_x, T_ax
      double precision T_t1h, T_t1l
      double precision T_t2h, T_t2l
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ABS, INT, LOG, MAX, MIN, SQRT
*     ..
*     .. Executable Statements ..
*
      ZEROh = 0.0d+0
      ZEROl = 0.0d+0
      ONEh = 1.0d+0
      ONEl = 0.0d+0
      TWOh = 2.0d0
      TWOl = 0.0d+0
      THREEh = 3.0d0
      THREEl = 0.0d+0
      FOURh = 4.0d0
      FOURl = 0.0d+0
      EIGHTh = 8.0d0
      EIGHTl = 0.0d+0
*
      INFO = 0
*
      IF( ORGATI ) THEN
!         LBD = D(2)
      LBDh = Dh(2)
      LBDl = Dl(2)
!         UBD = D(3)
      UBDh = Dh(3)
      UBDl = Dl(3)
      ELSE
!         LBD = D(1)
      LBDh = Dh(1)
      LBDl = Dl(1)
!         UBD = D(2)
      UBDh = Dh(2)
      UBDl = Dl(2)
      END IF
      IF(
!         FINIT .LT. ZERO
!       (FINIT .lt. ZERO)
     + ((FINITh .lt. ZEROh) .or.
     +  ((FINITh .eq. ZEROh) .and.
     +   (FINITl .lt. ZEROl)))
     $    )THEN
!         LBD = ZERO
      LBDh = ZEROh
      LBDl = ZEROl
      ELSE
!         UBD = ZERO 
      UBDh = ZEROh
      UBDl = ZEROl
      END IF
*
      NITER = 1
!      TAU = ZERO
      TAUh = ZEROh
      TAUl = ZEROl
      IF( KNITER.EQ.2 ) THEN
         IF( ORGATI ) THEN
!            TEMP = ( D( 3 )-D( 2 ) ) / TWO
!<<<  tmp1 = D(3) - D(2)
      T_p1 = Dh(3) - Dh(2)
      T_p2 = T_p1 - Dh(3)
      tmp1h = T_p1
      tmp1l = (Dh(3) - (T_p1 - T_p2))
     +         - (Dh(2) + T_p2)
      tmp1l = tmp1l + Dl(3) - Dl(2)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(3) - D(2)
!<<<  TEMP = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      TEMPh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (TWOl*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = tmp1h - TEMPh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = tmp1 / TWO
!            C = RHO + Z( 1 ) / ( ( D( 1 )-D( 2 ) )-TEMP )
!<<<  tmp1 = D(1) - D(2)
      T_p1 = Dh(1) - Dh(2)
      T_p2 = T_p1 - Dh(1)
      tmp1h = T_p1
      tmp1l = (Dh(1) - (T_p1 - T_p2))
     +         - (Dh(2) + T_p2)
      tmp1l = tmp1l + Dl(1) - Dl(2)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(1) - D(2)
!<<<  tmp2 = tmp1 - TEMP
      T_p1 = tmp1h - TEMPh
      T_p2 = T_p1 - tmp1h
      tmp2h = T_p1
      tmp2l = (tmp1h - (T_p1 - T_p2))
     +         - (TEMPh + T_p2)
      tmp2l = tmp2l + tmp1l - TEMPl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp2 = tmp1 - TEMP
!<<<  tmp3 = Z(1) / tmp2
      T_q1 = Zh(1) / tmp2h
      tmp3h = tmp2h * T_q1
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp3l = ((T_p1*T_r1
     +       - tmp3h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp3l = tmp3l + (tmp2l*T_q1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
      T_s1 = Zh(1) - tmp3h
      T_r1 = T_s1 - Zh(1)
      T_s2 = (Zh(1) 
     +         - (T_s1 - T_r1))
     +         - (tmp3h + T_r1)
      T_s2 = T_s2 - tmp3l
      T_s2 = T_s2 + Zl(1)
      T_q2 = (T_s1 + T_s2)
     +         / tmp2h
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = Z(1) / tmp2
!<<<  C = RHO + tmp3
      T_p1 = RHOh + tmp3h
      T_p2 = T_p1 - RHOh
      Ch = T_p1
      Cl = (RHOh - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      Cl = Cl + RHOl + tmp3l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  C = RHO + tmp3
!            A = C*( D( 2 )+D( 3 ) ) + Z( 2 ) + Z( 3 )
!<<<  tmp1 = D(2) + D(3)
      T_p1 = Dh(2) + Dh(3)
      T_p2 = T_p1 - Dh(2)
      tmp1h = T_p1
      tmp1l = (Dh(2) - (T_p1 - T_p2))
     +      + (Dh(3) - T_p2)
      tmp1l = tmp1l + Dl(2) + Dl(3)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = D(2) + D(3)
!<<<  tmp2 = C * tmp1
      tmp2h = Ch * tmp1h
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ch*tmp1l
     +      + Cl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = C * tmp1
!<<<  tmp3 = Z(2) + Z(3)
      T_p1 = Zh(2) + Zh(3)
      T_p2 = T_p1 - Zh(2)
      tmp3h = T_p1
      tmp3l = (Zh(2) - (T_p1 - T_p2))
     +      + (Zh(3) - T_p2)
      tmp3l = tmp3l + Zl(2) + Zl(3)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = Z(2) + Z(3)
!<<<  A = tmp2 + tmp3
      T_p1 = tmp2h + tmp3h
      T_p2 = T_p1 - tmp2h
      Ah = T_p1
      Al = (tmp2h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      Al = Al + tmp2l + tmp3l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A = tmp2 + tmp3
!            B = C*D( 2 )*D( 3 ) + Z( 2 )*D( 3 ) + Z( 3 )*D( 2 )
!<<<  tmp1 = C * D(2)
      tmp1h = Ch * Dh(2)
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * Dh(2)
      T_r1 = T_u1
     +         - (T_u1 - Dh(2))
      T_r2 = Dh(2) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Ch*Dl(2)
     +      + Cl*Dh(2))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = C * D(2)
!<<<  tmp2 = tmp1 * D(3)
      tmp2h = tmp1h * Dh(3)
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Dh(3)
      T_r1 = T_u1
     +         - (T_u1 - Dh(3))
      T_r2 = Dh(3) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Dl(3)
     +      + tmp1l*Dh(3))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * D(3)
!<<<  tmp3 = Z(2) * D(3)
      tmp3h = Zh(2) * Dh(3)
      T_u1 = 134217729.0 * Zh(2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(2))
      T_p2 = Zh(2) - T_p1
      T_u1 = 134217729.0 * Dh(3)
      T_r1 = T_u1
     +         - (T_u1 - Dh(3))
      T_r2 = Dh(3) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Zh(2)*Dl(3)
     +      + Zl(2)*Dh(3))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = Z(2) * D(3)
!<<<  tmp4 = Z(3) * D(2)
      tmp4h = Zh(3) * Dh(2)
      T_u1 = 134217729.0 * Zh(3)
      T_p1 = T_u1
     +         - (T_u1 - Zh(3))
      T_p2 = Zh(3) - T_p1
      T_u1 = 134217729.0 * Dh(2)
      T_r1 = T_u1
     +         - (T_u1 - Dh(2))
      T_r2 = Dh(2) - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (Zh(3)*Dl(2)
     +      + Zl(3)*Dh(2))
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = Z(3) * D(2)
!<<<  tmp5 = tmp2 + tmp3
      T_p1 = tmp2h + tmp3h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp3l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp3
!<<<  B = tmp5 + tmp4
      T_p1 = tmp5h + tmp4h
      T_p2 = T_p1 - tmp5h
      Bh = T_p1
      Bl = (tmp5h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Bl = Bl + tmp5l + tmp4l
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  B = tmp5 + tmp4
         ELSE
!            TEMP = ( D( 1 )-D( 2 ) ) / TWO
!<<<  tmp1 = D(1) - D(2)
      T_p1 = Dh(1) - Dh(2)
      T_p2 = T_p1 - Dh(1)
      tmp1h = T_p1
      tmp1l = (Dh(1) - (T_p1 - T_p2))
     +         - (Dh(2) + T_p2)
      tmp1l = tmp1l + Dl(1) - Dl(2)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(1) - D(2)
!<<<  TEMP = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      TEMPh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (TWOl*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = tmp1h - TEMPh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = tmp1 / TWO
!            C = RHO + Z( 3 ) / ( ( D( 3 )-D( 2 ) )-TEMP )
!<<<  tmp1 = D(3) - D(2)
      T_p1 = Dh(3) - Dh(2)
      T_p2 = T_p1 - Dh(3)
      tmp1h = T_p1
      tmp1l = (Dh(3) - (T_p1 - T_p2))
     +         - (Dh(2) + T_p2)
      tmp1l = tmp1l + Dl(3) - Dl(2)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(3) - D(2)
!<<<  tmp2 = tmp1 - TEMP
      T_p1 = tmp1h - TEMPh
      T_p2 = T_p1 - tmp1h
      tmp2h = T_p1
      tmp2l = (tmp1h - (T_p1 - T_p2))
     +         - (TEMPh + T_p2)
      tmp2l = tmp2l + tmp1l - TEMPl
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp2 = tmp1 - TEMP
!<<<  tmp3 = Z(3) / tmp2
      T_q1 = Zh(3) / tmp2h
      tmp3h = tmp2h * T_q1
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp3l = ((T_p1*T_r1
     +       - tmp3h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp3l = tmp3l + (tmp2l*T_q1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
      T_s1 = Zh(3) - tmp3h
      T_r1 = T_s1 - Zh(3)
      T_s2 = (Zh(3) 
     +         - (T_s1 - T_r1))
     +         - (tmp3h + T_r1)
      T_s2 = T_s2 - tmp3l
      T_s2 = T_s2 + Zl(3)
      T_q2 = (T_s1 + T_s2)
     +         / tmp2h
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = Z(3) / tmp2
!<<<  C = RHO + tmp3
      T_p1 = RHOh + tmp3h
      T_p2 = T_p1 - RHOh
      Ch = T_p1
      Cl = (RHOh - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      Cl = Cl + RHOl + tmp3l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  C = RHO + tmp3
!            A = C*( D( 1 )+D( 2 ) ) + Z( 1 ) + Z( 2 )
!<<<  tmp1 = D(1) + D(2)
      T_p1 = Dh(1) + Dh(2)
      T_p2 = T_p1 - Dh(1)
      tmp1h = T_p1
      tmp1l = (Dh(1) - (T_p1 - T_p2))
     +      + (Dh(2) - T_p2)
      tmp1l = tmp1l + Dl(1) + Dl(2)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = D(1) + D(2)
!<<<  tmp2 = C * tmp1
      tmp2h = Ch * tmp1h
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ch*tmp1l
     +      + Cl*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = C * tmp1
!<<<  tmp3 = tmp2 + Z(1)
      T_p1 = tmp2h + Zh(1)
      T_p2 = T_p1 - tmp2h
      tmp3h = T_p1
      tmp3l = (tmp2h - (T_p1 - T_p2))
     +      + (Zh(1) - T_p2)
      tmp3l = tmp3l + tmp2l + Zl(1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp3 = tmp2 + Z(1)
!<<<  A = tmp3 + Z(2)
      T_p1 = tmp3h + Zh(2)
      T_p2 = T_p1 - tmp3h
      Ah = T_p1
      Al = (tmp3h - (T_p1 - T_p2))
     +      + (Zh(2) - T_p2)
      Al = Al + tmp3l + Zl(2)
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  A = tmp3 + Z(2)
!            B = C*D( 1 )*D( 2 ) + Z( 1 )*D( 2 ) + Z( 2 )*D( 1 )
!<<<  tmp1 = C * D(1)
      tmp1h = Ch * Dh(1)
      T_u1 = 134217729.0 * Ch
      T_p1 = T_u1
     +         - (T_u1 - Ch)
      T_p2 = Ch - T_p1
      T_u1 = 134217729.0 * Dh(1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(1))
      T_r2 = Dh(1) - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Ch*Dl(1)
     +      + Cl*Dh(1))
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = C * D(1)
!<<<  tmp2 = tmp1 * D(2)
      tmp2h = tmp1h * Dh(2)
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Dh(2)
      T_r1 = T_u1
     +         - (T_u1 - Dh(2))
      T_r2 = Dh(2) - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Dl(2)
     +      + tmp1l*Dh(2))
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * D(2)
!<<<  tmp3 = Z(1) * D(2)
      tmp3h = Zh(1) * Dh(2)
      T_u1 = 134217729.0 * Zh(1)
      T_p1 = T_u1
     +         - (T_u1 - Zh(1))
      T_p2 = Zh(1) - T_p1
      T_u1 = 134217729.0 * Dh(2)
      T_r1 = T_u1
     +         - (T_u1 - Dh(2))
      T_r2 = Dh(2) - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (Zh(1)*Dl(2)
     +      + Zl(1)*Dh(2))
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = Z(1) * D(2)
!<<<  tmp4 = Z(2) * D(1)
      tmp4h = Zh(2) * Dh(1)
      T_u1 = 134217729.0 * Zh(2)
      T_p1 = T_u1
     +         - (T_u1 - Zh(2))
      T_p2 = Zh(2) - T_p1
      T_u1 = 134217729.0 * Dh(1)
      T_r1 = T_u1
     +         - (T_u1 - Dh(1))
      T_r2 = Dh(1) - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (Zh(2)*Dl(1)
     +      + Zl(2)*Dh(1))
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = Z(2) * D(1)
!<<<  tmp5 = tmp2 + tmp3
      T_p1 = tmp2h + tmp3h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +      + (tmp3h - T_p2)
      tmp5l = tmp5l + tmp2l + tmp3l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp2 + tmp3
!<<<  B = tmp5 + tmp4
      T_p1 = tmp5h + tmp4h
      T_p2 = T_p1 - tmp5h
      Bh = T_p1
      Bl = (tmp5h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Bl = Bl + tmp5l + tmp4l
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  B = tmp5 + tmp4
         END IF
!         TEMP = MAX( ABS( A ), ABS( B ), ABS( C ) )
!<<<  tmp1 = abs(A)
      if (Ah .lt. 0.0d+0) then
        tmp1h = -Ah
        tmp1l = -Al
      elseif (
     +  (Ah .eq. 0.0d+0).and.
     +  (Al .lt. 0.0d+0)) then
        tmp1h = Ah
        tmp1l = -Al
      else
        tmp1h = Ah
        tmp1l = Al
      endif
!>>>  tmp1 = abs(A)
!<<<  tmp2 = abs(B)
      if (Bh .lt. 0.0d+0) then
        tmp2h = -Bh
        tmp2l = -Bl
      elseif (
     +  (Bh .eq. 0.0d+0).and.
     +  (Bl .lt. 0.0d+0)) then
        tmp2h = Bh
        tmp2l = -Bl
      else
        tmp2h = Bh
        tmp2l = Bl
      endif
!>>>  tmp2 = abs(B)
!<<<  tmp3 = abs(C)
      if (Ch .lt. 0.0d+0) then
        tmp3h = -Ch
        tmp3l = -Cl
      elseif (
     +  (Ch .eq. 0.0d+0).and.
     +  (Cl .lt. 0.0d+0)) then
        tmp3h = Ch
        tmp3l = -Cl
      else
        tmp3h = Ch
        tmp3l = Cl
      endif
!>>>  tmp3 = abs(C)
         IF(
!       (tmp1 .ge. tmp2)
     + ((tmp1h .gt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .ge. tmp2l)))
     &      ) THEN
              IF(
!       (tmp1 .ge. tmp3)
     + ((tmp1h .gt. tmp3h) .or.
     +  ((tmp1h .eq. tmp3h) .and.
     +   (tmp1l .ge. tmp3l)))
     &           ) THEN
      TEMPh = tmp1h
      TEMPl = tmp1l
              ELSE
      TEMPh = tmp3h
      TEMPl = tmp3l
              END IF
         ELSE
              IF(
!       (tmp1 .ge. tmp3)
     + ((tmp1h .gt. tmp3h) .or.
     +  ((tmp1h .eq. tmp3h) .and.
     +   (tmp1l .ge. tmp3l)))
     &           ) THEN
      TEMPh = tmp2h
      TEMPl = tmp2l
              ELSE IF(
!       (tmp2 .ge. tmp3)
     + ((tmp2h .gt. tmp3h) .or.
     +  ((tmp2h .eq. tmp3h) .and.
     +   (tmp2l .ge. tmp3l)))
     &          ) THEN
      TEMPh = tmp2h
      TEMPl = tmp2l
              ELSE
      TEMPh = tmp3h
      TEMPl = tmp3l
              END IF
         END IF
!         A = A / TEMP
!<<<  A = A / TEMP
      T_q1 = Ah / TEMPh
      T_ch = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TEMPl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Ah - T_ch
      T_r1 = T_s1 - Ah
      T_s2 = (Ah
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Al
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      Ah = T_q1 + T_q2
      Al = T_q2
     +      - (Ah - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  A = A / TEMP
!         B = B / TEMP
!<<<  B = B / TEMP
      T_q1 = Bh / TEMPh
      T_ch = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TEMPl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Bh - T_ch
      T_r1 = T_s1 - Bh
      T_s2 = (Bh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Bl
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      Bh = T_q1 + T_q2
      Bl = T_q2
     +      - (Bh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  B = B / TEMP
!         C = C / TEMP
!<<<  C = C / TEMP
      T_q1 = Ch / TEMPh
      T_ch = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TEMPl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Ch - T_ch
      T_r1 = T_s1 - Ch
      T_s2 = (Ch
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Cl
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      Ch = T_q1 + T_q2
      Cl = T_q2
     +      - (Ch - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  C = C / TEMP
         IF(
!            C.EQ.ZERO
!       (C .eq. ZERO)
     + ((Ch .eq. ZEROh) .and.
     +  (Cl .eq. ZEROl))
     $       ) THEN
!            TAU = B / A
!<<<  TAU = B / A
      T_q1 = Bh / Ah
      TAUh = Ah * T_q1
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (Al*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = Bh - TAUh
      T_r1 = T_s1 - Bh
      T_s2 = (Bh 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + Bl
      T_q2 = (T_s1 + T_s2)
     +         / Ah
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = B / A
         ELSE IF(
!                 A.LE.ZERO
!       (A .le. ZERO)
     + ((Ah .lt. ZEROh) .or.
     +  ((Ah .eq. ZEROh) .and.
     +   (Al .le. ZEROl)))
     $            ) THEN
!            TAU = ( A-SQRT( ABS( A*A-FOUR*B*C ) ) ) / ( TWO*C )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = FOUR * B
      tmp2h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = FOUR * B
!<<<  tmp3 = tmp2 * C
      tmp3h = tmp2h * Ch
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (tmp2h*Cl
     +      + tmp2l*Ch)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = tmp2 * C
!<<<  tmp4 = A * A
      tmp4h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = A * A
!<<<  tmp5 = tmp4 - tmp3
      T_p1 = tmp4h - tmp3h
      T_p2 = T_p1 - tmp4h
      tmp5h = T_p1
      tmp5l = (tmp4h - (T_p1 - T_p2))
     +         - (tmp3h + T_p2)
      tmp5l = tmp5l + tmp4l - tmp3l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp5 = tmp4 - tmp3
!<<<  tmp6 = abs(tmp5)
      if (tmp5h .lt. 0.0d+0) then
        tmp6h = -tmp5h
        tmp6l = -tmp5l
      elseif (
     +  (tmp5h .eq. 0.0d+0).and.
     +  (tmp5l .lt. 0.0d+0)) then
        tmp6h = tmp5h
        tmp6l = -tmp5l
      else
        tmp6h = tmp5h
        tmp6l = tmp5l
      endif
!>>>  tmp6 = abs(tmp5)
!<<<  tmp7 = sqrt(tmp6)
      if (tmp6h.eq.0.0D+0.and.tmp6l.eq.0.0D+0) then
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
      endif
      if (tmp6h.lt.0.0d+0.or.
     +   (tmp6h.eq.0.0d+0.and.tmp6l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp6h)
      T_ax = tmp6h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp6h,tmp6l,T_z1,T_z2)
          T_p1 = tmp6h - T_z1
          T_p2 = T_p1 - tmp6h
          T_t2l = (tmp6h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp6l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp7h = T_t1h
          tmp7l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp7 = sqrt(tmp6)
!<<<  tmp8 = A - tmp7
      T_p1 = Ah - tmp7h
      T_p2 = T_p1 - Ah
      tmp8h = T_p1
      tmp8l = (Ah - (T_p1 - T_p2))
     +         - (tmp7h + T_p2)
      tmp8l = tmp8l + Al - tmp7l
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp8 = A - tmp7
!<<<  TAU = tmp8 / tmp1
      T_q1 = tmp8h / tmp1h
      TAUh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp1l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp8h - TAUh
      T_r1 = T_s1 - tmp8h
      T_s2 = (tmp8h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp8l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp8 / tmp1
         ELSE
!            TAU = TWO*B / ( A+SQRT( ABS( A*A-FOUR*B*C ) ) )
!<<<  tmp1 = TWO * B
      tmp1h = TWOh * Bh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Bl
     +      + TWOl*Bh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * B
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp4 = tmp3 * C
      tmp4h = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Cl
     +      + tmp3l*Ch)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * C
!<<<  tmp5 = tmp2 - tmp4
      T_p1 = tmp2h - tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp4h + T_p2)
      tmp5l = tmp5l + tmp2l - tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp5 = tmp2 - tmp4
!<<<  tmp6 = abs(tmp5)
      if (tmp5h .lt. 0.0d+0) then
        tmp6h = -tmp5h
        tmp6l = -tmp5l
      elseif (
     +  (tmp5h .eq. 0.0d+0).and.
     +  (tmp5l .lt. 0.0d+0)) then
        tmp6h = tmp5h
        tmp6l = -tmp5l
      else
        tmp6h = tmp5h
        tmp6l = tmp5l
      endif
!>>>  tmp6 = abs(tmp5)
!<<<  tmp7 = sqrt(tmp6)
      if (tmp6h.eq.0.0D+0.and.tmp6l.eq.0.0D+0) then
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
      endif
      if (tmp6h.lt.0.0d+0.or.
     +   (tmp6h.eq.0.0d+0.and.tmp6l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp6h)
      T_ax = tmp6h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp6h,tmp6l,T_z1,T_z2)
          T_p1 = tmp6h - T_z1
          T_p2 = T_p1 - tmp6h
          T_t2l = (tmp6h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp6l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp7h = T_t1h
          tmp7l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp7 = sqrt(tmp6)
!<<<  tmp8 = A + tmp7
      T_p1 = Ah + tmp7h
      T_p2 = T_p1 - Ah
      tmp8h = T_p1
      tmp8l = (Ah - (T_p1 - T_p2))
     +      + (tmp7h - T_p2)
      tmp8l = tmp8l + Al + tmp7l
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp8 = A + tmp7
!<<<  TAU = tmp1 / tmp8
      T_q1 = tmp1h / tmp8h
      TAUh = tmp8h * T_q1
      T_u1 = 134217729.0 * tmp8h
      T_p1 = T_u1
     +         - (T_u1 - tmp8h)
      T_p2 = tmp8h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (tmp8l*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp1h - TAUh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp8h
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp1 / tmp8
         END IF
         IF(
!            TAU .LT. LBD 
!       (TAU .lt. LBD)
     + ((TAUh .lt. LBDh) .or.
     +  ((TAUh .eq. LBDh) .and.
     +   (TAUl .lt. LBDl)))
     $      .OR. 
!            TAU .GT. UBD
!       (TAU .gt. UBD)
     + ((TAUh .gt. UBDh) .or.
     +  ((TAUh .eq. UBDh) .and.
     +   (TAUl .gt. UBDl)))
     $       )THEN
!           TAU = ( LBD+UBD )/TWO
!<<<  tmp1 = LBD + UBD
      T_p1 = LBDh + UBDh
      T_p2 = T_p1 - LBDh
      tmp1h = T_p1
      tmp1l = (LBDh - (T_p1 - T_p2))
     +      + (UBDh - T_p2)
      tmp1l = tmp1l + LBDl + UBDl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = LBD + UBD
!<<<  TAU = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      TAUh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (TWOl*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp1h - TAUh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp1 / TWO
         END IF
         IF(
!            D(1).EQ.TAU 
!       (D(1) .eq. TAU)
     + ((Dh(1) .eq. TAUh) .and.
     +  (Dl(1) .eq. TAUl))
     $      .OR. 
!            D(2).EQ.TAU 
!       (D(2) .eq. TAU)
     + ((Dh(2) .eq. TAUh) .and.
     +  (Dl(2) .eq. TAUl))
     $      .OR. 
!            D(3).EQ.TAU
!       (D(3) .eq. TAU)
     + ((Dh(3) .eq. TAUh) .and.
     +  (Dl(3) .eq. TAUl))
     $       ) THEN
!            TAU = ZERO
      TAUh = ZEROh
      TAUl = ZEROl
         ELSE
!            TEMP = FINIT + TAU*Z(1)/( D(1)*( D( 1 )-TAU ) ) +
!     $                     TAU*Z(2)/( D(2)*( D( 2 )-TAU ) ) +
!     $                     TAU*Z(3)/( D(3)*( D( 3 )-TAU ) )
!<<<  tmp1 = D(1) - TAU
      T_p1 = Dh(1) - TAUh
      T_p2 = T_p1 - Dh(1)
      tmp1h = T_p1
      tmp1l = (Dh(1) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + Dl(1) - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(1) - TAU
!<<<  tmp2 = D(1) * tmp1
      tmp2h = Dh(1) * tmp1h
      T_u1 = 134217729.0 * Dh(1)
      T_p1 = T_u1
     +         - (T_u1 - Dh(1))
      T_p2 = Dh(1) - T_p1
      T_u1 = 134217729.0 * tmp1h
      T_r1 = T_u1
     +         - (T_u1 - tmp1h)
      T_r2 = tmp1h - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Dh(1)*tmp1l
     +      + Dl(1)*tmp1h)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = D(1) * tmp1
!<<<  tmp3 = Z(1) / tmp2
      T_q1 = Zh(1) / tmp2h
      tmp3h = tmp2h * T_q1
      T_u1 = 134217729.0 * tmp2h
      T_p1 = T_u1
     +         - (T_u1 - tmp2h)
      T_p2 = tmp2h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp3l = ((T_p1*T_r1
     +       - tmp3h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp3l = tmp3l + (tmp2l*T_q1)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
      T_s1 = Zh(1) - tmp3h
      T_r1 = T_s1 - Zh(1)
      T_s2 = (Zh(1) 
     +         - (T_s1 - T_r1))
     +         - (tmp3h + T_r1)
      T_s2 = T_s2 - tmp3l
      T_s2 = T_s2 + Zl(1)
      T_q2 = (T_s1 + T_s2)
     +         / tmp2h
      tmp3h = T_q1 + T_q2
      tmp3l = T_q2
     +      - (tmp3h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp3 = Z(1) / tmp2
!<<<  tmp4 = TAU * tmp3
      tmp4h = TAUh * tmp3h
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * tmp3h
      T_r1 = T_u1
     +         - (T_u1 - tmp3h)
      T_r2 = tmp3h - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (TAUh*tmp3l
     +      + TAUl*tmp3h)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = TAU * tmp3
!<<<  tmp5 = D(2) - TAU
      T_p1 = Dh(2) - TAUh
      T_p2 = T_p1 - Dh(2)
      tmp5h = T_p1
      tmp5l = (Dh(2) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp5l = tmp5l + Dl(2) - TAUl
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp5 = D(2) - TAU
!<<<  tmp6 = D(2) * tmp5
      tmp6h = Dh(2) * tmp5h
      T_u1 = 134217729.0 * Dh(2)
      T_p1 = T_u1
     +         - (T_u1 - Dh(2))
      T_p2 = Dh(2) - T_p1
      T_u1 = 134217729.0 * tmp5h
      T_r1 = T_u1
     +         - (T_u1 - tmp5h)
      T_r2 = tmp5h - T_r1
      tmp6l = ((T_p1*T_r1
     +        - tmp6h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp6l = tmp6l
     +      + (Dh(2)*tmp5l
     +      + Dl(2)*tmp5h)
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp6 = D(2) * tmp5
!<<<  tmp7 = Z(2) / tmp6
      T_q1 = Zh(2) / tmp6h
      tmp7h = tmp6h * T_q1
      T_u1 = 134217729.0 * tmp6h
      T_p1 = T_u1
     +         - (T_u1 - tmp6h)
      T_p2 = tmp6h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp7l = ((T_p1*T_r1
     +       - tmp7h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp7l = tmp7l + (tmp6l*T_q1)
      T_s1 = tmp7h + tmp7l
      tmp7l = tmp7l - (T_s1 - tmp7h)
      tmp7h = T_s1
      T_s1 = Zh(2) - tmp7h
      T_r1 = T_s1 - Zh(2)
      T_s2 = (Zh(2) 
     +         - (T_s1 - T_r1))
     +         - (tmp7h + T_r1)
      T_s2 = T_s2 - tmp7l
      T_s2 = T_s2 + Zl(2)
      T_q2 = (T_s1 + T_s2)
     +         / tmp6h
      tmp7h = T_q1 + T_q2
      tmp7l = T_q2
     +      - (tmp7h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp7 = Z(2) / tmp6
!<<<  tmp8 = TAU * tmp7
      tmp8h = TAUh * tmp7h
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * tmp7h
      T_r1 = T_u1
     +         - (T_u1 - tmp7h)
      T_r2 = tmp7h - T_r1
      tmp8l = ((T_p1*T_r1
     +        - tmp8h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp8l = tmp8l
     +      + (TAUh*tmp7l
     +      + TAUl*tmp7h)
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp8 = TAU * tmp7
!<<<  tmp9 = D(3) - TAU
      T_p1 = Dh(3) - TAUh
      T_p2 = T_p1 - Dh(3)
      tmp9h = T_p1
      tmp9l = (Dh(3) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp9l = tmp9l + Dl(3) - TAUl
      T_s1 = tmp9h + tmp9l
      tmp9l = tmp9l - (T_s1 - tmp9h)
      tmp9h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp9 = D(3) - TAU
!<<<  tmp10 = D(3) * tmp9
      tmp10h = Dh(3) * tmp9h
      T_u1 = 134217729.0 * Dh(3)
      T_p1 = T_u1
     +         - (T_u1 - Dh(3))
      T_p2 = Dh(3) - T_p1
      T_u1 = 134217729.0 * tmp9h
      T_r1 = T_u1
     +         - (T_u1 - tmp9h)
      T_r2 = tmp9h - T_r1
      tmp10l = ((T_p1*T_r1
     +        - tmp10h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp10l = tmp10l
     +      + (Dh(3)*tmp9l
     +      + Dl(3)*tmp9h)
      T_s1 = tmp10h + tmp10l
      tmp10l = tmp10l - (T_s1 - tmp10h)
      tmp10h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp10 = D(3) * tmp9
!<<<  tmp11 = Z(3) / tmp10
      T_q1 = Zh(3) / tmp10h
      tmp11h = tmp10h * T_q1
      T_u1 = 134217729.0 * tmp10h
      T_p1 = T_u1
     +         - (T_u1 - tmp10h)
      T_p2 = tmp10h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp11l = ((T_p1*T_r1
     +       - tmp11h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp11l = tmp11l + (tmp10l*T_q1)
      T_s1 = tmp11h + tmp11l
      tmp11l = tmp11l - (T_s1 - tmp11h)
      tmp11h = T_s1
      T_s1 = Zh(3) - tmp11h
      T_r1 = T_s1 - Zh(3)
      T_s2 = (Zh(3) 
     +         - (T_s1 - T_r1))
     +         - (tmp11h + T_r1)
      T_s2 = T_s2 - tmp11l
      T_s2 = T_s2 + Zl(3)
      T_q2 = (T_s1 + T_s2)
     +         / tmp10h
      tmp11h = T_q1 + T_q2
      tmp11l = T_q2
     +      - (tmp11h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp11 = Z(3) / tmp10
!<<<  tmp12 = TAU * tmp11
      tmp12h = TAUh * tmp11h
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * tmp11h
      T_r1 = T_u1
     +         - (T_u1 - tmp11h)
      T_r2 = tmp11h - T_r1
      tmp12l = ((T_p1*T_r1
     +        - tmp12h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp12l = tmp12l
     +      + (TAUh*tmp11l
     +      + TAUl*tmp11h)
      T_s1 = tmp12h + tmp12l
      tmp12l = tmp12l - (T_s1 - tmp12h)
      tmp12h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp12 = TAU * tmp11
!<<<  tmp13 = FINIT + tmp4
      T_p1 = FINITh + tmp4h
      T_p2 = T_p1 - FINITh
      tmp13h = T_p1
      tmp13l = (FINITh - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp13l = tmp13l + FINITl + tmp4l
      T_s1 = tmp13h + tmp13l
      tmp13l = tmp13l - (T_s1 - tmp13h)
      tmp13h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp13 = FINIT + tmp4
!<<<  tmp14 = tmp13 + tmp8
      T_p1 = tmp13h + tmp8h
      T_p2 = T_p1 - tmp13h
      tmp14h = T_p1
      tmp14l = (tmp13h - (T_p1 - T_p2))
     +      + (tmp8h - T_p2)
      tmp14l = tmp14l + tmp13l + tmp8l
      T_s1 = tmp14h + tmp14l
      tmp14l = tmp14l - (T_s1 - tmp14h)
      tmp14h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp14 = tmp13 + tmp8
!<<<  TEMP = tmp14 + tmp12
      T_p1 = tmp14h + tmp12h
      T_p2 = T_p1 - tmp14h
      TEMPh = T_p1
      TEMPl = (tmp14h - (T_p1 - T_p2))
     +      + (tmp12h - T_p2)
      TEMPl = TEMPl + tmp14l + tmp12l
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  TEMP = tmp14 + tmp12
            IF(
!               TEMP .LE. ZERO
!       (TEMP .le. ZERO)
     + ((TEMPh .lt. ZEROh) .or.
     +  ((TEMPh .eq. ZEROh) .and.
     +   (TEMPl .le. ZEROl)))
     $          )THEN
!               LBD = TAU
      LBDh = TAUh
      LBDl = TAUl
            ELSE
!               UBD = TAU
      UBDh = TAUh
      UBDl = TAUl
            END IF
!<<<  tmp1 = abs(FINIT)
      if (FINITh .lt. 0.0d+0) then
        tmp1h = -FINITh
        tmp1l = -FINITl
      elseif (
     +  (FINITh .eq. 0.0d+0).and.
     +  (FINITl .lt. 0.0d+0)) then
        tmp1h = FINITh
        tmp1l = -FINITl
      else
        tmp1h = FINITh
        tmp1l = FINITl
      endif
!>>>  tmp1 = abs(FINIT)
!<<<  tmp2 = abs(TEMP)
      if (TEMPh .lt. 0.0d+0) then
        tmp2h = -TEMPh
        tmp2l = -TEMPl
      elseif (
     +  (TEMPh .eq. 0.0d+0).and.
     +  (TEMPl .lt. 0.0d+0)) then
        tmp2h = TEMPh
        tmp2l = -TEMPl
      else
        tmp2h = TEMPh
        tmp2l = TEMPl
      endif
!>>>  tmp2 = abs(TEMP)
            IF(
!               ABS( FINIT ).LE.ABS( TEMP )
!       (tmp1 .le. tmp2)
     + ((tmp1h .lt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .le. tmp2l)))
     $          ) THEN
!               TAU = ZERO
      TAUh = ZEROh
      TAUl = ZEROl
         END IF
      END IF
      END iF
*
*     get machine parameters for possible scaling to avoid overflow
*
*     modified by Sven: parameters SMALL1, SMINV1, SMALL2,
*     SMINV2, EPS are not SAVEd anymore between one call to the
*     others but recomputed at each call
*
!      EPS = DLAMCH( 'Epsilon' )
      CALL DDLAMCH('Epsilon',EPSh,EPSl)
!      BASE = DLAMCH( 'Base' )
      CALL DDLAMCH('Base',BASEh,BASEl)
!      SMALL1 = BASE**( INT( LOG( DLAMCH( 'SafMin' ) ) / LOG( BASE ) /
!     $         THREE ) )
      CALL DDLAMCH('SafMin',tmp1h,tmp1l)
      SMALL1h = BASEh**( INT( LOG( tmp1h ) / LOG( BASEh ) / THREEh ) )
      SMALL1l = 0.0d0

!      SMINV1 = ONE / SMALL1
!<<<  SMINV1 = ONE / SMALL1
      T_q1 = ONEh / SMALL1h
      SMINV1h = SMALL1h * T_q1
      T_u1 = 134217729.0 * SMALL1h
      T_p1 = T_u1
     +         - (T_u1 - SMALL1h)
      T_p2 = SMALL1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      SMINV1l = ((T_p1*T_r1
     +       - SMINV1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      SMINV1l = SMINV1l + (SMALL1l*T_q1)
      T_s1 = SMINV1h + SMINV1l
      SMINV1l = SMINV1l - (T_s1 - SMINV1h)
      SMINV1h = T_s1
      T_s1 = ONEh - SMINV1h
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh 
     +         - (T_s1 - T_r1))
     +         - (SMINV1h + T_r1)
      T_s2 = T_s2 - SMINV1l
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / SMALL1h
      SMINV1h = T_q1 + T_q2
      SMINV1l = T_q2
     +      - (SMINV1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  SMINV1 = ONE / SMALL1
!      SMALL2 = SMALL1*SMALL1
!<<<  SMALL2 = SMALL1 * SMALL1
      SMALL2h = SMALL1h * SMALL1h
      T_u1 = 134217729.0 * SMALL1h
      T_p1 = T_u1
     +         - (T_u1 - SMALL1h)
      T_p2 = SMALL1h - T_p1
      T_u1 = 134217729.0 * SMALL1h
      T_r1 = T_u1
     +         - (T_u1 - SMALL1h)
      T_r2 = SMALL1h - T_r1
      SMALL2l = ((T_p1*T_r1
     +        - SMALL2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      SMALL2l = SMALL2l
     +      + (SMALL1h*SMALL1l
     +      + SMALL1l*SMALL1h)
      T_s1 = SMALL2h + SMALL2l
      SMALL2l = SMALL2l - (T_s1 - SMALL2h)
      SMALL2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  SMALL2 = SMALL1 * SMALL1
!      SMINV2 = SMINV1*SMINV1
!<<<  SMINV2 = SMINV1 * SMINV1
      SMINV2h = SMINV1h * SMINV1h
      T_u1 = 134217729.0 * SMINV1h
      T_p1 = T_u1
     +         - (T_u1 - SMINV1h)
      T_p2 = SMINV1h - T_p1
      T_u1 = 134217729.0 * SMINV1h
      T_r1 = T_u1
     +         - (T_u1 - SMINV1h)
      T_r2 = SMINV1h - T_r1
      SMINV2l = ((T_p1*T_r1
     +        - SMINV2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      SMINV2l = SMINV2l
     +      + (SMINV1h*SMINV1l
     +      + SMINV1l*SMINV1h)
      T_s1 = SMINV2h + SMINV2l
      SMINV2l = SMINV2l - (T_s1 - SMINV2h)
      SMINV2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  SMINV2 = SMINV1 * SMINV1
*
*     Determine if scaling of inputs necessary to avoid overflow
*     when computing 1/TEMP**3
*
      IF( ORGATI ) THEN
!         TEMP = MIN( ABS( D( 2 )-TAU ), ABS( D( 3 )-TAU ) )
!<<<  tmp1 = D(2) - TAU
      T_p1 = Dh(2) - TAUh
      T_p2 = T_p1 - Dh(2)
      tmp1h = T_p1
      tmp1l = (Dh(2) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + Dl(2) - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(2) - TAU
!<<<  tmp2 = abs(tmp1)
      if (tmp1h .lt. 0.0d+0) then
        tmp2h = -tmp1h
        tmp2l = -tmp1l
      elseif (
     +  (tmp1h .eq. 0.0d+0).and.
     +  (tmp1l .lt. 0.0d+0)) then
        tmp2h = tmp1h
        tmp2l = -tmp1l
      else
        tmp2h = tmp1h
        tmp2l = tmp1l
      endif
!>>>  tmp2 = abs(tmp1)
!<<<  tmp3 = D(3) - TAU
      T_p1 = Dh(3) - TAUh
      T_p2 = T_p1 - Dh(3)
      tmp3h = T_p1
      tmp3l = (Dh(3) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp3l = tmp3l + Dl(3) - TAUl
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp3 = D(3) - TAU
!<<<  tmp4 = abs(tmp3)
      if (tmp3h .lt. 0.0d+0) then
        tmp4h = -tmp3h
        tmp4l = -tmp3l
      elseif (
     +  (tmp3h .eq. 0.0d+0).and.
     +  (tmp3l .lt. 0.0d+0)) then
        tmp4h = tmp3h
        tmp4l = -tmp3l
      else
        tmp4h = tmp3h
        tmp4l = tmp3l
      endif
!>>>  tmp4 = abs(tmp3)
          IF(
!       (tmp2 .le. tmp4)
     + ((tmp2h .lt. tmp4h) .or.
     +  ((tmp2h .eq. tmp4h) .and.
     +   (tmp2l .le. tmp4l)))
     $      )THEN
      TEMPh = tmp2h
      TEMPl = tmp2l
          ELSE
      TEMPh = tmp4h
      TEMPl = tmp4l
          END IF
      ELSE
!         TEMP = MIN( ABS( D( 1 )-TAU ), ABS( D( 2 )-TAU ) )
!<<<  tmp1 = D(1) - TAU
      T_p1 = Dh(1) - TAUh
      T_p2 = T_p1 - Dh(1)
      tmp1h = T_p1
      tmp1l = (Dh(1) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + Dl(1) - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = D(1) - TAU
!<<<  tmp2 = abs(tmp1)
      if (tmp1h .lt. 0.0d+0) then
        tmp2h = -tmp1h
        tmp2l = -tmp1l
      elseif (
     +  (tmp1h .eq. 0.0d+0).and.
     +  (tmp1l .lt. 0.0d+0)) then
        tmp2h = tmp1h
        tmp2l = -tmp1l
      else
        tmp2h = tmp1h
        tmp2l = tmp1l
      endif
!>>>  tmp2 = abs(tmp1)
!<<<  tmp3 = D(2) - TAU
      T_p1 = Dh(2) - TAUh
      T_p2 = T_p1 - Dh(2)
      tmp3h = T_p1
      tmp3l = (Dh(2) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp3l = tmp3l + Dl(2) - TAUl
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp3 = D(2) - TAU
!<<<  tmp4 = abs(tmp3)
      if (tmp3h .lt. 0.0d+0) then
        tmp4h = -tmp3h
        tmp4l = -tmp3l
      elseif (
     +  (tmp3h .eq. 0.0d+0).and.
     +  (tmp3l .lt. 0.0d+0)) then
        tmp4h = tmp3h
        tmp4l = -tmp3l
      else
        tmp4h = tmp3h
        tmp4l = tmp3l
      endif
!>>>  tmp4 = abs(tmp3)
          IF(
!       (tmp2 .le. tmp4)
     + ((tmp2h .lt. tmp4h) .or.
     +  ((tmp2h .eq. tmp4h) .and.
     +   (tmp2l .le. tmp4l)))
     $      )THEN
      TEMPh = tmp2h
      TEMPl = tmp2l
          ELSE
      TEMPh = tmp4h
      TEMPl = tmp4l
          END IF
      END IF
      SCALE = .FALSE.
      IF(
!         TEMP.LE.SMALL1
!       (TEMP .le. SMALL1)
     + ((TEMPh .lt. SMALL1h) .or.
     +  ((TEMPh .eq. SMALL1h) .and.
     +   (TEMPl .le. SMALL1l)))
     $    ) THEN
         SCALE = .TRUE.
         IF(
!            TEMP.LE.SMALL2
!       (TEMP .le. SMALL2)
     + ((TEMPh .lt. SMALL2h) .or.
     +  ((TEMPh .eq. SMALL2h) .and.
     +   (TEMPl .le. SMALL2l)))
     $       ) THEN
*
*        Scale up by power of radix nearest 1/SAFMIN**(2/3)
*
!            SCLFAC = SMINV2
      SCLFACh = SMINV2h
      SCLFACl = SMINV2l
!            SCLINV = SMALL2
      SCLINVh = SMALL2h
      SCLINVl = SMALL2l
         ELSE
*
*        Scale up by power of radix nearest 1/SAFMIN**(1/3)
*
!            SCLFAC = SMINV1
      SCLFACh = SMINV1h
      SCLFACl = SMINV1l
!            SCLINV = SMALL1
      SCLINVh = SMALL1h
      SCLINVl = SMALL1l
         END IF
*
*        Scaling up safe because D, Z, TAU scaled elsewhere to be O(1)
*
         DO 10 I = 1, 3
!            DSCALE( I ) = D( I )*SCLFAC
!<<<  DSCALE(I) = D(I) * SCLFAC
      DSCALEh(I) = Dh(I) * SCLFACh
      T_u1 = 134217729.0 * Dh(I)
      T_p1 = T_u1
     +         - (T_u1 - Dh(I))
      T_p2 = Dh(I) - T_p1
      T_u1 = 134217729.0 * SCLFACh
      T_r1 = T_u1
     +         - (T_u1 - SCLFACh)
      T_r2 = SCLFACh - T_r1
      DSCALEl(I) = ((T_p1*T_r1
     +        - DSCALEh(I))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      DSCALEl(I) = DSCALEl(I)
     +      + (Dh(I)*SCLFACl
     +      + Dl(I)*SCLFACh)
      T_s1 = DSCALEh(I) + DSCALEl(I)
      DSCALEl(I) = DSCALEl(I) - (T_s1 - DSCALEh(I))
      DSCALEh(I) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  DSCALE(I) = D(I) * SCLFAC
!            ZSCALE( I ) = Z( I )*SCLFAC
!<<<  ZSCALE(I) = Z(I) * SCLFAC
      ZSCALEh(I) = Zh(I) * SCLFACh
      T_u1 = 134217729.0 * Zh(I)
      T_p1 = T_u1
     +         - (T_u1 - Zh(I))
      T_p2 = Zh(I) - T_p1
      T_u1 = 134217729.0 * SCLFACh
      T_r1 = T_u1
     +         - (T_u1 - SCLFACh)
      T_r2 = SCLFACh - T_r1
      ZSCALEl(I) = ((T_p1*T_r1
     +        - ZSCALEh(I))
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      ZSCALEl(I) = ZSCALEl(I)
     +      + (Zh(I)*SCLFACl
     +      + Zl(I)*SCLFACh)
      T_s1 = ZSCALEh(I) + ZSCALEl(I)
      ZSCALEl(I) = ZSCALEl(I) - (T_s1 - ZSCALEh(I))
      ZSCALEh(I) = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  ZSCALE(I) = Z(I) * SCLFAC
   10    CONTINUE
!         TAU = TAU*SCLFAC
!<<<  TAU = TAU * SCLFAC
      T_ch = TAUh * SCLFACh
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * SCLFACh
      T_r1 = T_u1
     +         - (T_u1 - SCLFACh)
      T_r2 = SCLFACh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TAUh*SCLFACl
     +         +  TAUl*SCLFACh)
      T_s1 = T_ch + T_cl
      TAUl = T_cl - (T_s1 - T_ch)
      TAUh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TAU = TAU * SCLFAC
!         LBD = LBD*SCLFAC
!<<<  LBD = LBD * SCLFAC
      T_ch = LBDh * SCLFACh
      T_u1 = 134217729.0 * LBDh
      T_p1 = T_u1
     +         - (T_u1 - LBDh)
      T_p2 = LBDh - T_p1
      T_u1 = 134217729.0 * SCLFACh
      T_r1 = T_u1
     +         - (T_u1 - SCLFACh)
      T_r2 = SCLFACh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (LBDh*SCLFACl
     +         +  LBDl*SCLFACh)
      T_s1 = T_ch + T_cl
      LBDl = T_cl - (T_s1 - T_ch)
      LBDh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  LBD = LBD * SCLFAC
!         UBD = UBD*SCLFAC
!<<<  UBD = UBD * SCLFAC
      T_ch = UBDh * SCLFACh
      T_u1 = 134217729.0 * UBDh
      T_p1 = T_u1
     +         - (T_u1 - UBDh)
      T_p2 = UBDh - T_p1
      T_u1 = 134217729.0 * SCLFACh
      T_r1 = T_u1
     +         - (T_u1 - SCLFACh)
      T_r2 = SCLFACh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (UBDh*SCLFACl
     +         +  UBDl*SCLFACh)
      T_s1 = T_ch + T_cl
      UBDl = T_cl - (T_s1 - T_ch)
      UBDh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  UBD = UBD * SCLFAC
      ELSE
*
*        Copy D and Z to DSCALE and ZSCALE
*
         DO 20 I = 1, 3
!            DSCALE( I ) = D( I )
      DSCALEh(I) = Dh(I)
      DSCALEl(I) = Dl(I)
!            ZSCALE( I ) = Z( I )
      ZSCALEh(I) = Zh(I)
      ZSCALEl(I) = Zl(I)
   20    CONTINUE
      END IF
*
!      FC = ZERO
      FCh = ZEROh
      FCl = ZEROl
!      DF = ZERO
      DFh = ZEROh
      DFl = ZEROl
!      DDF = ZERO
      DDFh = ZEROh
      DDFl = ZEROl
      DO 30 I = 1, 3
!         TEMP = ONE / ( DSCALE( I )-TAU )
!<<<  tmp1 = DSCALE(I) - TAU
      T_p1 = DSCALEh(I) - TAUh
      T_p2 = T_p1 - DSCALEh(I)
      tmp1h = T_p1
      tmp1l = (DSCALEh(I) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + DSCALEl(I) - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DSCALE(I) - TAU
!<<<  TEMP = ONE / tmp1
      T_q1 = ONEh / tmp1h
      TEMPh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (tmp1l*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = ONEh - TEMPh
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = ONE / tmp1
!         TEMP1 = ZSCALE( I )*TEMP
!<<<  TEMP1 = ZSCALE(I) * TEMP
      TEMP1h = ZSCALEh(I) * TEMPh
      T_u1 = 134217729.0 * ZSCALEh(I)
      T_p1 = T_u1
     +         - (T_u1 - ZSCALEh(I))
      T_p2 = ZSCALEh(I) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      TEMP1l = ((T_p1*T_r1
     +        - TEMP1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      TEMP1l = TEMP1l
     +      + (ZSCALEh(I)*TEMPl
     +      + ZSCALEl(I)*TEMPh)
      T_s1 = TEMP1h + TEMP1l
      TEMP1l = TEMP1l - (T_s1 - TEMP1h)
      TEMP1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  TEMP1 = ZSCALE(I) * TEMP
!         TEMP2 = TEMP1*TEMP
!<<<  TEMP2 = TEMP1 * TEMP
      TEMP2h = TEMP1h * TEMPh
      T_u1 = 134217729.0 * TEMP1h
      T_p1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_p2 = TEMP1h - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      TEMP2l = ((T_p1*T_r1
     +        - TEMP2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      TEMP2l = TEMP2l
     +      + (TEMP1h*TEMPl
     +      + TEMP1l*TEMPh)
      T_s1 = TEMP2h + TEMP2l
      TEMP2l = TEMP2l - (T_s1 - TEMP2h)
      TEMP2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  TEMP2 = TEMP1 * TEMP
!         TEMP3 = TEMP2*TEMP
!<<<  TEMP3 = TEMP2 * TEMP
      TEMP3h = TEMP2h * TEMPh
      T_u1 = 134217729.0 * TEMP2h
      T_p1 = T_u1
     +         - (T_u1 - TEMP2h)
      T_p2 = TEMP2h - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      TEMP3l = ((T_p1*T_r1
     +        - TEMP3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      TEMP3l = TEMP3l
     +      + (TEMP2h*TEMPl
     +      + TEMP2l*TEMPh)
      T_s1 = TEMP3h + TEMP3l
      TEMP3l = TEMP3l - (T_s1 - TEMP3h)
      TEMP3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  TEMP3 = TEMP2 * TEMP
!         FC = FC + TEMP1 / DSCALE( I )
!<<<  tmp1 = TEMP1 / DSCALE(I)
      T_q1 = TEMP1h / DSCALEh(I)
      tmp1h = DSCALEh(I) * T_q1
      T_u1 = 134217729.0 * DSCALEh(I)
      T_p1 = T_u1
     +         - (T_u1 - DSCALEh(I))
      T_p2 = DSCALEh(I) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp1l = ((T_p1*T_r1
     +       - tmp1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp1l = tmp1l + (DSCALEl(I)*T_q1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
      T_s1 = TEMP1h - tmp1h
      T_r1 = T_s1 - TEMP1h
      T_s2 = (TEMP1h 
     +         - (T_s1 - T_r1))
     +         - (tmp1h + T_r1)
      T_s2 = T_s2 - tmp1l
      T_s2 = T_s2 + TEMP1l
      T_q2 = (T_s1 + T_s2)
     +         / DSCALEh(I)
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = TEMP1 / DSCALE(I)
!<<<  FC = FC + tmp1
      T_p1 = FCh + tmp1h
      T_p2 = T_p1 - FCh
      T_q1 = T_p1
      T_q2 = (FCh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + FCl + tmp1l
      FCh = T_q1 + T_q2
      FCl = T_q2 - (FCh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  FC = FC + tmp1
!         DF = DF + TEMP2
!<<<  DF = DF + TEMP2
      T_p1 = DFh + TEMP2h
      T_p2 = T_p1 - DFh
      T_q1 = T_p1
      T_q2 = (DFh - (T_p1 - T_p2))
     +         + (TEMP2h - T_p2)
      T_q2 = T_q2 + DFl + TEMP2l
      DFh = T_q1 + T_q2
      DFl = T_q2 - (DFh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DF = DF + TEMP2
!         DDF = DDF + TEMP3
!<<<  DDF = DDF + TEMP3
      T_p1 = DDFh + TEMP3h
      T_p2 = T_p1 - DDFh
      T_q1 = T_p1
      T_q2 = (DDFh - (T_p1 - T_p2))
     +         + (TEMP3h - T_p2)
      T_q2 = T_q2 + DDFl + TEMP3l
      DDFh = T_q1 + T_q2
      DDFl = T_q2 - (DDFh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DDF = DDF + TEMP3
   30 CONTINUE
!      F = FINIT + TAU*FC
!<<<  tmp1 = TAU * FC
      tmp1h = TAUh * FCh
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * FCh
      T_r1 = T_u1
     +         - (T_u1 - FCh)
      T_r2 = FCh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TAUh*FCl
     +      + TAUl*FCh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TAU * FC
!<<<  F = FINIT + tmp1
      T_p1 = FINITh + tmp1h
      T_p2 = T_p1 - FINITh
      Fh = T_p1
      Fl = (FINITh - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      Fl = Fl + FINITl + tmp1l
      T_s1 = Fh + Fl
      Fl = Fl - (T_s1 - Fh)
      Fh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  F = FINIT + tmp1
*
!<<<  tmp1 = abs(F)
      if (Fh .lt. 0.0d+0) then
        tmp1h = -Fh
        tmp1l = -Fl
      elseif (
     +  (Fh .eq. 0.0d+0).and.
     +  (Fl .lt. 0.0d+0)) then
        tmp1h = Fh
        tmp1l = -Fl
      else
        tmp1h = Fh
        tmp1l = Fl
      endif
!>>>  tmp1 = abs(F)
      IF(
!         ABS( F ).LE.ZERO
!       (tmp1 .le. ZERO)
     + ((tmp1h .lt. ZEROh) .or.
     +  ((tmp1h .eq. ZEROh) .and.
     +   (tmp1l .le. ZEROl)))
     $ ) GO TO 60
      IF(
!         F .LE. ZERO
!       (F .le. ZERO)
     + ((Fh .lt. ZEROh) .or.
     +  ((Fh .eq. ZEROh) .and.
     +   (Fl .le. ZEROl)))
     $    )THEN
!         LBD = TAU
      LBDh = TAUh
      LBDl = TAUl
      ELSE
!         UBD = TAU
      UBDh = TAUh
      UBDl = TAUl
      END IF
*
*        Iteration begins -- Use Gragg-Thornton-Warner cubic convergent
*                            scheme
*
*     It is not hard to see that
*
*           1) Iterations will go up monotonically
*              if FINIT < 0;
*
*           2) Iterations will go down monotonically
*              if FINIT > 0.
*
      ITER = NITER + 1
*
      DO 50 NITER = ITER, MAXIT
*
         IF( ORGATI ) THEN
!            TEMP1 = DSCALE( 2 ) - TAU
!<<<  TEMP1 = DSCALE(2) - TAU
      T_p1 = DSCALEh(2) - TAUh
      T_p2 = T_p1 - DSCALEh(2)
      TEMP1h = T_p1
      TEMP1l = (DSCALEh(2) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      TEMP1l = TEMP1l + DSCALEl(2) - TAUl
      T_s1 = TEMP1h + TEMP1l
      TEMP1l = TEMP1l - (T_s1 - TEMP1h)
      TEMP1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  TEMP1 = DSCALE(2) - TAU
!            TEMP2 = DSCALE( 3 ) - TAU
!<<<  TEMP2 = DSCALE(3) - TAU
      T_p1 = DSCALEh(3) - TAUh
      T_p2 = T_p1 - DSCALEh(3)
      TEMP2h = T_p1
      TEMP2l = (DSCALEh(3) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      TEMP2l = TEMP2l + DSCALEl(3) - TAUl
      T_s1 = TEMP2h + TEMP2l
      TEMP2l = TEMP2l - (T_s1 - TEMP2h)
      TEMP2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  TEMP2 = DSCALE(3) - TAU
         ELSE
!            TEMP1 = DSCALE( 1 ) - TAU
!<<<  TEMP1 = DSCALE(1) - TAU
      T_p1 = DSCALEh(1) - TAUh
      T_p2 = T_p1 - DSCALEh(1)
      TEMP1h = T_p1
      TEMP1l = (DSCALEh(1) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      TEMP1l = TEMP1l + DSCALEl(1) - TAUl
      T_s1 = TEMP1h + TEMP1l
      TEMP1l = TEMP1l - (T_s1 - TEMP1h)
      TEMP1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  TEMP1 = DSCALE(1) - TAU
!            TEMP2 = DSCALE( 2 ) - TAU
!<<<  TEMP2 = DSCALE(2) - TAU
      T_p1 = DSCALEh(2) - TAUh
      T_p2 = T_p1 - DSCALEh(2)
      TEMP2h = T_p1
      TEMP2l = (DSCALEh(2) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      TEMP2l = TEMP2l + DSCALEl(2) - TAUl
      T_s1 = TEMP2h + TEMP2l
      TEMP2l = TEMP2l - (T_s1 - TEMP2h)
      TEMP2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  TEMP2 = DSCALE(2) - TAU
         END IF
!         A = ( TEMP1+TEMP2 )*F - TEMP1*TEMP2*DF
!<<<  tmp1 = TEMP1 + TEMP2
      T_p1 = TEMP1h + TEMP2h
      T_p2 = T_p1 - TEMP1h
      tmp1h = T_p1
      tmp1l = (TEMP1h - (T_p1 - T_p2))
     +      + (TEMP2h - T_p2)
      tmp1l = tmp1l + TEMP1l + TEMP2l
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = TEMP1 + TEMP2
!<<<  tmp2 = tmp1 * F
      tmp2h = tmp1h * Fh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Fh
      T_r1 = T_u1
     +         - (T_u1 - Fh)
      T_r2 = Fh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*Fl
     +      + tmp1l*Fh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * F
!<<<  tmp3 = TEMP1 * TEMP2
      tmp3h = TEMP1h * TEMP2h
      T_u1 = 134217729.0 * TEMP1h
      T_p1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_p2 = TEMP1h - T_p1
      T_u1 = 134217729.0 * TEMP2h
      T_r1 = T_u1
     +         - (T_u1 - TEMP2h)
      T_r2 = TEMP2h - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (TEMP1h*TEMP2l
     +      + TEMP1l*TEMP2h)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = TEMP1 * TEMP2
!<<<  tmp4 = tmp3 * DF
      tmp4h = tmp3h * DFh
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * DFh
      T_r1 = T_u1
     +         - (T_u1 - DFh)
      T_r2 = DFh - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*DFl
     +      + tmp3l*DFh)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * DF
!<<<  A = tmp2 - tmp4
      T_p1 = tmp2h - tmp4h
      T_p2 = T_p1 - tmp2h
      Ah = T_p1
      Al = (tmp2h - (T_p1 - T_p2))
     +         - (tmp4h + T_p2)
      Al = Al + tmp2l - tmp4l
      T_s1 = Ah + Al
      Al = Al - (T_s1 - Ah)
      Ah = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  A = tmp2 - tmp4
!         B = TEMP1*TEMP2*F
!<<<  tmp1 = TEMP1 * TEMP2
      tmp1h = TEMP1h * TEMP2h
      T_u1 = 134217729.0 * TEMP1h
      T_p1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_p2 = TEMP1h - T_p1
      T_u1 = 134217729.0 * TEMP2h
      T_r1 = T_u1
     +         - (T_u1 - TEMP2h)
      T_r2 = TEMP2h - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TEMP1h*TEMP2l
     +      + TEMP1l*TEMP2h)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TEMP1 * TEMP2
!<<<  B = tmp1 * F
      Bh = tmp1h * Fh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * Fh
      T_r1 = T_u1
     +         - (T_u1 - Fh)
      T_r2 = Fh - T_r1
      Bl = ((T_p1*T_r1
     +        - Bh)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      Bl = Bl
     +      + (tmp1h*Fl
     +      + tmp1l*Fh)
      T_s1 = Bh + Bl
      Bl = Bl - (T_s1 - Bh)
      Bh = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  B = tmp1 * F
!         C = F - ( TEMP1+TEMP2 )*DF + TEMP1*TEMP2*DDF
!<<<  tmp1 = TEMP1 + TEMP2
      T_p1 = TEMP1h + TEMP2h
      T_p2 = T_p1 - TEMP1h
      tmp1h = T_p1
      tmp1l = (TEMP1h - (T_p1 - T_p2))
     +      + (TEMP2h - T_p2)
      tmp1l = tmp1l + TEMP1l + TEMP2l
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = TEMP1 + TEMP2
!<<<  tmp2 = tmp1 * DF
      tmp2h = tmp1h * DFh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * DFh
      T_r1 = T_u1
     +         - (T_u1 - DFh)
      T_r2 = DFh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*DFl
     +      + tmp1l*DFh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * DF
!<<<  tmp3 = TEMP1 * TEMP2
      tmp3h = TEMP1h * TEMP2h
      T_u1 = 134217729.0 * TEMP1h
      T_p1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_p2 = TEMP1h - T_p1
      T_u1 = 134217729.0 * TEMP2h
      T_r1 = T_u1
     +         - (T_u1 - TEMP2h)
      T_r2 = TEMP2h - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (TEMP1h*TEMP2l
     +      + TEMP1l*TEMP2h)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = TEMP1 * TEMP2
!<<<  tmp4 = tmp3 * DDF
      tmp4h = tmp3h * DDFh
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * DDFh
      T_r1 = T_u1
     +         - (T_u1 - DDFh)
      T_r2 = DDFh - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*DDFl
     +      + tmp3l*DDFh)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * DDF
!<<<  tmp5 = F - tmp2
      T_p1 = Fh - tmp2h
      T_p2 = T_p1 - Fh
      tmp5h = T_p1
      tmp5l = (Fh - (T_p1 - T_p2))
     +         - (tmp2h + T_p2)
      tmp5l = tmp5l + Fl - tmp2l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp5 = F - tmp2
!<<<  C = tmp5 + tmp4
      T_p1 = tmp5h + tmp4h
      T_p2 = T_p1 - tmp5h
      Ch = T_p1
      Cl = (tmp5h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      Cl = Cl + tmp5l + tmp4l
      T_s1 = Ch + Cl
      Cl = Cl - (T_s1 - Ch)
      Ch = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  C = tmp5 + tmp4
!         TEMP = MAX( ABS( A ), ABS( B ), ABS( C ) )
!<<<  tmp1 = abs(A)
      if (Ah .lt. 0.0d+0) then
        tmp1h = -Ah
        tmp1l = -Al
      elseif (
     +  (Ah .eq. 0.0d+0).and.
     +  (Al .lt. 0.0d+0)) then
        tmp1h = Ah
        tmp1l = -Al
      else
        tmp1h = Ah
        tmp1l = Al
      endif
!>>>  tmp1 = abs(A)
!<<<  tmp2 = abs(B)
      if (Bh .lt. 0.0d+0) then
        tmp2h = -Bh
        tmp2l = -Bl
      elseif (
     +  (Bh .eq. 0.0d+0).and.
     +  (Bl .lt. 0.0d+0)) then
        tmp2h = Bh
        tmp2l = -Bl
      else
        tmp2h = Bh
        tmp2l = Bl
      endif
!>>>  tmp2 = abs(B)
!<<<  tmp3 = abs(C)
      if (Ch .lt. 0.0d+0) then
        tmp3h = -Ch
        tmp3l = -Cl
      elseif (
     +  (Ch .eq. 0.0d+0).and.
     +  (Cl .lt. 0.0d+0)) then
        tmp3h = Ch
        tmp3l = -Cl
      else
        tmp3h = Ch
        tmp3l = Cl
      endif
!>>>  tmp3 = abs(C)
         IF(
!       (tmp1 .ge. tmp2)
     + ((tmp1h .gt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .ge. tmp2l)))
     &      ) THEN
              IF(
!       (tmp1 .ge. tmp3)
     + ((tmp1h .gt. tmp3h) .or.
     +  ((tmp1h .eq. tmp3h) .and.
     +   (tmp1l .ge. tmp3l)))
     &           ) THEN
      TEMPh = tmp1h
      TEMPl = tmp1l
              ELSE
      TEMPh = tmp3h
      TEMPl = tmp3l
              END IF
         ELSE
              IF(
!       (tmp1 .ge. tmp3)
     + ((tmp1h .gt. tmp3h) .or.
     +  ((tmp1h .eq. tmp3h) .and.
     +   (tmp1l .ge. tmp3l)))
     &           ) THEN
      TEMPh = tmp2h
      TEMPl = tmp2l
              ELSE IF(
!       (tmp2 .ge. tmp3)
     + ((tmp2h .gt. tmp3h) .or.
     +  ((tmp2h .eq. tmp3h) .and.
     +   (tmp2l .ge. tmp3l)))
     &          ) THEN
      TEMPh = tmp2h
      TEMPl = tmp2l
              ELSE
      TEMPh = tmp3h
      TEMPl = tmp3l
              END IF
          END IF
!         A = A / TEMP
!<<<  A = A / TEMP
      T_q1 = Ah / TEMPh
      T_ch = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TEMPl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Ah - T_ch
      T_r1 = T_s1 - Ah
      T_s2 = (Ah
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Al
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      Ah = T_q1 + T_q2
      Al = T_q2
     +      - (Ah - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  A = A / TEMP
!         B = B / TEMP
!<<<  B = B / TEMP
      T_q1 = Bh / TEMPh
      T_ch = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TEMPl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Bh - T_ch
      T_r1 = T_s1 - Bh
      T_s2 = (Bh
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Bl
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      Bh = T_q1 + T_q2
      Bl = T_q2
     +      - (Bh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  B = B / TEMP
!         C = C / TEMP
!<<<  C = C / TEMP
      T_q1 = Ch / TEMPh
      T_ch = TEMPh * T_q1
      T_u1 = 134217729.0 * TEMPh
      T_p1 = T_u1
     +         - (T_u1 - TEMPh)
      T_p2 = TEMPh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +         + T_p1*T_r2
     +         + T_p2*T_r1)
     +         + T_p2*T_r2
      T_cl = T_cl
     +         + (TEMPl * T_q1)
      T_s1 = T_ch + T_cl
      T_cl = T_cl
     +         - (T_s1 - T_ch)
      T_ch = T_s1
      T_s1 = Ch - T_ch
      T_r1 = T_s1 - Ch
      T_s2 = (Ch
     +         - (T_s1 - T_r1))
     +         - (T_ch + T_r1)
      T_s2 = T_s2 - T_cl
      T_s2 = T_s2 + Cl
      T_q2 = (T_s1 + T_s2)
     +         / TEMPh
      Ch = T_q1 + T_q2
      Cl = T_q2
     +      - (Ch - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  C = C / TEMP
         IF(
!            C.EQ.ZERO
!       (C .eq. ZERO)
     + ((Ch .eq. ZEROh) .and.
     +  (Cl .eq. ZEROl))
     $       ) THEN
!            ETA = B / A
!<<<  ETA = B / A
      T_q1 = Bh / Ah
      ETAh = Ah * T_q1
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (Al*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = Bh - ETAh
      T_r1 = T_s1 - Bh
      T_s2 = (Bh 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + Bl
      T_q2 = (T_s1 + T_s2)
     +         / Ah
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = B / A
         ELSE IF(
!             A.LE.ZERO
!       (A .le. ZERO)
     + ((Ah .lt. ZEROh) .or.
     +  ((Ah .eq. ZEROh) .and.
     +   (Al .le. ZEROl)))
     $         ) THEN
!            ETA = ( A-SQRT( ABS( A*A-FOUR*B*C ) ) ) / ( TWO*C )
!<<<  tmp1 = TWO * C
      tmp1h = TWOh * Ch
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Cl
     +      + TWOl*Ch)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * C
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp4 = tmp3 * C
      tmp4h = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Cl
     +      + tmp3l*Ch)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * C
!<<<  tmp5 = tmp2 - tmp4
      T_p1 = tmp2h - tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp4h + T_p2)
      tmp5l = tmp5l + tmp2l - tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp5 = tmp2 - tmp4
!<<<  tmp6 = abs(tmp5)
      if (tmp5h .lt. 0.0d+0) then
        tmp6h = -tmp5h
        tmp6l = -tmp5l
      elseif (
     +  (tmp5h .eq. 0.0d+0).and.
     +  (tmp5l .lt. 0.0d+0)) then
        tmp6h = tmp5h
        tmp6l = -tmp5l
      else
        tmp6h = tmp5h
        tmp6l = tmp5l
      endif
!>>>  tmp6 = abs(tmp5)
!<<<  tmp7 = sqrt(tmp6)
      if (tmp6h.eq.0.0D+0.and.tmp6l.eq.0.0D+0) then
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
      endif
      if (tmp6h.lt.0.0d+0.or.
     +   (tmp6h.eq.0.0d+0.and.tmp6l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp6h)
      T_ax = tmp6h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp6h,tmp6l,T_z1,T_z2)
          T_p1 = tmp6h - T_z1
          T_p2 = T_p1 - tmp6h
          T_t2l = (tmp6h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp6l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp7h = T_t1h
          tmp7l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp7 = sqrt(tmp6)
!<<<  tmp8 = A - tmp7
      T_p1 = Ah - tmp7h
      T_p2 = T_p1 - Ah
      tmp8h = T_p1
      tmp8l = (Ah - (T_p1 - T_p2))
     +         - (tmp7h + T_p2)
      tmp8l = tmp8l + Al - tmp7l
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp8 = A - tmp7
!<<<  ETA = tmp8 / tmp1
      T_q1 = tmp8h / tmp1h
      ETAh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (tmp1l*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp8h - ETAh
      T_r1 = T_s1 - tmp8h
      T_s2 = (tmp8h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp8l
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp8 / tmp1
         ELSE
!            ETA = TWO*B / ( A+SQRT( ABS( A*A-FOUR*B*C ) ) )
!<<<  tmp1 = TWO * B
      tmp1h = TWOh * Bh
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TWOh*Bl
     +      + TWOl*Bh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TWO * B
!<<<  tmp2 = A * A
      tmp2h = Ah * Ah
      T_u1 = 134217729.0 * Ah
      T_p1 = T_u1
     +         - (T_u1 - Ah)
      T_p2 = Ah - T_p1
      T_u1 = 134217729.0 * Ah
      T_r1 = T_u1
     +         - (T_u1 - Ah)
      T_r2 = Ah - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (Ah*Al
     +      + Al*Ah)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = A * A
!<<<  tmp3 = FOUR * B
      tmp3h = FOURh * Bh
      T_u1 = 134217729.0 * FOURh
      T_p1 = T_u1
     +         - (T_u1 - FOURh)
      T_p2 = FOURh - T_p1
      T_u1 = 134217729.0 * Bh
      T_r1 = T_u1
     +         - (T_u1 - Bh)
      T_r2 = Bh - T_r1
      tmp3l = ((T_p1*T_r1
     +        - tmp3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp3l = tmp3l
     +      + (FOURh*Bl
     +      + FOURl*Bh)
      T_s1 = tmp3h + tmp3l
      tmp3l = tmp3l - (T_s1 - tmp3h)
      tmp3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp3 = FOUR * B
!<<<  tmp4 = tmp3 * C
      tmp4h = tmp3h * Ch
      T_u1 = 134217729.0 * tmp3h
      T_p1 = T_u1
     +         - (T_u1 - tmp3h)
      T_p2 = tmp3h - T_p1
      T_u1 = 134217729.0 * Ch
      T_r1 = T_u1
     +         - (T_u1 - Ch)
      T_r2 = Ch - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp3h*Cl
     +      + tmp3l*Ch)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp3 * C
!<<<  tmp5 = tmp2 - tmp4
      T_p1 = tmp2h - tmp4h
      T_p2 = T_p1 - tmp2h
      tmp5h = T_p1
      tmp5l = (tmp2h - (T_p1 - T_p2))
     +         - (tmp4h + T_p2)
      tmp5l = tmp5l + tmp2l - tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp5 = tmp2 - tmp4
!<<<  tmp6 = abs(tmp5)
      if (tmp5h .lt. 0.0d+0) then
        tmp6h = -tmp5h
        tmp6l = -tmp5l
      elseif (
     +  (tmp5h .eq. 0.0d+0).and.
     +  (tmp5l .lt. 0.0d+0)) then
        tmp6h = tmp5h
        tmp6l = -tmp5l
      else
        tmp6h = tmp5h
        tmp6l = tmp5l
      endif
!>>>  tmp6 = abs(tmp5)
!<<<  tmp7 = sqrt(tmp6)
      if (tmp6h.eq.0.0D+0.and.tmp6l.eq.0.0D+0) then
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
      endif
      if (tmp6h.lt.0.0d+0.or.
     +   (tmp6h.eq.0.0d+0.and.tmp6l.lt.0.0d+0)) then
        write(6,'(a)')
     +     'SQRT CALLED WITH NEGATIVE ARGUMENT'
        tmp7h = 0.0d+0
        tmp7l = 0.0d+0
        return
      endif
      T_x = 1.0D+0/sqrt(tmp6h)
      T_ax = tmp6h * T_x
!         call dd_mul(T_y1,T_y2,T_x,
!                     0.0,0.5,0.0)
          T_y1 = T_x * 0.5d+0
          T_u1 = 134217729.0 * T_x
          T_p1 = T_u1
     +             - (T_u1 - T_x)
          T_p2 = T_x - T_p1
          T_u1 = 134217729.0 * 0.5d+0
          T_r1 = T_u1
     +             - (T_u1 - 0.5d+0)
          T_r2 = 0.5d+0 - T_r1
          T_y2 = ((T_p1*T_r1
     +             - T_y1)
     +             + T_p1*T_r2
     +             + T_p2 * T_r1)
     +             + T_p2 * T_r2
          T_s1 = T_y1 + T_y2
          T_y2 = T_y2
     +             - (T_s1 - T_y1)
          T_y1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_mul(T_z1,T_z2,T_ax,
!                     0.0,T_ax,0.0)
          T_z1 = T_ax * T_ax
          T_u1 = 134217729.0 * T_ax
          T_p1 = T_u1
     +             - (T_u1 - T_ax)
          T_p2 = T_ax - T_p1
          T_u1 = 134217729.0 * T_ax
          T_r1 = T_u1
     +             - (T_u1 - T_ax)
          T_r2 = T_ax - T_r1
          T_z2 = ((T_p1 * T_r1
     +             - T_z1)
     +             + T_p1 * T_r2
     +             + T_p2 * T_r1)
     +             + T_p2*T_r2
          T_s1 = T_z1 + T_z2
          T_z2 = T_z2
     +             - (T_s1 - T_z1)
          T_z1 = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_sub(T_t2h,T_t2l,
!                           tmp6h,tmp6l,T_z1,T_z2)
          T_p1 = tmp6h - T_z1
          T_p2 = T_p1 - tmp6h
          T_t2l = (tmp6h
     +             -(T_p1 - T_p2))
     +             -(T_z1 + T_p2)
          T_t2h = T_p1
          T_t2l = T_t2l
     +             + tmp6l - T_z2
          T_s1 = T_t2h + T_t2l
          T_t2l = T_t2l
     +             - (T_s1 - T_t2h)
          T_t2h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!         call dd_mul(T_t1h,T_t1l,
!                     T_t2h,T_t2l,
!                     T_y1,T_y2)
          T_t1h = T_t2h * T_y1
          T_u1 = 134217729.0 * T_t2h
          T_p1 = T_u1
     +             - (T_u1 - T_t2h)
          T_p2 = T_t2h - T_p1
          T_u1 = 134217729.0 * T_y1
          T_r1 = T_u1
     +             - (T_u1 - T_y1)
          T_r2 = T_y1 - T_r1
          T_t1l = ((T_p1*T_r1
     +             - T_t1h)
     +             + T_p1*T_r2
     +             + T_p2*T_r1)
     +             + T_p2*T_r2
          T_t1l = T_t1l
     +             + (T_t2h * T_y2
     +             +  T_t2l * T_y1)
          T_s1 = T_t1h + T_t1l
          T_t1l = T_t1l - (T_s1
     +              - T_t1h)
          T_t1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!         call dd_selfadd(T_t1h,T_t1l,
!                     T_ax,0.0)
          T_p1 = T_t1h + T_ax
          T_p2 = T_p1 - T_t1h
          T_q2 = (T_t1h
     +             - (T_p1 - T_p2))
     +             + (T_ax - T_p2)
          T_q1 = T_p1
          T_q2 = T_q2 + T_t1l
          T_t1h = T_q1 + T_q2
          T_t1l = T_q2
     +              - (T_t1h - T_q1)
          tmp7h = T_t1h
          tmp7l = T_t1l
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  tmp7 = sqrt(tmp6)
!<<<  tmp8 = A + tmp7
      T_p1 = Ah + tmp7h
      T_p2 = T_p1 - Ah
      tmp8h = T_p1
      tmp8l = (Ah - (T_p1 - T_p2))
     +      + (tmp7h - T_p2)
      tmp8l = tmp8l + Al + tmp7l
      T_s1 = tmp8h + tmp8l
      tmp8l = tmp8l - (T_s1 - tmp8h)
      tmp8h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp8 = A + tmp7
!<<<  ETA = tmp1 / tmp8
      T_q1 = tmp1h / tmp8h
      ETAh = tmp8h * T_q1
      T_u1 = 134217729.0 * tmp8h
      T_p1 = T_u1
     +         - (T_u1 - tmp8h)
      T_p2 = tmp8h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      ETAl = ((T_p1*T_r1
     +       - ETAh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      ETAl = ETAl + (tmp8l*T_q1)
      T_s1 = ETAh + ETAl
      ETAl = ETAl - (T_s1 - ETAh)
      ETAh = T_s1
      T_s1 = tmp1h - ETAh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (ETAh + T_r1)
      T_s2 = T_s2 - ETAl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / tmp8h
      ETAh = T_q1 + T_q2
      ETAl = T_q2
     +      - (ETAh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  ETA = tmp1 / tmp8
         END IF
!<<<  tmp1 = F * ETA
      tmp1h = Fh * ETAh
      T_u1 = 134217729.0 * Fh
      T_p1 = T_u1
     +         - (T_u1 - Fh)
      T_p2 = Fh - T_p1
      T_u1 = 134217729.0 * ETAh
      T_r1 = T_u1
     +         - (T_u1 - ETAh)
      T_r2 = ETAh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (Fh*ETAl
     +      + Fl*ETAh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = F * ETA
         IF(
!            F*ETA.GE.ZERO
!       (tmp1 .ge. ZERO)
     + ((tmp1h .gt. ZEROh) .or.
     +  ((tmp1h .eq. ZEROh) .and.
     +   (tmp1l .ge. ZEROl)))
     $       ) THEN
!            ETA = -F / DF
!<<<  tmp1 = F / DF
      T_q1 = Fh / DFh
      tmp1h = DFh * T_q1
      T_u1 = 134217729.0 * DFh
      T_p1 = T_u1
     +         - (T_u1 - DFh)
      T_p2 = DFh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      tmp1l = ((T_p1*T_r1
     +       - tmp1h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      tmp1l = tmp1l + (DFl*T_q1)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
      T_s1 = Fh - tmp1h
      T_r1 = T_s1 - Fh
      T_s2 = (Fh 
     +         - (T_s1 - T_r1))
     +         - (tmp1h + T_r1)
      T_s2 = T_s2 - tmp1l
      T_s2 = T_s2 + Fl
      T_q2 = (T_s1 + T_s2)
     +         / DFh
      tmp1h = T_q1 + T_q2
      tmp1l = T_q2
     +      - (tmp1h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  tmp1 = F / DF
!<<<  ETA = -tmp1
      ETAh = -tmp1h
      ETAl = -tmp1l
!>>>  ETA = -tmp1
         END IF
*
!         TAU = TAU + ETA
!<<<  TAU = TAU + ETA
      T_p1 = TAUh + ETAh
      T_p2 = T_p1 - TAUh
      T_q1 = T_p1
      T_q2 = (TAUh - (T_p1 - T_p2))
     +         + (ETAh - T_p2)
      T_q2 = T_q2 + TAUl + ETAl
      TAUh = T_q1 + T_q2
      TAUl = T_q2 - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  TAU = TAU + ETA
         IF(
!            TAU .LT. LBD 
!       (TAU .lt. LBD)
     + ((TAUh .lt. LBDh) .or.
     +  ((TAUh .eq. LBDh) .and.
     +   (TAUl .lt. LBDl)))
     $      .OR.
!            TAU .GT. UBD
!       (TAU .gt. UBD)
     + ((TAUh .gt. UBDh) .or.
     +  ((TAUh .eq. UBDh) .and.
     +   (TAUl .gt. UBDl)))
     $       )THEN
!            TAU = ( LBD + UBD )/TWO 
!<<<  tmp1 = LBD + UBD
      T_p1 = LBDh + UBDh
      T_p2 = T_p1 - LBDh
      tmp1h = T_p1
      tmp1l = (LBDh - (T_p1 - T_p2))
     +      + (UBDh - T_p2)
      tmp1l = tmp1l + LBDl + UBDl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp1 = LBD + UBD
!<<<  TAU = tmp1 / TWO
      T_q1 = tmp1h / TWOh
      TAUh = TWOh * T_q1
      T_u1 = 134217729.0 * TWOh
      T_p1 = T_u1
     +         - (T_u1 - TWOh)
      T_p2 = TWOh - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TAUl = ((T_p1*T_r1
     +       - TAUh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TAUl = TAUl + (TWOl*T_q1)
      T_s1 = TAUh + TAUl
      TAUl = TAUl - (T_s1 - TAUh)
      TAUh = T_s1
      T_s1 = tmp1h - TAUh
      T_r1 = T_s1 - tmp1h
      T_s2 = (tmp1h 
     +         - (T_s1 - T_r1))
     +         - (TAUh + T_r1)
      T_s2 = T_s2 - TAUl
      T_s2 = T_s2 + tmp1l
      T_q2 = (T_s1 + T_s2)
     +         / TWOh
      TAUh = T_q1 + T_q2
      TAUl = T_q2
     +      - (TAUh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TAU = tmp1 / TWO
          END IF
*
!         FC = ZERO
      FCh = ZEROh
      FCl = ZEROl
!         ERRETM = ZERO
      ERRETMh = ZEROh
      ERRETMl = ZEROl
!         DF = ZERO
      DFh = ZEROh
      DFl = ZEROl
!         DDF = ZERO
      DDFh = ZEROh
      DDFl = ZEROl
         DO 40 I = 1, 3
!            TEMP = ONE / ( DSCALE( I )-TAU )
!<<<  tmp1 = DSCALE(I) - TAU
      T_p1 = DSCALEh(I) - TAUh
      T_p2 = T_p1 - DSCALEh(I)
      tmp1h = T_p1
      tmp1l = (DSCALEh(I) - (T_p1 - T_p2))
     +         - (TAUh + T_p2)
      tmp1l = tmp1l + DSCALEl(I) - TAUl
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fsub.h"
#endif
!>>>  tmp1 = DSCALE(I) - TAU
!<<<  TEMP = ONE / tmp1
      T_q1 = ONEh / tmp1h
      TEMPh = tmp1h * T_q1
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMPl = ((T_p1*T_r1
     +       - TEMPh)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMPl = TEMPl + (tmp1l*T_q1)
      T_s1 = TEMPh + TEMPl
      TEMPl = TEMPl - (T_s1 - TEMPh)
      TEMPh = T_s1
      T_s1 = ONEh - TEMPh
      T_r1 = T_s1 - ONEh
      T_s2 = (ONEh 
     +         - (T_s1 - T_r1))
     +         - (TEMPh + T_r1)
      T_s2 = T_s2 - TEMPl
      T_s2 = T_s2 + ONEl
      T_q2 = (T_s1 + T_s2)
     +         / tmp1h
      TEMPh = T_q1 + T_q2
      TEMPl = T_q2
     +      - (TEMPh - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP = ONE / tmp1
!            TEMP1 = ZSCALE( I )*TEMP
!<<<  TEMP1 = ZSCALE(I) * TEMP
      TEMP1h = ZSCALEh(I) * TEMPh
      T_u1 = 134217729.0 * ZSCALEh(I)
      T_p1 = T_u1
     +         - (T_u1 - ZSCALEh(I))
      T_p2 = ZSCALEh(I) - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      TEMP1l = ((T_p1*T_r1
     +        - TEMP1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      TEMP1l = TEMP1l
     +      + (ZSCALEh(I)*TEMPl
     +      + ZSCALEl(I)*TEMPh)
      T_s1 = TEMP1h + TEMP1l
      TEMP1l = TEMP1l - (T_s1 - TEMP1h)
      TEMP1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  TEMP1 = ZSCALE(I) * TEMP
!            TEMP2 = TEMP1*TEMP
!<<<  TEMP2 = TEMP1 * TEMP
      TEMP2h = TEMP1h * TEMPh
      T_u1 = 134217729.0 * TEMP1h
      T_p1 = T_u1
     +         - (T_u1 - TEMP1h)
      T_p2 = TEMP1h - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      TEMP2l = ((T_p1*T_r1
     +        - TEMP2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      TEMP2l = TEMP2l
     +      + (TEMP1h*TEMPl
     +      + TEMP1l*TEMPh)
      T_s1 = TEMP2h + TEMP2l
      TEMP2l = TEMP2l - (T_s1 - TEMP2h)
      TEMP2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  TEMP2 = TEMP1 * TEMP
!            TEMP3 = TEMP2*TEMP
!<<<  TEMP3 = TEMP2 * TEMP
      TEMP3h = TEMP2h * TEMPh
      T_u1 = 134217729.0 * TEMP2h
      T_p1 = T_u1
     +         - (T_u1 - TEMP2h)
      T_p2 = TEMP2h - T_p1
      T_u1 = 134217729.0 * TEMPh
      T_r1 = T_u1
     +         - (T_u1 - TEMPh)
      T_r2 = TEMPh - T_r1
      TEMP3l = ((T_p1*T_r1
     +        - TEMP3h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      TEMP3l = TEMP3l
     +      + (TEMP2h*TEMPl
     +      + TEMP2l*TEMPh)
      T_s1 = TEMP3h + TEMP3l
      TEMP3l = TEMP3l - (T_s1 - TEMP3h)
      TEMP3h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  TEMP3 = TEMP2 * TEMP
!            TEMP4 = TEMP1 / DSCALE( I )
!<<<  TEMP4 = TEMP1 / DSCALE(I)
      T_q1 = TEMP1h / DSCALEh(I)
      TEMP4h = DSCALEh(I) * T_q1
      T_u1 = 134217729.0 * DSCALEh(I)
      T_p1 = T_u1
     +         - (T_u1 - DSCALEh(I))
      T_p2 = DSCALEh(I) - T_p1
      T_u1 = 134217729.0 * T_q1
      T_r1 = T_u1
     +         - (T_u1 - T_q1)
      T_r2 = T_q1 - T_r1
      TEMP4l = ((T_p1*T_r1
     +       - TEMP4h)
     +       + T_p1*T_r2
     +       + T_p2*T_r1)
     +       + T_p2*T_r2
      TEMP4l = TEMP4l + (DSCALEl(I)*T_q1)
      T_s1 = TEMP4h + TEMP4l
      TEMP4l = TEMP4l - (T_s1 - TEMP4h)
      TEMP4h = T_s1
      T_s1 = TEMP1h - TEMP4h
      T_r1 = T_s1 - TEMP1h
      T_s2 = (TEMP1h 
     +         - (T_s1 - T_r1))
     +         - (TEMP4h + T_r1)
      T_s2 = T_s2 - TEMP4l
      T_s2 = T_s2 + TEMP1l
      T_q2 = (T_s1 + T_s2)
     +         / DSCALEh(I)
      TEMP4h = T_q1 + T_q2
      TEMP4l = T_q2
     +      - (TEMP4h - T_q1)
#ifdef FPCOUNTER
#include "fdiv.h"
#endif
!>>>  TEMP4 = TEMP1 / DSCALE(I)
!            FC = FC + TEMP4
!<<<  FC = FC + TEMP4
      T_p1 = FCh + TEMP4h
      T_p2 = T_p1 - FCh
      T_q1 = T_p1
      T_q2 = (FCh - (T_p1 - T_p2))
     +         + (TEMP4h - T_p2)
      T_q2 = T_q2 + FCl + TEMP4l
      FCh = T_q1 + T_q2
      FCl = T_q2 - (FCh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  FC = FC + TEMP4
!            ERRETM = ERRETM + ABS( TEMP4 )
!<<<  tmp1 = abs(TEMP4)
      if (TEMP4h .lt. 0.0d+0) then
        tmp1h = -TEMP4h
        tmp1l = -TEMP4l
      elseif (
     +  (TEMP4h .eq. 0.0d+0).and.
     +  (TEMP4l .lt. 0.0d+0)) then
        tmp1h = TEMP4h
        tmp1l = -TEMP4l
      else
        tmp1h = TEMP4h
        tmp1l = TEMP4l
      endif
!>>>  tmp1 = abs(TEMP4)
!<<<  ERRETM = ERRETM + tmp1
      T_p1 = ERRETMh + tmp1h
      T_p2 = T_p1 - ERRETMh
      T_q1 = T_p1
      T_q2 = (ERRETMh - (T_p1 - T_p2))
     +         + (tmp1h - T_p2)
      T_q2 = T_q2 + ERRETMl + tmp1l
      ERRETMh = T_q1 + T_q2
      ERRETMl = T_q2 - (ERRETMh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  ERRETM = ERRETM + tmp1
!            DF = DF + TEMP2
!<<<  DF = DF + TEMP2
      T_p1 = DFh + TEMP2h
      T_p2 = T_p1 - DFh
      T_q1 = T_p1
      T_q2 = (DFh - (T_p1 - T_p2))
     +         + (TEMP2h - T_p2)
      T_q2 = T_q2 + DFl + TEMP2l
      DFh = T_q1 + T_q2
      DFl = T_q2 - (DFh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DF = DF + TEMP2
!            DDF = DDF + TEMP3
!<<<  DDF = DDF + TEMP3
      T_p1 = DDFh + TEMP3h
      T_p2 = T_p1 - DDFh
      T_q1 = T_p1
      T_q2 = (DDFh - (T_p1 - T_p2))
     +         + (TEMP3h - T_p2)
      T_q2 = T_q2 + DDFl + TEMP3l
      DDFh = T_q1 + T_q2
      DDFl = T_q2 - (DDFh - T_q1)
#ifdef FPCOUNTER
#include "fselfadd.h"
#endif
!>>>  DDF = DDF + TEMP3
   40    CONTINUE
!         F = FINIT + TAU*FC
!<<<  tmp1 = TAU * FC
      tmp1h = TAUh * FCh
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * FCh
      T_r1 = T_u1
     +         - (T_u1 - FCh)
      T_r2 = FCh - T_r1
      tmp1l = ((T_p1*T_r1
     +        - tmp1h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp1l = tmp1l
     +      + (TAUh*FCl
     +      + TAUl*FCh)
      T_s1 = tmp1h + tmp1l
      tmp1l = tmp1l - (T_s1 - tmp1h)
      tmp1h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp1 = TAU * FC
!<<<  F = FINIT + tmp1
      T_p1 = FINITh + tmp1h
      T_p2 = T_p1 - FINITh
      Fh = T_p1
      Fl = (FINITh - (T_p1 - T_p2))
     +      + (tmp1h - T_p2)
      Fl = Fl + FINITl + tmp1l
      T_s1 = Fh + Fl
      Fl = Fl - (T_s1 - Fh)
      Fh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  F = FINIT + tmp1
!         ERRETM = EIGHT*( ABS( FINIT )+ABS( TAU )*ERRETM ) +
!     $            ABS( TAU )*DF
!<<<  tmp1 = abs(TAU)
      if (TAUh .lt. 0.0d+0) then
        tmp1h = -TAUh
        tmp1l = -TAUl
      elseif (
     +  (TAUh .eq. 0.0d+0).and.
     +  (TAUl .lt. 0.0d+0)) then
        tmp1h = TAUh
        tmp1l = -TAUl
      else
        tmp1h = TAUh
        tmp1l = TAUl
      endif
!>>>  tmp1 = abs(TAU)
!<<<  tmp2 = tmp1 * DF
      tmp2h = tmp1h * DFh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * DFh
      T_r1 = T_u1
     +         - (T_u1 - DFh)
      T_r2 = DFh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (tmp1h*DFl
     +      + tmp1l*DFh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = tmp1 * DF
!<<<  tmp3 = abs(FINIT)
      if (FINITh .lt. 0.0d+0) then
        tmp3h = -FINITh
        tmp3l = -FINITl
      elseif (
     +  (FINITh .eq. 0.0d+0).and.
     +  (FINITl .lt. 0.0d+0)) then
        tmp3h = FINITh
        tmp3l = -FINITl
      else
        tmp3h = FINITh
        tmp3l = FINITl
      endif
!>>>  tmp3 = abs(FINIT)
!<<<  tmp4 = tmp1 * ERRETM
      tmp4h = tmp1h * ERRETMh
      T_u1 = 134217729.0 * tmp1h
      T_p1 = T_u1
     +         - (T_u1 - tmp1h)
      T_p2 = tmp1h - T_p1
      T_u1 = 134217729.0 * ERRETMh
      T_r1 = T_u1
     +         - (T_u1 - ERRETMh)
      T_r2 = ERRETMh - T_r1
      tmp4l = ((T_p1*T_r1
     +        - tmp4h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp4l = tmp4l
     +      + (tmp1h*ERRETMl
     +      + tmp1l*ERRETMh)
      T_s1 = tmp4h + tmp4l
      tmp4l = tmp4l - (T_s1 - tmp4h)
      tmp4h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp4 = tmp1 * ERRETM
!<<<  tmp5 = tmp3 + tmp4
      T_p1 = tmp3h + tmp4h
      T_p2 = T_p1 - tmp3h
      tmp5h = T_p1
      tmp5l = (tmp3h - (T_p1 - T_p2))
     +      + (tmp4h - T_p2)
      tmp5l = tmp5l + tmp3l + tmp4l
      T_s1 = tmp5h + tmp5l
      tmp5l = tmp5l - (T_s1 - tmp5h)
      tmp5h = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  tmp5 = tmp3 + tmp4
!<<<  tmp6 = EIGHT * tmp5
      tmp6h = EIGHTh * tmp5h
      T_u1 = 134217729.0 * EIGHTh
      T_p1 = T_u1
     +         - (T_u1 - EIGHTh)
      T_p2 = EIGHTh - T_p1
      T_u1 = 134217729.0 * tmp5h
      T_r1 = T_u1
     +         - (T_u1 - tmp5h)
      T_r2 = tmp5h - T_r1
      tmp6l = ((T_p1*T_r1
     +        - tmp6h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp6l = tmp6l
     +      + (EIGHTh*tmp5l
     +      + EIGHTl*tmp5h)
      T_s1 = tmp6h + tmp6l
      tmp6l = tmp6l - (T_s1 - tmp6h)
      tmp6h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp6 = EIGHT * tmp5
!<<<  ERRETM = tmp6 + tmp2
      T_p1 = tmp6h + tmp2h
      T_p2 = T_p1 - tmp6h
      ERRETMh = T_p1
      ERRETMl = (tmp6h - (T_p1 - T_p2))
     +      + (tmp2h - T_p2)
      ERRETMl = ERRETMl + tmp6l + tmp2l
      T_s1 = ERRETMh + ERRETMl
      ERRETMl = ERRETMl - (T_s1 - ERRETMh)
      ERRETMh = T_s1
#ifdef FPCOUNTER
#include "fadd.h"
#endif
!>>>  ERRETM = tmp6 + tmp2

!<<<  tmp1 = abs(F)
      if (Fh .lt. 0.0d+0) then
        tmp1h = -Fh
        tmp1l = -Fl
      elseif (
     +  (Fh .eq. 0.0d+0).and.
     +  (Fl .lt. 0.0d+0)) then
        tmp1h = Fh
        tmp1l = -Fl
      else
        tmp1h = Fh
        tmp1l = Fl
      endif
!>>>  tmp1 = abs(F)
!<<<  tmp2 = EPS * ERRETM
      tmp2h = EPSh * ERRETMh
      T_u1 = 134217729.0 * EPSh
      T_p1 = T_u1
     +         - (T_u1 - EPSh)
      T_p2 = EPSh - T_p1
      T_u1 = 134217729.0 * ERRETMh
      T_r1 = T_u1
     +         - (T_u1 - ERRETMh)
      T_r2 = ERRETMh - T_r1
      tmp2l = ((T_p1*T_r1
     +        - tmp2h)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      tmp2l = tmp2l
     +      + (EPSh*ERRETMl
     +      + EPSl*ERRETMh)
      T_s1 = tmp2h + tmp2l
      tmp2l = tmp2l - (T_s1 - tmp2h)
      tmp2h = T_s1
#ifdef FPCOUNTER
#include "fmul.h"
#endif
!>>>  tmp2 = EPS * ERRETM
         IF(
!            ABS( F ).LE.EPS*ERRETM
!       (tmp1 .le. tmp2)
     + ((tmp1h .lt. tmp2h) .or.
     +  ((tmp1h .eq. tmp2h) .and.
     +   (tmp1l .le. tmp2l)))
     $      ) GO TO 60
         IF(
!             F .LE. ZERO
!       (F .le. ZERO)
     + ((Fh .lt. ZEROh) .or.
     +  ((Fh .eq. ZEROh) .and.
     +   (Fl .le. ZEROl)))
     $       )THEN
!            LBD = TAU
      LBDh = TAUh
      LBDl = TAUl
         ELSE
!            UBD = TAU
      UBDh = TAUh
      UBDl = TAUl
         END IF
   50 CONTINUE
      INFO = 1
   60 CONTINUE
*
*     Undo scaling
*
      IF( SCALE )THEN
!     $   TAU = TAU*SCLINV
!<<<  TAU = TAU * SCLINV
      T_ch = TAUh * SCLINVh
      T_u1 = 134217729.0 * TAUh
      T_p1 = T_u1
     +         - (T_u1 - TAUh)
      T_p2 = TAUh - T_p1
      T_u1 = 134217729.0 * SCLINVh
      T_r1 = T_u1
     +         - (T_u1 - SCLINVh)
      T_r2 = SCLINVh - T_r1
      T_cl = ((T_p1*T_r1
     +         - T_ch)
     +        + T_p1 * T_r2
     +        + T_p2 * T_r1)
     +        + T_p2 * T_r2
      T_cl = T_cl
     +         + (TAUh*SCLINVl
     +         +  TAUl*SCLINVh)
      T_s1 = T_ch + T_cl
      TAUl = T_cl - (T_s1 - T_ch)
      TAUh = T_s1
#ifdef FPCOUNTER
#include "fselfmul.h"
#endif
!>>>  TAU = TAU * SCLINV
      END IF
      RETURN
*
*     End of DLAED6
*
      END
